@@ -71,43 +71,11 @@ API int ncdirect_set_fg_rgb(struct ncdirect* nc, unsigned rgb)
 API int ncdirect_set_bg_rgb(struct ncdirect* nc, unsigned rgb)
   __attribute__ ((nonnull (1)));
 
-static inline int ncdirect_fg_rgb(struct ncdirect* nc, unsigned rgb)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_fg_rgb(struct ncdirect* nc, unsigned rgb){
-  return ncdirect_set_fg_rgb(nc, rgb);
-}
-
-static inline int ncdirect_bg_rgb(struct ncdirect* nc, unsigned rgb)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_bg_rgb(struct ncdirect* nc, unsigned rgb){
-  return ncdirect_set_bg_rgb(nc, rgb);
-}
-
 API int ncdirect_set_fg_palindex(struct ncdirect* nc, int pidx)
   __attribute__ ((nonnull (1)));
 API int ncdirect_set_bg_palindex(struct ncdirect* nc, int pidx)
   __attribute__ ((nonnull (1)));
 
-static inline int ncdirect_fg_palindex(struct ncdirect* nc, int pidx)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_fg_palindex(struct ncdirect* nc, int pidx){
-  return ncdirect_set_fg_palindex(nc, pidx);
-}
-
-static inline int ncdirect_bg_palindex(struct ncdirect* nc, int pidx)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_bg_palindex(struct ncdirect* nc, int pidx){
-  return ncdirect_set_bg_palindex(nc, pidx);
-}
-
 // Returns the number of simultaneous colors claimed to be supported, or 1 if
 // there is no color support. Note that several terminal emulators advertise
 // more colors than they actually support, downsampling internally.
@@ -116,7 +84,7 @@ API unsigned ncdirect_palette_size(const struct ncdirect* nc)
 
 // Output the string |utf8| according to the channels |channels|. Note that
 // ncdirect_putstr() does not explicitly flush output buffers, so it will not
-// necessarily be immediately visible.
+// necessarily be immediately visible. Returns EOF on error.
 API int ncdirect_putstr(struct ncdirect* nc, uint64_t channels, const char* utf8)
   __attribute__ ((nonnull (1, 3)));
 
@@ -154,54 +122,21 @@ ncdirect_set_fg_rgb8(struct ncdirect* nc, unsigned r, unsigned g, unsigned b){
   return ncdirect_set_fg_rgb(nc, (r << 16u) + (g << 8u) + b);
 }
 
-static inline int
-ncdirect_fg_rgb8(struct ncdirect* nc, unsigned r, unsigned g, unsigned b)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_fg_rgb8(struct ncdirect* nc, unsigned r, unsigned g, unsigned b){
-  return ncdirect_set_fg_rgb8(nc, r, g, b);
-}
-
-static inline int
-ncdirect_bg_rgb8(struct ncdirect* nc, unsigned r, unsigned g, unsigned b)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_bg_rgb8(struct ncdirect* nc, unsigned r, unsigned g, unsigned b){
-  return ncdirect_set_bg_rgb8(nc, r, g, b);
-}
-
 API int ncdirect_set_fg_default(struct ncdirect* nc)
   __attribute__ ((nonnull (1)));
 API int ncdirect_set_bg_default(struct ncdirect* nc)
   __attribute__ ((nonnull (1)));
 
-static inline int ncdirect_fg_default(struct ncdirect* nc)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_fg_default(struct ncdirect* nc){
-  return ncdirect_set_fg_default(nc);
-}
-
-static inline int ncdirect_bg_default(struct ncdirect* nc)
-  __attribute__ ((deprecated));
-
-static inline int
-ncdirect_bg_default(struct ncdirect* nc){
-  return ncdirect_set_bg_default(nc);
-}
-
 // Get the current number of columns/rows.
-API int ncdirect_dim_x(struct ncdirect* nc) __attribute__ ((nonnull (1)));
-API int ncdirect_dim_y(struct ncdirect* nc) __attribute__ ((nonnull (1)));
+API unsigned ncdirect_dim_x(struct ncdirect* nc) __attribute__ ((nonnull (1)));
+API unsigned ncdirect_dim_y(struct ncdirect* nc) __attribute__ ((nonnull (1)));
 
 // Returns a 16-bit bitmask of supported curses-style attributes
 // (NCSTYLE_UNDERLINE, NCSTYLE_BOLD, etc.) The attribute is only
 // indicated as supported if the terminal can support it together with color.
 // For more information, see the "ncv" capability in terminfo(5).
-API unsigned ncdirect_supported_styles(const struct ncdirect* nc);
+API uint16_t ncdirect_supported_styles(const struct ncdirect* nc)
+  __attribute__ ((nonnull (1)));
 
 // ncplane_styles_*() analogues
 API int ncdirect_set_styles(struct ncdirect* n, unsigned stylebits)
@@ -210,17 +145,9 @@ API int ncdirect_on_styles(struct ncdirect* n, unsigned stylebits)
   __attribute__ ((nonnull (1)));
 API int ncdirect_off_styles(struct ncdirect* n, unsigned stylebits)
   __attribute__ ((nonnull (1)));
-API unsigned ncdirect_styles(const struct ncdirect* n)
+API uint16_t ncdirect_styles(const struct ncdirect* n)
   __attribute__ ((nonnull (1)));
 
-// Deprecated forms of above.
-API int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits)
-  __attribute__ ((deprecated));
-API int ncdirect_styles_on(struct ncdirect* n, unsigned stylebits)
-  __attribute__ ((deprecated));
-API int ncdirect_styles_off(struct ncdirect* n, unsigned stylebits)
-  __attribute__ ((deprecated));
-
 // Move the cursor in direct mode. -1 to retain current location on that axis.
 API int ncdirect_cursor_move_yx(struct ncdirect* n, int y, int x)
   __attribute__ ((nonnull (1)));
@@ -240,7 +167,7 @@ API int ncdirect_cursor_down(struct ncdirect* nc, int num)
 // Get the cursor position, when supported. This requires writing to the
 // terminal, and then reading from it. If the terminal doesn't reply, or
 // doesn't reply in a way we understand, the results might be deleterious.
-API int ncdirect_cursor_yx(struct ncdirect* n, int* y, int* x)
+API int ncdirect_cursor_yx(struct ncdirect* n, unsigned* y, unsigned* x)
   __attribute__ ((nonnull (1)));
 
 // Push or pop the cursor location to the terminal's stack. The depth of this
@@ -263,13 +190,13 @@ API const nccapabilities* ncdirect_capabilities(const struct ncdirect* n)
 // horizontal line, |len| cannot exceed the screen width minus the cursor's
 // offset. For a vertical line, it may be as long as you'd like; the screen
 // will scroll as necessary. All lines start at the current cursor position.
-API int ncdirect_hline_interp(struct ncdirect* n, const char* egc, int len,
-                              uint64_t h1, uint64_t h2)
-  __attribute__ ((nonnull (1)));
+API int ncdirect_hline_interp(struct ncdirect* n, const char* egc,
+                              unsigned len, uint64_t h1, uint64_t h2)
+  __attribute__ ((nonnull (1, 2)));
 
-API int ncdirect_vline_interp(struct ncdirect* n, const char* egc, int len,
-                              uint64_t h1, uint64_t h2)
-  __attribute__ ((nonnull (1)));
+API int ncdirect_vline_interp(struct ncdirect* n, const char* egc,
+                              unsigned len, uint64_t h1, uint64_t h2)
+  __attribute__ ((nonnull (1, 2)));
 
 // Draw a box with its upper-left corner at the current cursor position, having
 // dimensions |ylen|x|xlen|. See ncplane_box() for more information. The
@@ -277,53 +204,53 @@ API int ncdirect_vline_interp(struct ncdirect* n, const char* egc, int len,
 // array of 6 wide characters: UL, UR, LL, LR, HL, VL.
 API int ncdirect_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                      uint64_t ll, uint64_t lr, const wchar_t* wchars,
-                     int ylen, int xlen, unsigned ctlword)
-  __attribute__ ((nonnull (1)));
+                     unsigned ylen, unsigned xlen, unsigned ctlword)
+  __attribute__ ((nonnull (1, 6)));
 
 __attribute__ ((nonnull (1))) static inline int
 ncdirect_light_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                    uint64_t ll, uint64_t lr,
-                   int ylen, int xlen, unsigned ctlword){
+                   unsigned ylen, unsigned xlen, unsigned ctlword){
   return ncdirect_box(n, ul, ur, ll, lr, NCBOXLIGHTW, ylen, xlen, ctlword);
 }
 
 __attribute__ ((nonnull (1))) static inline int
 ncdirect_heavy_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                    uint64_t ll, uint64_t lr,
-                   int ylen, int xlen, unsigned ctlword){
+                   unsigned ylen, unsigned xlen, unsigned ctlword){
   return ncdirect_box(n, ul, ur, ll, lr, NCBOXHEAVYW, ylen, xlen, ctlword);
 }
 
 __attribute__ ((nonnull (1))) static inline int
 ncdirect_ascii_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                    uint64_t ll, uint64_t lr,
-                   int ylen, int xlen, unsigned ctlword){
+                   unsigned ylen, unsigned xlen, unsigned ctlword){
   return ncdirect_box(n, ul, ur, ll, lr, NCBOXASCIIW, ylen, xlen, ctlword);
 }
 
 // ncdirect_box() with the rounded box-drawing characters
 API int ncdirect_rounded_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                              uint64_t ll, uint64_t lr,
-                             int ylen, int xlen, unsigned ctlword)
+                             unsigned ylen, unsigned xlen, unsigned ctlword)
   __attribute__ ((nonnull (1)));
 
 // ncdirect_box() with the double box-drawing characters
 API int ncdirect_double_box(struct ncdirect* n, uint64_t ul, uint64_t ur,
                             uint64_t ll, uint64_t lr,
-                            int ylen, int xlen, unsigned ctlword)
+                            unsigned ylen, unsigned xlen, unsigned ctlword)
   __attribute__ ((nonnull (1)));
 
 // Provide a NULL 'ts' to block at length, a 'ts' of 0 for non-blocking
-// operation, and otherwise a timespec to bound blocking. Returns a single
-// Unicode code point, or (uint32_t)-1 on error. Returns 0 on a timeout. If
-// an event is processed, the return value is the 'id' field from that
-// event. 'ni' may be NULL.
-API uint32_t ncdirect_get(struct ncdirect* n, const struct timespec* ts,
+// operation, and otherwise an absolute deadline in terms of CLOCK_MONOTONIC.
+// Returns a single Unicode code point, a synthesized special key constant,
+// or (uint32_t)-1 on error. Returns 0 on a timeout. If an event is processed,
+// the return value is the 'id' field from that event. 'ni' may be NULL.
+API uint32_t ncdirect_get(struct ncdirect* n, const struct timespec* absdl,
                           ncinput* ni)
   __attribute__ ((nonnull (1)));
 
 // Get a file descriptor suitable for input event poll()ing. When this
-// descriptor becomes available, you can call ncdirect_getc_nblock(),
+// descriptor becomes available, you can call ncdirect_get_nblock(),
 // and input ought be ready. This file descriptor is *not* necessarily
 // the file descriptor associated with stdin (but it might be!).
 API int ncdirect_inputready_fd(struct ncdirect* n)
@@ -332,7 +259,7 @@ API int ncdirect_inputready_fd(struct ncdirect* n)
 // 'ni' may be NULL if the caller is uninterested in event details. If no event
 // is ready, returns 0.
 static inline uint32_t
-ncdirect_getc_nblock(struct ncdirect* n, ncinput* ni){
+ncdirect_get_nblock(struct ncdirect* n, ncinput* ni){
   struct timespec ts = { .tv_sec = 0, .tv_nsec = 0 };
   return ncdirect_get(n, &ts, ni);
 }
@@ -340,7 +267,7 @@ ncdirect_getc_nblock(struct ncdirect* n, ncinput* ni){
 // 'ni' may be NULL if the caller is uninterested in event details. Blocks
 // until an event is processed or a signal is received.
 static inline uint32_t
-ncdirect_getc_blocking(struct ncdirect* n, ncinput* ni){
+ncdirect_get_blocking(struct ncdirect* n, ncinput* ni){
   return ncdirect_get(n, NULL, ni);
 }
 
@@ -350,31 +277,6 @@ API int ncdirect_stop(struct ncdirect* nc);
 typedef struct ncplane ncdirectv;
 typedef struct ncvisual ncdirectf;
 
-// FIXME this ought be used in the rendered mode API as well; it's currently
-// only used by direct mode. describes all geometries of an ncvisual--both those
-// which are inherent, and those in a given rendering regime. pixy and pixx are
-// the true internal pixel geometry, taken directly from the load (and updated
-// by ncvisual_resize()). cdimy/cdimx are the cell pixel geometry *at the time
-// of this call* (it can change with a font change, in which case all values
-// other than pixy/pixx are invalidated). rpixy/rpixx are the pixel geometry as
-// handed to the blitter, following any scaling. scaley/scalex are the number
-// of input pixels drawn to full cell; when using NCBLIT_PIXEL, they are
-// equivalent to cdimy/cdimx. rcelly/rcellx are the cell geometry as written by
-// the blitter, following any padding (there is padding whenever rpix{y, x} is
-// not evenly divided by scale{y, x}, and also sometimes for Sixel).
-// maxpixely/maxpixelx are defined only when NCBLIT_PIXEL is used, and specify
-// the largest bitmap that the terminal is willing to accept.
-typedef struct ncvgeom {
-  int pixy, pixx;     // true pixel geometry of ncvisual data
-  int cdimy, cdimx;   // terminal cell geometry when this was calculated
-  int rpixy, rpixx;   // rendered pixel geometry
-  int rcelly, rcellx; // rendered cell geometry
-  int scaley, scalex; // pixels per filled cell
-  // only defined for NCBLIT_PIXEL
-  int maxpixely, maxpixelx;
-  ncblitter_e blitter;// blitter that will be used
-} ncvgeom;
-
 // Display an image using the specified blitter and scaling. The image may
 // be arbitrarily many rows -- the output will scroll -- but will only occupy
 // the column of the cursor, and those to the right. The render/raster process
@@ -434,28 +336,31 @@ ALLOC API char* ncdirect_detected_terminal(const struct ncdirect* n)
 
 // Can we directly specify RGB values per cell, or only use palettes?
 static inline bool
-ncdirect_cantruecolor(const struct ncdirect* nc){
-  return ncdirect_capabilities(nc)->rgb;
+ncdirect_cantruecolor(const struct ncdirect* n){
+  return ncdirect_capabilities(n)->rgb;
 }
 
 // Can we set the "hardware" palette? Requires the "ccc" terminfo capability.
 static inline bool
-ncdirect_canchangecolor(const struct ncdirect* nc){
-  return nccapability_canchangecolor(ncdirect_capabilities(nc));
+ncdirect_canchangecolor(const struct ncdirect* n){
+  return nccapability_canchangecolor(ncdirect_capabilities(n));
 }
 
 // Can we fade? Fading requires either the "rgb" or "ccc" terminfo capability.
 static inline bool
-ncdirect_canfade(const struct ncdirect* nc){
-  return ncdirect_canchangecolor(nc) || ncdirect_cantruecolor(nc);
+ncdirect_canfade(const struct ncdirect* n){
+  return ncdirect_canchangecolor(n) || ncdirect_cantruecolor(n);
 }
 
 // Can we load images? This requires being built against FFmpeg/OIIO.
-API bool ncdirect_canopen_images(const struct ncdirect* n);
+static inline bool
+ncdirect_canopen_images(const struct ncdirect* n __attribute__ ((unused))){
+  return notcurses_canopen_images(NULL);
+}
 
 // Can we load videos? This requires being built against FFmpeg.
 static inline bool
-ncdirect_canopen_videos(const struct ncdirect* nc __attribute__ ((unused))){
+ncdirect_canopen_videos(const struct ncdirect* n __attribute__ ((unused))){
   return notcurses_canopen_videos(NULL);
 }
 
