@@ -1,19 +1,54 @@
 #include "compat/compat.h"
+#include <builddef.h>
+#include <pthread.h>
 #ifdef  __MINGW64__
 #include <string.h>
 #include <stdlib.h>
 #include <synchapi.h>
 #include <handleapi.h>
 #include <processthreadsapi.h>
+#include <stdio.h>
+
+char* notcurses_data_dir(void){
+  const char key[] = "Software\\Notcurses\\DataDir";
+  DWORD plen = 0;
+  LSTATUS r = RegGetValueA(HKEY_CURRENT_USER, key,
+			   NULL, RRF_RT_REG_SZ, NULL,
+                           NULL, &plen);
+  if(r){
+    goto err;
+  }
+  char* val = malloc(plen + 1);
+  if(val == NULL){
+    goto err;
+  }
+  r = RegGetValueA(HKEY_CURRENT_USER, key, NULL, RRF_RT_REG_SZ, NULL, val, &plen);
+  if(r){
+    free(val);
+    goto err;
+  }
+  return val;
+
+err:
+  return NOTCURSES_SHARE; // fall back to build path
+}
+
 char* strndup(const char* str, size_t size){
+  if(size == 0){
+    return NULL;
+  }
+  // t is how many bytes we're copying from the old string (not including
+  // any NUL terminator). it cannot be larger than strlen(str), nor can it
+  // be larger than size.
   size_t t = strlen(str);
   if(t > size){
     t = size;
   }
-  ++t;
-  char* r = malloc(t);
+  char* r = malloc(t + 1);
   if(r){
-    memcpy(r, str, t);
+    if(t){
+      memcpy(r, str, t);
+    }
     r[t] = '\0';
   }
   return r;
@@ -48,7 +83,12 @@ pid_t waitpid(pid_t pid, int* state, int options){ // FIXME
 #include <time.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <string.h>
 #include <fcntl.h>
+char* notcurses_data_dir(void){
+  return strdup(NOTCURSES_SHARE);
+}
+
 int set_fd_nonblocking(int fd, unsigned state, unsigned* oldstate){
   int flags = fcntl(fd, F_GETFL);
   if(flags < 0){
@@ -124,3 +164,28 @@ int clock_nanosleep(clockid_t clockid, int flags, const struct timespec *request
 }
 #endif
 #endif
+
+// initializes a pthread_cond_t to use CLOCK_MONOTONIC (as opposed to the
+// default CLOCK_REALTIME) if possible. if not possible, initializes a
+// regular ol' CLOCK_REALTIME condvar.
+int pthread_condmonotonic_init(pthread_cond_t* cond){
+  pthread_condattr_t cat;
+  if(pthread_condattr_init(&cat)){
+    return -1;
+  }
+  // FIXME we need a solution for this on macos/windows
+#ifndef __APPLE__
+#ifndef __MINGW64__
+  if(pthread_condattr_setclock(&cat, CLOCK_MONOTONIC)){
+    pthread_condattr_destroy(&cat);
+    return -1;
+  }
+#endif
+#endif
+  if(pthread_cond_init(cond, &cat)){
+    pthread_condattr_destroy(&cat);
+    return -1;
+  }
+  pthread_condattr_destroy(&cat);
+  return 0;
+}
