@@ -47,9 +47,10 @@ dragonmayer(struct ncvisual* ncv, const char* str, int iters){
   return total;
 }
 
-int dragon_demo(struct notcurses* nc){
+int dragon_demo(struct notcurses* nc, uint64_t startns){
+  (void)startns;
   done = false;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   --dimy; // don't disturb the menu bar
   // we use a Lindenmayer string rewriting system. the classic dragon curve
