@@ -1,5 +1,8 @@
-#include <zlib.h>
 #include "internal.h"
+#include <curses.h>
+#ifdef USE_DEFLATE
+#include <libdeflate.h>
+#endif
 
 // only invoked without suppress banners flag. prints various warnings based on
 // the environment / terminal definition. returns the number of lines printed.
@@ -24,7 +27,7 @@ int init_banner(const notcurses* nc, fbuf* f){
   if(clreol == NULL){
     clreol = "";
   }
-  if(!nc->suppress_banner){
+  if(!(nc->flags & NCOPTION_SUPPRESS_BANNERS)){
     term_fg_palindex(nc, f, 50 % nc->tcache.caps.colors);
     char* osver = notcurses_osversion();
     fbuf_printf(f, "%snotcurses %s on %s %s%s(%s)" NL,
@@ -35,13 +38,13 @@ int init_banner(const notcurses* nc, fbuf* f){
     free(osver);
     term_fg_palindex(nc, f, nc->tcache.caps.colors <= 256 ?
                      14 % nc->tcache.caps.colors : 0x2080e0);
-    if(nc->tcache.cellpixy && nc->tcache.cellpixx){
+    if(nc->tcache.cellpxy && nc->tcache.cellpxx){
       fbuf_printf(f, "%s%d rows (%dpx) %d cols (%dpx) %dx%d ",
                   clreol,
-                  nc->stdplane->leny, nc->tcache.cellpixy,
-                  nc->stdplane->lenx, nc->tcache.cellpixx,
-                  nc->stdplane->leny * nc->tcache.cellpixy,
-                  nc->stdplane->lenx * nc->tcache.cellpixx);
+                  nc->stdplane->leny, nc->tcache.cellpxy,
+                  nc->stdplane->lenx, nc->tcache.cellpxx,
+                  nc->stdplane->leny * nc->tcache.cellpxy,
+                  nc->stdplane->lenx * nc->tcache.cellpxx);
     }else{
       fbuf_printf(f, "%d rows %d cols ",
                   nc->stdplane->leny, nc->stdplane->lenx);
@@ -57,7 +60,11 @@ int init_banner(const notcurses* nc, fbuf* f){
                        14 % nc->tcache.caps.colors : 0x2080e0);
       fbuf_putc(f, '+');
     }
-    fbuf_printf(f, "%u colors" NL "%s%s%s (%s)" NL "%sterminfo from %s zlib %s GPM %s" NL,
+    // we want the terminfo version, which is tied to ncurses
+    const char* ncursesver = curses_version();
+    const char* ncver = strchr(ncursesver, ' ');
+    ncver = ncver ? ncver + 1 : ncursesver;
+    fbuf_printf(f, "%u colors" NL "%s%s%s (%s)" NL "%sterminfo %s libdeflate %s GPM %s" NL,
                 nc->tcache.caps.colors, clreol,
 #ifdef __clang__
             "", // name is contained in __VERSION__
@@ -78,7 +85,13 @@ int init_banner(const notcurses* nc, fbuf* f){
 #else
 #error "No __BYTE_ORDER__ definition"
 #endif
-            clreol, curses_version(), zlibVersion(), gpm_version());
+            clreol, ncver,
+#ifdef USE_DEFLATE
+            LIBDEFLATE_VERSION_STRING,
+#else
+            "n/a",
+#endif
+            gpm_version());
     fbuf_puts(f, clreol); // for ncvisual banner
     ncvisual_printbanner(f);
     init_banner_warnings(nc, f, clreol);
