@@ -66,6 +66,7 @@ int ncdirect_putegc(ncdirect* nc, uint64_t channels, const char* utf8,
 
 int ncdirect_cursor_up(ncdirect* nc, int num){
   if(num < 0){
+    logerror("requested negative move %d\n", num);
     return -1;
   }
   if(num == 0){
@@ -80,6 +81,7 @@ int ncdirect_cursor_up(ncdirect* nc, int num){
 
 int ncdirect_cursor_left(ncdirect* nc, int num){
   if(num < 0){
+    logerror("requested negative move %d\n", num);
     return -1;
   }
   if(num == 0){
@@ -94,6 +96,7 @@ int ncdirect_cursor_left(ncdirect* nc, int num){
 
 int ncdirect_cursor_right(ncdirect* nc, int num){
   if(num < 0){
+    logerror("requested negative move %d\n", num);
     return -1;
   }
   if(num == 0){
@@ -112,6 +115,7 @@ int ncdirect_cursor_right(ncdirect* nc, int num){
 // necessary but performing no carriage return -- a pure line feed.
 int ncdirect_cursor_down(ncdirect* nc, int num){
   if(num < 0){
+    logerror("requested negative move %d\n", num);
     return -1;
   }
   if(num == 0){
@@ -140,28 +144,30 @@ int ncdirect_clear(ncdirect* nc){
   return -1;
 }
 
-int ncdirect_dim_x(ncdirect* nc){
-  int x;
+unsigned ncdirect_dim_x(ncdirect* nc){
+  unsigned x;
   if(nc->tcache.ttyfd >= 0){
-    if(update_term_dimensions(NULL, &x, &nc->tcache, 0) == 0){
+    unsigned cgeo, pgeo; // don't care about either
+    if(update_term_dimensions(NULL, &x, &nc->tcache, 0, &cgeo, &pgeo) == 0){
       return x;
     }
   }else{
     return 80; // lol
   }
-  return -1;
+  return 0;
 }
 
-int ncdirect_dim_y(ncdirect* nc){
-  int y;
+unsigned ncdirect_dim_y(ncdirect* nc){
+  unsigned y;
   if(nc->tcache.ttyfd >= 0){
-    if(update_term_dimensions(&y, NULL, &nc->tcache, 0) == 0){
+    unsigned cgeo, pgeo; // don't care about either
+    if(update_term_dimensions(&y, NULL, &nc->tcache, 0, &cgeo, &pgeo) == 0){
       return y;
     }
   }else{
     return 24; // lol
   }
-  return -1;
+  return 0;
 }
 
 int ncdirect_cursor_enable(ncdirect* nc){
@@ -181,12 +187,12 @@ int ncdirect_cursor_disable(ncdirect* nc){
 }
 
 static int
-cursor_yx_get(ncdirect* n, const char* u7, int* y, int* x){
+cursor_yx_get(ncdirect* n, const char* u7, unsigned* y, unsigned* x){
   struct inputctx* ictx = n->tcache.ictx;
   if(ncdirect_flush(n)){
     return -1;
   }
-  int fakey, fakex;
+  unsigned fakey, fakex;
   if(y == NULL){
     y = &fakey;
   }
@@ -194,7 +200,7 @@ cursor_yx_get(ncdirect* n, const char* u7, int* y, int* x){
     x = &fakex;
   }
   get_cursor_location(ictx, u7, y, x);
-  loginfo("cursor at y=%d x=%d\n", *y, *x);
+  loginfo("cursor at y=%u x=%u\n", *y, *x);
   return 0;
 }
 
@@ -211,9 +217,11 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(hpa){
       return term_emit(tiparm(hpa, x), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n, u7, &y, NULL)){
+      unsigned yprime;
+      if(cursor_yx_get(n, u7, &yprime, NULL)){
         return -1;
       }
+      y = yprime;
     }else{
       y = 0;
     }
@@ -221,9 +229,11 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(!vpa){
       return term_emit(tiparm(vpa, y), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n, u7, NULL, &x)){
+      unsigned xprime;
+      if(cursor_yx_get(n, u7, NULL, &xprime)){
         return -1;
       }
+      x = xprime;
     }else{
       x = 0;
     }
@@ -357,10 +367,8 @@ detect_cursor_inversion_wrapper(ncdirect* n, const char* u7, int* y, int* x){
 */
 
 // no terminfo capability for this. dangerous--it involves writing controls to
-// the terminal, and then reading a response. many things can distupt this
-// non-atomic procedure, leading to unexpected results. a garbage function.
-int ncdirect_cursor_yx(ncdirect* n, int* y, int* x){
-  struct termios termio, oldtermios;
+// the terminal, and then reading a response.
+int ncdirect_cursor_yx(ncdirect* n, unsigned* y, unsigned* x){
   // this is only meaningful for real terminals
   if(n->tcache.ttyfd < 0){
     return -1;
@@ -370,33 +378,14 @@ int ncdirect_cursor_yx(ncdirect* n, int* y, int* x){
     fprintf(stderr, "Terminal doesn't support cursor reporting\n");
     return -1;
   }
-  if(tcgetattr(n->tcache.ttyfd, &termio)){
-    fprintf(stderr, "Couldn't get terminal info from %d (%s)\n",
-            n->tcache.ttyfd, strerror(errno));
-    return -1;
-  }
-  memcpy(&oldtermios, &termio, sizeof(termio));
-  // we might already be in cbreak mode from ncdirect_init(), but just in case
-  // it got changed by the client code since then, duck into cbreak mode anew.
-  termio.c_lflag &= ~(ICANON | ECHO);
-  if(tcsetattr(n->tcache.ttyfd, TCSAFLUSH, &termio)){
-    fprintf(stderr, "Couldn't put terminal into cbreak mode via %d (%s)\n",
-            n->tcache.ttyfd, strerror(errno));
-    return -1;
-  }
-  int ret, yval, xval;
+  unsigned yval, xval;
   if(!y){
     y = &yval;
   }
   if(!x){
     x = &xval;
   }
-  ret = cursor_yx_get(n, u7, y, x);
-  if(tcsetattr(n->tcache.ttyfd, TCSANOW, &oldtermios)){
-    fprintf(stderr, "Couldn't restore terminal mode on %d (%s)\n",
-            n->tcache.ttyfd, strerror(errno)); // don't return error for this
-  }
-  return ret;
+  return cursor_yx_get(n, u7, y, x);
 }
 
 int ncdirect_cursor_push(ncdirect* n){
@@ -416,11 +405,11 @@ int ncdirect_cursor_pop(ncdirect* n){
 }
 
 static inline int
-ncdirect_align(struct ncdirect* n, ncalign_e align, int c){
+ncdirect_align(struct ncdirect* n, ncalign_e align, unsigned c){
   if(align == NCALIGN_LEFT){
     return 0;
   }
-  int cols = ncdirect_dim_x(n);
+  unsigned cols = ncdirect_dim_x(n);
   if(c > cols){
     return 0;
   }
@@ -434,10 +423,10 @@ ncdirect_align(struct ncdirect* n, ncalign_e align, int c){
 
 // y is an out-only param, indicating the location where drawing started
 static int
-ncdirect_dump_sprixel(ncdirect* n, const ncplane* np, int xoff, int* y, fbuf* f){
-  int dimy, dimx;
+ncdirect_dump_sprixel(ncdirect* n, const ncplane* np, int xoff, unsigned* y, fbuf* f){
+  unsigned dimy, dimx;
   ncplane_dim_yx(np, &dimy, &dimx);
-  const int toty = ncdirect_dim_y(n);
+  const unsigned toty = ncdirect_dim_y(n);
   // flush our FILE*, as we're about to use UNIX I/O (since we can't rely on
   // stdio to transfer large amounts at once).
   if(ncdirect_flush(n)){
@@ -448,9 +437,10 @@ ncdirect_dump_sprixel(ncdirect* n, const ncplane* np, int xoff, int* y, fbuf* f)
   }
   if(toty - dimy < *y){
     int scrolls = *y - 1;
-    *y = toty - dimy;
-    if(*y < 0){
+    if(toty <= dimy){
       *y = 0;
+    }else{
+      *y = toty - dimy;
     }
     scrolls -= *y;
     // perform our scrolling outside of the fbuf framework, as we need it
@@ -521,16 +511,16 @@ ncdirect_set_fg_default_f(ncdirect* nc, fbuf* f){
 
 static int
 ncdirect_dump_cellplane(ncdirect* n, const ncplane* np, fbuf* f, int xoff){
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncplane_dim_yx(np, &dimy, &dimx);
-  const int toty = ncdirect_dim_y(n);
+  const unsigned toty = ncdirect_dim_y(n);
   // save the existing style and colors
   const bool fgdefault = ncdirect_fg_default_p(n);
   const bool bgdefault = ncdirect_bg_default_p(n);
   const uint32_t fgrgb = ncchannels_fg_rgb(n->channels);
   const uint32_t bgrgb = ncchannels_bg_rgb(n->channels);
-  for(int y = 0 ; y < dimy ; ++y){
-    for(int x = 0 ; x < dimx ; ++x){
+  for(unsigned y = 0 ; y < dimy ; ++y){
+    for(unsigned x = 0 ; x < dimx ; ++x){
       uint16_t stylemask;
       uint64_t channels;
       char* egc = ncplane_at_yx(np, y, x, &stylemask, &channels);
@@ -597,7 +587,7 @@ ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
     return -1;
   }
   if(np->sprite){
-    int y;
+    unsigned y;
     if(ncdirect_dump_sprixel(n, np, xoff, &y, &f)){
       fbuf_free(&f);
       return -1;
@@ -645,10 +635,6 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
   if(!vopts){
     vopts = &defvopts;
   }
-  if(vopts->leny < 0 || vopts->lenx < 0){
-    fprintf(stderr, "Invalid render geometry %d/%d\n", vopts->leny, vopts->lenx);
-    return NULL;
-  }
 //fprintf(stderr, "OUR DATA: %p rows/cols: %d/%d outsize: %d/%d %d/%d\n", ncv->data, ncv->pixy, ncv->pixx, dimy, dimx, ymax, xmax);
 //fprintf(stderr, "render %d/%d to scaling: %d\n", ncv->pixy, ncv->pixx, vopts->scaling);
   const struct blitset* bset = rgba_blitter_low(&n->tcache, vopts->scaling,
@@ -657,19 +643,19 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
   if(!bset){
     return NULL;
   }
-  int ymax = vopts->leny / bset->height;
-  int xmax = vopts->lenx / bset->width;
+  unsigned ymax = vopts->leny / bset->height;
+  unsigned xmax = vopts->lenx / bset->width;
   unsigned dimy = vopts->leny > 0 ? ymax : ncdirect_dim_y(n);
   unsigned dimx = vopts->lenx > 0 ? xmax : ncdirect_dim_x(n);
-  int disprows, dispcols, outy;
+  unsigned disprows, dispcols, outy;
   if(vopts->scaling != NCSCALE_NONE && vopts->scaling != NCSCALE_NONE_HIRES){
     if(bset->geom != NCBLIT_PIXEL){
       dispcols = dimx * encoding_x_scale(&n->tcache, bset);
       disprows = dimy * encoding_y_scale(&n->tcache, bset) - 1;
       outy = disprows;
     }else{
-      dispcols = dimx * n->tcache.cellpixx;
-      disprows = dimy * n->tcache.cellpixy;
+      dispcols = dimx * n->tcache.cellpxx;
+      disprows = dimy * n->tcache.cellpxy;
       clamp_to_sixelmax(&n->tcache, &disprows, &dispcols, &outy, vopts->scaling);
     }
     if(vopts->scaling == NCSCALE_SCALE || vopts->scaling == NCSCALE_SCALE_HIRES){
@@ -689,7 +675,7 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
     }
   }
   if(bset->geom == NCBLIT_PIXEL){
-    while((outy + n->tcache.cellpixy - 1) / n->tcache.cellpixy > dimy){
+    while((outy + n->tcache.cellpxy - 1) / n->tcache.cellpxy > dimy){
       outy -= n->tcache.sprixel_scale_height;
       disprows = outy;
     }
@@ -707,8 +693,8 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
     .flags = 0,
   };
   if(bset->geom == NCBLIT_PIXEL){
-    nopts.rows = outy / n->tcache.cellpixy + !!(outy % n->tcache.cellpixy);
-    nopts.cols = dispcols / n->tcache.cellpixx + !!(dispcols % n->tcache.cellpixx);
+    nopts.rows = outy / n->tcache.cellpxy + !!(outy % n->tcache.cellpxy);
+    nopts.cols = dispcols / n->tcache.cellpxx + !!(dispcols % n->tcache.cellpxx);
   }
   if(ymax && nopts.rows > ymax){
     nopts.rows = ymax;
@@ -731,7 +717,9 @@ ncdirect_render_visual(ncdirect* n, ncvisual* ncv,
   }
   if(bset->geom == NCBLIT_PIXEL){
     bargs.u.pixel.colorregs = n->tcache.color_registers;
-    if((bargs.u.pixel.spx = sprixel_alloc(&n->tcache, ncdv, nopts.rows, nopts.cols)) == NULL){
+    bargs.u.pixel.cellpxy = n->tcache.cellpxy;
+    bargs.u.pixel.cellpxx = n->tcache.cellpxx;
+    if((bargs.u.pixel.spx = sprixel_alloc(ncdv, nopts.rows, nopts.cols)) == NULL){
       free_plane(ncdv);
       return NULL;
     }
@@ -813,7 +801,11 @@ int ncdirect_vprintf_aligned(ncdirect* n, int y, ncalign_e align, const char* fm
   if(r == NULL){
     return -1;
   }
-  const size_t len = ncstrwidth(r);
+  const int len = ncstrwidth(r, NULL, NULL);
+  if(len < 0){
+    free(r);
+    return -1;
+  }
   const int x = ncdirect_align(n, align, len);
   if(ncdirect_cursor_move_yx(n, y, x)){
     free(r);
@@ -845,9 +837,11 @@ ncdirect_stop_minimal(void* vnc){
     ret |= fbuf_finalize(&f, stdout);
   }
   if(nc->tcache.ttyfd >= 0){
-    if(nc->tcache.kbdlevel){
-      if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
-        ret = -1;
+    if(!(nc->flags & NCDIRECT_OPTION_DRAIN_INPUT)){
+      if(nc->tcache.kbdlevel){
+        if(tty_emit(KKEYBOARD_POP, nc->tcache.ttyfd)){
+          ret = -1;
+        }
       }
     }
     const char* cnorm = get_escape(&nc->tcache, ESCAPE_CNORM);
@@ -930,7 +924,8 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   if(ncvisual_init(loglevel)){
     goto err;
   }
-  update_term_dimensions(NULL, NULL, &ret->tcache, 0);
+  unsigned cgeo, pgeo; // both are don't-cares
+  update_term_dimensions(NULL, NULL, &ret->tcache, 0, &cgeo, &pgeo);
   ncdirect_set_styles(ret, 0);
   return ret;
 
@@ -979,14 +974,17 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
   if(fprintf(n->ttyfp, "%s", prompt) < 0){
     return NULL;
   }
-  int dimx = ncdirect_dim_x(n);
+  unsigned dimx = ncdirect_dim_x(n);
+  if(dimx == 0){
+    return NULL;
+  }
   // FIXME what if we're reading from redirected input, not a terminal?
-  int y, xstart;
+  unsigned y, xstart;
   if(cursor_yx_get(n, u7, &y, &xstart)){
     return NULL;
   }
   int tline = y;
-  int bline = y;
+  unsigned bline = y;
   wchar_t* str;
   int wspace = BUFSIZ / sizeof(*str);
   if((str = malloc(wspace * sizeof(*str))) == NULL){
@@ -997,8 +995,8 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
   str[wused++] = L'\0';
   ncinput ni;
   uint32_t id;
-  int oldx = xstart;
-  while((id = ncdirect_getc_blocking(n, &ni)) != (uint32_t)-1){
+  unsigned oldx = xstart;
+  while((id = ncdirect_get_blocking(n, &ni)) != (uint32_t)-1){
     if(ni.evtype == NCTYPE_RELEASE){
       continue;
     }
@@ -1033,7 +1031,7 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
       wpos = 1;
     }else if(id == 'e' && ni.ctrl){
       wpos = wused - 1;
-    }else if(nckey_supppuab_p(ni.id)){
+    }else if(nckey_synthesized_p(ni.id)){
       continue;
     }else{
       if(wspace - 1 < wused){
@@ -1057,7 +1055,7 @@ char* ncdirect_readline(ncdirect* n, const char* prompt){
         str[wused - 1] = L'\0';
       }
       // FIXME check modifiers
-      int x;
+      unsigned x;
       if(cursor_yx_get(n, u7, &y, &x)){
         break;
       }
@@ -1140,11 +1138,6 @@ ncdirect_style_emit(ncdirect* n, unsigned stylebits, fbuf* f){
   return r;
 }
 
-// turn on any specified stylebits
-int ncdirect_styles_on(ncdirect* n, unsigned stylebits){
-  return ncdirect_on_styles(n, stylebits);
-}
-
 int ncdirect_on_styles(ncdirect* n, unsigned stylebits){
   if((stylebits & n->tcache.supported_styles) < stylebits){ // unsupported styles
     return -1;
@@ -1164,11 +1157,7 @@ int ncdirect_on_styles(ncdirect* n, unsigned stylebits){
   return 0;
 }
 
-int ncdirect_styles_off(ncdirect* n, unsigned stylebits){
-  return ncdirect_off_styles(n, stylebits);
-}
-
-unsigned ncdirect_styles(const ncdirect* n){
+uint16_t ncdirect_styles(const ncdirect* n){
   return n->stylemask;
 }
 
@@ -1189,10 +1178,6 @@ int ncdirect_off_styles(ncdirect* n, unsigned stylebits){
   return 0;
 }
 
-int ncdirect_styles_set(ncdirect* n, unsigned stylebits){
-  return ncdirect_set_styles(n, stylebits);
-}
-
 // set the current stylebits to exactly those provided
 int ncdirect_set_styles(ncdirect* n, unsigned stylebits){
   if((stylebits & n->tcache.supported_styles) < stylebits){ // unsupported styles
@@ -1263,8 +1248,12 @@ int ncdirect_set_bg_default(ncdirect* nc){
   return 0;
 }
 
-int ncdirect_hline_interp(ncdirect* n, const char* egc, int len,
+int ncdirect_hline_interp(ncdirect* n, const char* egc, unsigned len,
                           uint64_t c1, uint64_t c2){
+  if(len == 0){
+    logerror("passed zero length\n");
+    return -1;
+  }
   unsigned ur, ug, ub;
   int r1, g1, b1, r2, g2, b2;
   int br1, bg1, bb1, br2, bg2, bb2;
@@ -1282,7 +1271,7 @@ int ncdirect_hline_interp(ncdirect* n, const char* egc, int len,
   int deltbr = br2 - br1;
   int deltbg = bg2 - bg1;
   int deltbb = bb2 - bb1;
-  int ret;
+  unsigned ret;
   bool fgdef = false, bgdef = false;
   if(ncchannels_fg_default_p(c1) && ncchannels_fg_default_p(c2)){
     if(ncdirect_set_fg_default(n)){
@@ -1297,12 +1286,12 @@ int ncdirect_hline_interp(ncdirect* n, const char* egc, int len,
     bgdef = true;
   }
   for(ret = 0 ; ret < len ; ++ret){
-    int r = (deltr * ret) / len + r1;
-    int g = (deltg * ret) / len + g1;
-    int b = (deltb * ret) / len + b1;
-    int br = (deltbr * ret) / len + br1;
-    int bg = (deltbg * ret) / len + bg1;
-    int bb = (deltbb * ret) / len + bb1;
+    int r = (deltr * (int)ret) / (int)len + r1;
+    int g = (deltg * (int)ret) / (int)len + g1;
+    int b = (deltb * (int)ret) / (int)len + b1;
+    int br = (deltbr * (int)ret) / (int)len + br1;
+    int bg = (deltbg * (int)ret) / (int)len + bg1;
+    int bb = (deltbb * (int)ret) / (int)len + bb1;
     if(!fgdef){
       ncdirect_set_fg_rgb8(n, r, g, b);
     }
@@ -1310,14 +1299,19 @@ int ncdirect_hline_interp(ncdirect* n, const char* egc, int len,
       ncdirect_set_bg_rgb8(n, br, bg, bb);
     }
     if(fprintf(n->ttyfp, "%s", egc) < 0){
-      break;
+      logerror("error emitting egc [%s]\n", egc);
+      return -1;
     }
   }
   return ret;
 }
 
-int ncdirect_vline_interp(ncdirect* n, const char* egc, int len,
+int ncdirect_vline_interp(ncdirect* n, const char* egc, unsigned len,
                           uint64_t c1, uint64_t c2){
+  if(len == 0){
+    logerror("passed zero length\n");
+    return -1;
+  }
   unsigned ur, ug, ub;
   int r1, g1, b1, r2, g2, b2;
   int br1, bg1, bb1, br2, bg2, bb2;
@@ -1329,13 +1323,13 @@ int ncdirect_vline_interp(ncdirect* n, const char* egc, int len,
   br1 = ur; bg1 = ug; bb1 = ub;
   ncchannels_bg_rgb8(c2, &ur, &ug, &ub);
   br2 = ur; bg2 = ug; bb2 = ub;
-  int deltr = (r2 - r1) / (len + 1);
-  int deltg = (g2 - g1) / (len + 1);
-  int deltb = (b2 - b1) / (len + 1);
-  int deltbr = (br2 - br1) / (len + 1);
-  int deltbg = (bg2 - bg1) / (len + 1);
-  int deltbb = (bb2 - bb1) / (len + 1);
-  int ret;
+  int deltr = (r2 - r1) / ((int)len + 1);
+  int deltg = (g2 - g1) / ((int)len + 1);
+  int deltb = (b2 - b1) / ((int)len + 1);
+  int deltbr = (br2 - br1) / ((int)len + 1);
+  int deltbg = (bg2 - bg1) / ((int)len + 1);
+  int deltbb = (bb2 - bb1) / ((int)len + 1);
+  unsigned ret;
   bool fgdef = false, bgdef = false;
   if(ncchannels_fg_default_p(c1) && ncchannels_fg_default_p(c2)){
     if(ncdirect_set_fg_default(n)){
@@ -1363,12 +1357,12 @@ int ncdirect_vline_interp(ncdirect* n, const char* egc, int len,
     if(!bgdef){
       ncchannels_set_bg_rgb8(&channels, br1, bg1, bb1);
     }
-    if(ncdirect_putstr(n, channels, egc) <= 0){
-      break;
+    if(ncdirect_putstr(n, channels, egc) == EOF){
+      return -1;
     }
     if(len - ret > 1){
       if(ncdirect_cursor_down(n, 1) || ncdirect_cursor_left(n, 1)){
-        break;
+        return -1;
       }
     }
   }
@@ -1379,19 +1373,21 @@ int ncdirect_vline_interp(ncdirect* n, const char* egc, int len,
 //  they cannot be complex EGCs, but only a single wchar_t, alas.
 int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
                  uint64_t ll, uint64_t lr, const wchar_t* wchars,
-                 int ylen, int xlen, unsigned ctlword){
+                 unsigned ylen, unsigned xlen, unsigned ctlword){
   if(xlen < 2 || ylen < 2){
     return -1;
   }
-  char hl[MB_CUR_MAX + 1];
-  char vl[MB_CUR_MAX + 1];
+  char hl[MB_LEN_MAX + 1];
+  char vl[MB_LEN_MAX + 1];
   unsigned edges;
   edges = !(ctlword & NCBOXMASK_TOP) + !(ctlword & NCBOXMASK_LEFT);
+  // FIXME rewrite all fprintfs as ncdirect_putstr()!
   if(edges >= box_corner_needs(ctlword)){
     if(activate_channels(n, ul)){
       return -1;
     }
     if(fprintf(n->ttyfp, "%lc", wchars[0]) < 0){
+      logerror("error emitting %lc\n", wchars[0]);
       return -1;
     }
   }else{
@@ -1400,11 +1396,13 @@ int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
   mbstate_t ps = {};
   size_t bytes;
   if((bytes = wcrtomb(hl, wchars[4], &ps)) == (size_t)-1){
+    logerror("error converting %lc\n", wchars[4]);
     return -1;
   }
   hl[bytes] = '\0';
   memset(&ps, 0, sizeof(ps));
   if((bytes = wcrtomb(vl, wchars[5], &ps)) == (size_t)-1){
+    logerror("error converting %lc\n", wchars[5]);
     return -1;
   }
   vl[bytes] = '\0';
@@ -1449,8 +1447,8 @@ int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
     }else{
       ncdirect_cursor_left(n, xlen - 1);
     }
+    ncdirect_cursor_down(n, 1);
   }
-  ncdirect_cursor_down(n, 1);
   // bottom line
   edges = !(ctlword & NCBOXMASK_BOTTOM) + !(ctlword & NCBOXMASK_LEFT);
   if(edges >= box_corner_needs(ctlword)){
@@ -1486,21 +1484,16 @@ int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
 
 int ncdirect_rounded_box(ncdirect* n, uint64_t ul, uint64_t ur,
                          uint64_t ll, uint64_t lr,
-                         int ylen, int xlen, unsigned ctlword){
+                         unsigned ylen, unsigned xlen, unsigned ctlword){
   return ncdirect_box(n, ul, ur, ll, lr, NCBOXROUNDW, ylen, xlen, ctlword);
 }
 
 int ncdirect_double_box(ncdirect* n, uint64_t ul, uint64_t ur,
-                         uint64_t ll, uint64_t lr,
-                         int ylen, int xlen, unsigned ctlword){
+                        uint64_t ll, uint64_t lr,
+                        unsigned ylen, unsigned xlen, unsigned ctlword){
   return ncdirect_box(n, ul, ur, ll, lr, NCBOXDOUBLEW, ylen, xlen, ctlword);
 }
 
-// Can we load images? This requires being built against FFmpeg/OIIO.
-bool ncdirect_canopen_images(const ncdirect* n __attribute__ ((unused))){
-  return notcurses_canopen_images(NULL);
-}
-
 // Is our encoding UTF-8? Requires LANG being set to a UTF8 locale.
 bool ncdirect_canutf8(const ncdirect* n){
   return n->tcache.caps.utf8;
@@ -1525,7 +1518,7 @@ int ncdirect_stream(ncdirect* n, const char* filename, ncstreamcb streamer,
   }
   // starting position *after displaying one frame* so as to effect any
   // necessary scrolling.
-  int y = -1, x = -1;
+  unsigned y = 0, x = 0;
   int lastid = -1;
   int thisid = -1;
   do{
@@ -1590,27 +1583,15 @@ ncdirectv* ncdirectf_render(ncdirect* n, ncdirectf* frame, const struct ncvisual
 
 int ncdirectf_geom(ncdirect* n, ncdirectf* frame,
                    const struct ncvisual_options* vopts, ncvgeom* geom){
-  geom->cdimy = n->tcache.cellpixy;
-  geom->cdimx = n->tcache.cellpixx;
-  geom->maxpixely = n->tcache.sixel_maxy;
-  geom->maxpixelx = n->tcache.sixel_maxx;
   const struct blitset* bset;
-  int r = ncvisual_blitset_geom(NULL, &n->tcache, frame, vopts,
-                                &geom->pixy, &geom->pixx,
-                                &geom->scaley, &geom->scalex,
-                                &geom->rpixy, &geom->rpixx, &bset);
-  if(r == 0){
-    // FIXME ncvisual_blitset_geom() ought calculate these two for us; until
-    // then, derive them ourselves. the row count might be short by one if
-    // we're using sixel, and we're not a multiple of 6
-    geom->rcelly = geom->pixy / geom->scaley;
-    geom->rcellx = geom->pixx / geom->scalex;
-    geom->blitter = bset->geom;
-  }
-  return r;
+  unsigned disppxy, disppxx, outy, outx;
+  int placey, placex;
+  return ncvisual_geom_inner(&n->tcache, frame, vopts, geom, &bset,
+                             &disppxy, &disppxx, &outy, &outx,
+                             &placey, &placex);
 }
 
-unsigned ncdirect_supported_styles(const ncdirect* nc){
+uint16_t ncdirect_supported_styles(const ncdirect* nc){
   return term_supported_styles(&nc->tcache);
 }
 
