@@ -3,6 +3,7 @@
 #include <unistd.h>
 #include "automaton.h"
 #include "internal.h"
+#include "render.h"
 #include "in.h"
 
 // Notcurses takes over stdin, and if it is not connected to a terminal, also
@@ -20,31 +21,22 @@
 // without unbounded allocation, and also without losing data. We'd furthermore
 // like to reliably differentiate escapes and regular input, even when that
 // latter contains escapes. Unbounded input will hopefully only be present when
-// redirected from a file (NCOPTION_TOSS_INPUT)
-
-// FIXME still need to:
-//  probably want pipes/eventfds rather than SIGCONT
+// redirected from a file.
 
+static sig_atomic_t cont_seen;
 static sig_atomic_t resize_seen;
 
 // called for SIGWINCH and SIGCONT, and causes block_on_input to return
 void sigwinch_handler(int signo){
   if(signo == SIGWINCH){
     resize_seen = signo;
+    sigcont_seen_for_render = 1;
+  }else if(signo == SIGCONT){
+    cont_seen = signo;
+    sigcont_seen_for_render = 1;
   }
 }
 
-// data collected from responses to our terminal queries.
-typedef struct termqueries {
-  int celly, cellx;     // cell geometry on startup
-  int pixy, pixx;       // pixel geometry on startup
-  int cursory, cursorx; // cursor location on startup
-  unsigned kittygraphs; // are kitty graphics supported?
-  int sixely, sixelx;   // maximum sixel size
-  int cregs;            // sixel color registers
-  unsigned appsync;     // application-sync supported?
-} termqueries;
-
 typedef struct cursorloc {
   int y, x;             // 0-indexed cursor location
 } cursorloc;
@@ -57,6 +49,11 @@ typedef HANDLE ipipe;
 
 // local state for the input thread. don't put this large struct on the stack.
 typedef struct inputctx {
+  // these two are not ringbuffers; we always move any leftover materia to the
+  // front of the queue (it ought be a handful of bytes at most).
+  unsigned char tbuf[BUFSIZ]; // only used if we have distinct terminal fd
+  unsigned char ibuf[BUFSIZ]; // might be intermingled bulk/control data
+
   int stdinfd;          // bulk in fd. always >= 0 (almost always 0). we do not
                         //  own this descriptor, and must not close() it.
   int termfd;           // terminal fd: -1 with no controlling terminal, or
@@ -69,11 +66,6 @@ typedef struct inputctx {
   int rmargin, bmargin; // margins in use at right and bottom
 
   automaton amata;
-
-  // these two are not ringbuffers; we always move any leftover materia to the
-  // front of the queue (it ought be a handful of bytes at most).
-  unsigned char ibuf[BUFSIZ]; // might be intermingled bulk/control data
-  unsigned char tbuf[BUFSIZ]; // only used if we have distinct terminal fd
   int ibufvalid;      // we mustn't read() if ibufvalid == sizeof(ibuf)
   int tbufvalid;      // only used if we have distinct terminal connection
 
@@ -84,6 +76,7 @@ typedef struct inputctx {
   // ringbuffers for processed, structured input
   cursorloc* csrs;    // cursor reports are dumped here
   ncinput* inputs;    // processed input is dumped here
+  int coutstanding;   // outstanding cursor location requests
   int csize, isize;   // total number of slots in csrs/inputs
   int cvalid, ivalid; // population count of csrs/inputs
   int cwrite, iwrite; // slot where we'll write the next csr/input;
@@ -105,7 +98,8 @@ typedef struct inputctx {
   unsigned drain;     // drain away bulk input?
   ncsharedstats *stats; // stats shared with notcurses context
 
-  ipipe readypipes[2];  // pipes[0]: poll()able fd indicating the presence of user input
+  ipipe ipipes[2];
+  ipipe readypipes[2]; // pipes[0]: poll()able fd indicating the presence of user input
   struct initial_responses* initdata;
   struct initial_responses* initdata_complete;
   bool failed;          // error initializing input automaton, abort
@@ -448,12 +442,14 @@ mark_pipe_ready(ipipe pipes[static 2]){
   char sig = 1;
 #ifndef __MINGW64__
   if(write(pipes[1], &sig, sizeof(sig)) != 1){
-    logwarn("error writing to readypipe (%d) (%s)\n", pipes[1], strerror(errno));
+    logwarn("error writing to pipe (%d) (%s)\n", pipes[1], strerror(errno));
 #else
   DWORD wrote;
   if(!WriteFile(pipes[1], &sig, sizeof(sig), &wrote, NULL) || wrote != sizeof(sig)){
-    logwarn("error writing to readypipe\n");
+    logwarn("error writing to pipe\n");
 #endif
+  }else{
+    loginfo("wrote to readiness pipe\n");
   }
 }
 
@@ -502,11 +498,11 @@ mouse_click(inputctx* ictx, unsigned release, char follow){
     logwarn("dropping click in margins %ld/%ld\n", y, x);
     return;
   }
-  if(x >= ictx->ti->dimx - (ictx->rmargin + ictx->lmargin)){
+  if((unsigned)x >= ictx->ti->dimx - (ictx->rmargin + ictx->lmargin)){
     logwarn("dropping click in margins %ld/%ld\n", y, x);
     return;
   }
-  if(y >= ictx->ti->dimy - (ictx->bmargin + ictx->tmargin)){
+  if((unsigned)y >= ictx->ti->dimy - (ictx->bmargin + ictx->tmargin)){
     logwarn("dropping click in margins %ld/%ld\n", y, x);
     return;
   }
@@ -515,13 +511,11 @@ mouse_click(inputctx* ictx, unsigned release, char follow){
     .alt = mods & 0x08,
     .shift = mods & 0x04,
   };
-  if(mods < 64){
-    tni.id = NCKEY_BUTTON1 + (mods % 4);
-  }else if(mods >= 64 && mods < 128){
-    tni.id = NCKEY_BUTTON4 + (mods % 4);
-  }else if(mods >= 128 && mods < 192){
-    tni.id = NCKEY_BUTTON8 + (mods % 4);
-  }
+  // SGR mouse reporting: lower two bits signify base button + {0, 1, 2} press
+  // and no button pressed/release/{3}. bit 5 indicates motion. bits 6 and 7
+  // select device groups: 64 is buttons 4--7, 128 is 8--11. a pure motion
+  // report (no button) is 35 (32 + 3 (no button pressed)) with (oddly enough)
+  // 'M' (i.e. release == true).
   if(release){
     tni.evtype = NCTYPE_RELEASE;
   }else{
@@ -529,6 +523,18 @@ mouse_click(inputctx* ictx, unsigned release, char follow){
   }
   tni.x = x;
   tni.y = y;
+  if(mods % 4 == 3){
+    tni.id = NCKEY_MOTION;
+    tni.evtype = NCTYPE_RELEASE;
+  }else{
+    if(mods < 64){
+      tni.id = NCKEY_BUTTON1 + (mods % 4);
+    }else if(mods >= 64 && mods < 128){
+      tni.id = NCKEY_BUTTON4 + (mods % 4);
+    }else if(mods >= 128 && mods < 192){
+      tni.id = NCKEY_BUTTON8 + (mods % 4);
+    }
+  }
   load_ncinput(ictx, &tni, 0);
 }
 
@@ -549,12 +555,14 @@ cursor_location_cb(inputctx* ictx){
   unsigned y = amata_next_numeric(&ictx->amata, "\x1b[", ';') - 1;
   unsigned x = amata_next_numeric(&ictx->amata, "", 'R') - 1;
   // the first one doesn't go onto the queue; consume it here
+  pthread_mutex_lock(&ictx->clock);
+  --ictx->coutstanding;
   if(ictx->initdata){
+    pthread_mutex_unlock(&ictx->clock);
     ictx->initdata->cursory = y;
     ictx->initdata->cursorx = x;
     return 2;
   }
-  pthread_mutex_lock(&ictx->clock);
   if(ictx->cvalid == ictx->csize){
     pthread_mutex_unlock(&ictx->clock);
     logwarn("dropping cursor location report %u/%u\n", y, x);
@@ -684,6 +692,8 @@ kitty_kbd(inputctx* ictx, int val, int mods, int evtype){
         synth = SIGINT;
       }else if(tni.id == '\\'){
         synth = SIGQUIT;
+      }else if(tni.id == 'Z'){
+        synth = SIGSTOP;
       }
     }
   }
@@ -745,6 +755,12 @@ legacy_functional(uint32_t id){
   return id;
 }
 
+static int
+simple_cb_begin(inputctx* ictx){
+  kitty_kbd(ictx, NCKEY_BEGIN, 0, 0);
+  return 2;
+}
+
 static int
 kitty_cb_functional(inputctx* ictx){
   unsigned val = amata_next_numeric(&ictx->amata, "\x1b[", ';');
@@ -767,6 +783,27 @@ wezterm_cb(inputctx* ictx){
   return 2;
 }
 
+static int
+legacy_cb_f1(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'P');
+  kitty_kbd(ictx, NCKEY_F01, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_f2(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'Q');
+  kitty_kbd(ictx, NCKEY_F02, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_f4(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'S');
+  kitty_kbd(ictx, NCKEY_F04, mods, 0);
+  return 2;
+}
+
 static int
 kitty_cb_f1(inputctx* ictx){
   unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", ':');
@@ -799,6 +836,34 @@ kitty_cb_f4(inputctx* ictx){
   return 2;
 }
 
+static int
+legacy_cb_right(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'C');
+  kitty_kbd(ictx, NCKEY_RIGHT, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_left(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'D');
+  kitty_kbd(ictx, NCKEY_LEFT, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_down(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'B');
+  kitty_kbd(ictx, NCKEY_DOWN, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_up(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'A');
+  kitty_kbd(ictx, NCKEY_UP, mods, 0);
+  return 2;
+}
+
 static int
 kitty_cb_right(inputctx* ictx){
   unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", ':');
@@ -831,6 +896,27 @@ kitty_cb_up(inputctx* ictx){
   return 2;
 }
 
+static int
+legacy_cb_begin(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'E');
+  kitty_kbd(ictx, NCKEY_BEGIN, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_end(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'F');
+  kitty_kbd(ictx, NCKEY_END, mods, 0);
+  return 2;
+}
+
+static int
+legacy_cb_home(inputctx* ictx){
+  unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", 'H');
+  kitty_kbd(ictx, NCKEY_HOME, mods, 0);
+  return 2;
+}
+
 static int
 kitty_cb_begin(inputctx* ictx){
   unsigned mods = amata_next_numeric(&ictx->amata, "\x1b[1;", ':');
@@ -1035,6 +1121,8 @@ da2_cb(inputctx* ictx){
   if(pv == 0){
     return 2;
   }
+  // modern XTerm replies to XTVERSION, but older versions require extracting
+  // the version from secondary DA
   if(ictx->initdata->qterm == TERMINAL_XTERM){
     if(ictx->initdata->version == NULL){
       char ver[8];
@@ -1203,6 +1291,7 @@ xtversion_cb(inputctx* ictx){
   return 2;
 }
 
+// XTGETTCAP responses are delimited by semicolons
 static int
 tcap_cb(inputctx* ictx){
   char* str = amata_next_string(&ictx->amata, "\x1bP1+r");
@@ -1214,20 +1303,47 @@ tcap_cb(inputctx* ictx){
     free(str);
     return 2;
   }
-  // 'TN' (Terminal Name)
-  if(strncasecmp(str, "544e=", 5) == 0){
-    const char* tn = str + 5;
-    if(strcasecmp(tn, "6D6C7465726D") == 0){
-      ictx->initdata->qterm = TERMINAL_MLTERM;
-    }else if(strcasecmp(tn, "787465726d2d6b69747479") == 0){
-      ictx->initdata->qterm = TERMINAL_KITTY;
-    }else if(strcasecmp(tn, "787465726D2D323536636F6C6F72") == 0){
-      ictx->initdata->qterm = TERMINAL_XTERM; // "xterm-256color"
+  char* s = str;
+  while(*s){
+    // FIXME clean this crap up, grotesque
+    if(strncasecmp(s, "544e=", 5) == 0){
+      size_t len = 5;
+      while(s[len] && s[len] != ';'){
+        ++len;
+      }
+      const char* tn = s + 5;
+      len -= 5;
+      if(ictx->initdata->qterm == TERMINAL_UNKNOWN){
+        if(len == 10){
+          if(strncasecmp(tn, "787465726d", 10) == 0){
+            ictx->initdata->qterm = TERMINAL_XTERM; // "xterm"
+          }
+        }else if(len == 12){
+          if(strncasecmp(tn, "6D6C7465726D", 12) == 0){
+            ictx->initdata->qterm = TERMINAL_MLTERM;
+          }
+        }else if(len == 22){
+          if(strncasecmp(tn, "787465726d2d6b69747479", 22) == 0){
+            ictx->initdata->qterm = TERMINAL_KITTY; // "xterm-kitty"
+          }
+        }else if(len == 28){
+          if(strncasecmp(tn, "787465726d2d323536636f6c6f72", 28) == 0){
+            ictx->initdata->qterm = TERMINAL_XTERM; // "xterm-256color"
+          }
+        }else{
+          logdebug("unknown terminal name %s\n", tn);
+        }
+      }
+    }else if(strncasecmp(s, "524742=", 7) == 0){
+      loginfo("got rgb (%s)\n", s);
+      ictx->initdata->rgb = true;
     }else{
-      logdebug("unknown terminal name %s\n", tn);
+      logdebug("unknown capability=val %s\n", str);
     }
-  }else{
-    logdebug("unknown capability=val %s\n", str);
+    if((s = strchr(s, ';')) == NULL){
+      break;
+    }
+    ++s;
   }
   free(str);
   return 2;
@@ -1264,6 +1380,7 @@ build_cflow_automaton(inputctx* ictx){
     triefunc fxn;
   } csis[] = {
     // CSI (\e[)
+    { "[E", simple_cb_begin, },
     { "[<\\N;\\N;\\NM", mouse_press_cb, },
     { "[<\\N;\\N;\\Nm", mouse_release_cb, },
     // technically these must begin with "4" or "8"; enforce in callbacks
@@ -1274,6 +1391,16 @@ build_cflow_automaton(inputctx* ictx){
     { "[\\N;\\N:\\Nu", kitty_cb_complex, },
     { "[\\N;\\N;\\N~", xtmodkey_cb, },
     { "[\\N;\\N:\\N~", kitty_cb_functional, },
+    { "[1;\\NP", legacy_cb_f1, },
+    { "[1;\\NQ", legacy_cb_f2, },
+    { "[1;\\NS", legacy_cb_f4, },
+    { "[1;\\ND", legacy_cb_left, },
+    { "[1;\\NC", legacy_cb_right, },
+    { "[1;\\NB", legacy_cb_down, },
+    { "[1;\\NA", legacy_cb_up, },
+    { "[1;\\NE", legacy_cb_begin, },
+    { "[1;\\NF", legacy_cb_end, },
+    { "[1;\\NH", legacy_cb_home, },
     { "[1;\\N:\\NP", kitty_cb_f1, },
     { "[1;\\N:\\NQ", kitty_cb_f2, },
     { "[1;\\N:\\NR", kitty_cb_f3, },
@@ -1396,42 +1523,49 @@ create_inputctx(tinfo* ti, FILE* infp, int lmargin, int tmargin, int rmargin,
       i->isize = BUFSIZ;
       if( (i->inputs = malloc(sizeof(*i->inputs) * i->isize)) ){
         if(pthread_mutex_init(&i->ilock, NULL) == 0){
-          if(pthread_cond_init(&i->icond, NULL) == 0){
+          if(pthread_condmonotonic_init(&i->icond) == 0){
             if(pthread_mutex_init(&i->clock, NULL) == 0){
-              if(pthread_cond_init(&i->ccond, NULL) == 0){
+              if(pthread_condmonotonic_init(&i->ccond) == 0){
                 if((i->stdinfd = fileno(infp)) >= 0){
                   if( (i->initdata = malloc(sizeof(*i->initdata))) ){
                     if(getpipes(i->readypipes) == 0){
-                      memset(&i->amata, 0, sizeof(i->amata));
-                      if(prep_special_keys(i) == 0){
-                        if(set_fd_nonblocking(i->stdinfd, 1, &ti->stdio_blocking_save) == 0){
-                          i->termfd = tty_check(i->stdinfd) ? -1 : get_tty_fd(infp);
-                          memset(i->initdata, 0, sizeof(*i->initdata));
-                          i->initdata->qterm = ti->qterm;
-                          i->iread = i->iwrite = i->ivalid = 0;
-                          i->cread = i->cwrite = i->cvalid = 0;
-                          i->initdata_complete = NULL;
-                          i->stats = stats;
-                          i->ti = ti;
-                          i->stdineof = 0;
+                      if(getpipes(i->ipipes) == 0){
+                        memset(&i->amata, 0, sizeof(i->amata));
+                        if(prep_special_keys(i) == 0){
+                          if(set_fd_nonblocking(i->stdinfd, 1, &ti->stdio_blocking_save) == 0){
+                            i->termfd = tty_check(i->stdinfd) ? -1 : get_tty_fd(infp);
+                            memset(i->initdata, 0, sizeof(*i->initdata));
+                            i->coutstanding = 1; // one in initial request set
+                            i->initdata->qterm = ti->qterm;
+                            i->initdata->cursory = -1;
+                            i->initdata->cursorx = -1;
+                            i->iread = i->iwrite = i->ivalid = 0;
+                            i->cread = i->cwrite = i->cvalid = 0;
+                            i->initdata->kbdlevel = UINT_MAX;
+                            i->initdata_complete = NULL;
+                            i->stats = stats;
+                            i->ti = ti;
+                            i->stdineof = 0;
 #ifdef __MINGW64__
-                          i->stdinhandle = ti->inhandle;
+                            i->stdinhandle = ti->inhandle;
 #endif
-                          i->ibufvalid = 0;
-                          i->linesigs = linesigs_enabled;
-                          i->tbufvalid = 0;
-                          i->midescape = 0;
-                          i->lmargin = lmargin;
-                          i->tmargin = tmargin;
-                          i->rmargin = rmargin;
-                          i->bmargin = bmargin;
-                          i->drain = drain;
-                          i->failed = false;
-                          logdebug("input descriptors: %d/%d\n", i->stdinfd, i->termfd);
-                          return i;
+                            i->ibufvalid = 0;
+                            i->linesigs = linesigs_enabled;
+                            i->tbufvalid = 0;
+                            i->midescape = 0;
+                            i->lmargin = lmargin;
+                            i->tmargin = tmargin;
+                            i->rmargin = rmargin;
+                            i->bmargin = bmargin;
+                            i->drain = drain;
+                            i->failed = false;
+                            logdebug("input descriptors: %d/%d\n", i->stdinfd, i->termfd);
+                            return i;
+                          }
                         }
+                        input_free_esctrie(&i->amata);
                       }
-                      input_free_esctrie(&i->amata);
+                      endpipes(i->ipipes);
                     }
                     endpipes(i->readypipes);
                   }
@@ -1476,6 +1610,7 @@ free_inputctx(inputctx* i){
       free(i->initdata_complete);
     }
     endpipes(i->readypipes);
+    endpipes(i->ipipes);
     free(i->inputs);
     free(i->csrs);
     free(i);
@@ -1862,6 +1997,13 @@ process_ibuf(inputctx* ictx){
     load_ncinput(ictx, &tni, 0);
     resize_seen = 0;
   }
+  if(cont_seen){
+    ncinput tni = {
+      .id = NCKEY_SIGNAL,
+    };
+    load_ncinput(ictx, &tni, 0);
+    cont_seen = 0;
+  }
   if(ictx->tbufvalid){
     // we could theoretically do this in parallel with process_bulk, but it
     // hardly seems worthwhile without breaking apart the fetches of input.
@@ -1895,6 +2037,7 @@ int ncinput_shovel(inputctx* ictx, const void* buf, int len){
 // are set high iff they are ready for reading, and otherwise cleared.
 static int
 block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
+  logtrace("blocking on input availability\n");
   *rtfd = *rifd = 0;
   unsigned nonblock = ictx->midescape;
   if(nonblock){
@@ -1903,7 +2046,17 @@ block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
   }
 #ifdef __MINGW64__
   int timeoutms = nonblock ? 0 : -1;
-  DWORD d = WaitForSingleObject(ictx->stdinhandle, timeoutms);
+  DWORD ncount = 0;
+  HANDLE handles[2];
+  if(!ictx->stdineof){
+    if(ictx->ibufvalid != sizeof(ictx->ibuf)){
+      handles[ncount++] = ictx->stdinhandle;
+    }
+  }
+  if(ncount == 0){
+    handles[ncount++] = ictx->ipipes[0];
+  }
+  DWORD d = WaitForMultipleObjects(ncount, handles, false, timeoutms);
   if(d == WAIT_TIMEOUT){
     return 0;
   }else if(d == WAIT_FAILED){
@@ -1928,12 +2081,20 @@ block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
       ++pfdcount;
     }
   }
+  if(pfdcount == 0){
+    loginfo("output queues full; blocking on ipipes\n");
+    pfds[pfdcount].fd = ictx->ipipes[0];
+    pfds[pfdcount].events = inevents;
+    pfds[pfdcount].revents = 0;
+    ++pfdcount;
+  }
   if(ictx->termfd >= 0){
     pfds[pfdcount].fd = ictx->termfd;
     pfds[pfdcount].events = inevents;
     pfds[pfdcount].revents = 0;
     ++pfdcount;
   }
+  logtrace("waiting on %d fds (ibuf: %u/%"PRIuPTR")\n", pfdcount, ictx->ibufvalid, sizeof(ictx->ibuf));
   sigset_t smask;
   sigfillset(&smask);
   sigdelset(&smask, SIGCONT);
@@ -1943,12 +2104,6 @@ block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
   // on exit (in cancel_and_join()).
   sigdelset(&smask, SIGTHR);
 #endif
-  if(pfdcount == 0){
-    loginfo("output queues full; blocking on signals\n");
-    int signum;
-    sigwait(&smask, &signum);
-    return 0;
-  }
   int events;
 #if defined(__APPLE__) || defined(__MINGW64__)
   int timeoutms = nonblock ? 0 : -1;
@@ -1958,12 +2113,15 @@ block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
   struct timespec* pts = nonblock ? &ts : NULL;
   while((events = ppoll(pfds, pfdcount, pts, &smask)) < 0){
 #endif
-    if(errno != EAGAIN && errno != EBUSY && errno != EWOULDBLOCK){
-      return -1;
-    }else if(errno == EINTR){
+    if(errno == EINTR){
+      loginfo("interrupted by signal\n");
       return resize_seen;
+    }else if(errno != EAGAIN && errno != EBUSY && errno != EWOULDBLOCK){
+      logerror("error polling (%s)\n", strerror(errno));
+      return -1;
     }
   }
+  loginfo("poll returned %d\n", events);
   pfdcount = 0;
   while(events){
     if(pfds[pfdcount].revents){
@@ -1971,11 +2129,13 @@ block_on_input(inputctx* ictx, unsigned* rtfd, unsigned* rifd){
         *rifd = 1;
       }else if(pfds[pfdcount].fd == ictx->termfd){
         *rtfd = 1;
+      }else if(pfds[pfdcount].fd == ictx->ipipes[0]){
       }
       --events;
     }
+    ++pfdcount;
   }
-loginfo("got events: %c%c %d\n", *rtfd ? 'T' : 't', *rifd ? 'I' : 'i', pfds[0].revents);
+  loginfo("got events: %c%c\n", *rtfd ? 'T' : 't', *rifd ? 'I' : 'i');
   return events;
 #endif
 }
@@ -2063,6 +2223,10 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
   uint32_t id;
   if(ictx->drain){
     logerror("input is being drained\n");
+    if(ni){
+      memset(ni, 0, sizeof(*ni));
+      ni->id = (uint32_t)-1;
+    }
     return (uint32_t)-1;
   }
   pthread_mutex_lock(&ictx->ilock);
@@ -2079,12 +2243,19 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
     if(ts == NULL){
       pthread_cond_wait(&ictx->icond, &ictx->ilock);
     }else{
-      int r = pthread_cond_clockwait(&ictx->icond, &ictx->ilock, CLOCK_MONOTONIC, ts);
+      int r = pthread_cond_timedwait(&ictx->icond, &ictx->ilock, ts);
       if(r == ETIMEDOUT){
         pthread_mutex_unlock(&ictx->ilock);
+        if(ni){
+          memset(ni, 0, sizeof(*ni));
+        }
         return 0;
       }else if(r < 0){
         inc_input_errors(ictx);
+        if(ni){
+          memset(ni, 0, sizeof(*ni));
+          ni->id = (uint32_t)-1;
+        }
         return (uint32_t)-1;
       }
     }
@@ -2092,6 +2263,9 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
   id = ictx->inputs[ictx->iread].id;
   if(ni){
     memcpy(ni, &ictx->inputs[ictx->iread], sizeof(*ni));
+    if(notcurses_ucs32_to_utf8(&ni->id, 1, (unsigned char*)ni->utf8, sizeof(ni->utf8)) < 0){
+      ni->utf8[0] = 0;
+    }
   }
   if(++ictx->iread == ictx->isize){
     ictx->iread = 0;
@@ -2099,62 +2273,40 @@ internal_get(inputctx* ictx, const struct timespec* ts, ncinput* ni){
   bool sendsignal = false;
   if(ictx->ivalid-- == ictx->isize){
     sendsignal = true;
-  }else if(ictx->ivalid){
-    logtrace("draining event readiness pipe\n");
-    char c;
+  }else{
+    logtrace("draining event readiness pipe %d\n", ictx->ivalid);
 #ifndef __MINGW64__
+    char c;
     while(read(ictx->readypipes[0], &c, sizeof(c)) == 1){
       // FIXME accelerate?
     }
 #else
-    while(ReadFile(ictx->readypipes[0], &c, sizeof(c), NULL, NULL)){
+    // we ought be draining this, but it breaks everything, as we can't easily
+    // do nonblocking input from a pipe in windows, augh...
+    // Ne pleure pas, Alfred! J'ai besoin de tout mon courage pour mourir a vingt ans!
+    /*while(ReadFile(ictx->readypipes[0], &c, sizeof(c), NULL, NULL)){
       // FIXME accelerate?
-    }
+    }*/
 #endif
   }
   pthread_mutex_unlock(&ictx->ilock);
   if(sendsignal){
-    pthread_kill(ictx->tid, SIGCONT);
+    mark_pipe_ready(ictx->ipipes);
   }
   return id;
 }
 
-static void
-delaybound_to_deadline(const struct timespec* ts, struct timespec* absdl){
-  if(ts){
-    // incoming ts is a delay bound, but we want an absolute deadline for
-    // pthread_cond_timedwait(). convert it.
-    struct timeval tv;
-    gettimeofday(&tv, NULL);
-    absdl->tv_sec = ts->tv_sec + tv.tv_sec;
-    absdl->tv_nsec = ts->tv_nsec + tv.tv_usec * 1000;
-    if(absdl->tv_nsec > 1000000000){
-      ++absdl->tv_sec;
-      absdl->tv_nsec -= 1000000000;
-    }
-  }
-}
-
 // infp has already been set non-blocking
-uint32_t notcurses_get(notcurses* nc, const struct timespec* ts, ncinput* ni){
-  struct timespec absdl;
-  delaybound_to_deadline(ts, &absdl);
-  uint32_t id = internal_get(nc->tcache.ictx, ts ? &absdl : NULL, ni);
-  if(ni){
-    if(id == (uint32_t)-1){
-      ni->id = id;
-    }
-  }
-  return id;
+uint32_t notcurses_get(notcurses* nc, const struct timespec* absdl, ncinput* ni){
+  uint32_t ret = internal_get(nc->tcache.ictx, absdl, ni);
+  return ret;
 }
 
 // FIXME better performance if we move this within the locked area
-int notcurses_getvec(notcurses* n, const struct timespec* ts,
+int notcurses_getvec(notcurses* n, const struct timespec* absdl,
                      ncinput* ni, int vcount){
-  struct timespec absdl;
-  delaybound_to_deadline(ts, &absdl);
   for(int v = 0 ; v < vcount ; ++v){
-    uint32_t u = notcurses_get(n, ts ? &absdl : NULL, &ni[v]);
+    uint32_t u = notcurses_get(n, absdl, &ni[v]);
     if(u == (uint32_t)-1){
       if(v == 0){
         return -1;
@@ -2167,27 +2319,18 @@ int notcurses_getvec(notcurses* n, const struct timespec* ts,
   return vcount;
 }
 
-uint32_t ncdirect_get(ncdirect* n, const struct timespec* ts, ncinput* ni){
-  return internal_get(n->tcache.ictx, ts, ni);
+uint32_t ncdirect_get(ncdirect* n, const struct timespec* absdl, ncinput* ni){
+  return internal_get(n->tcache.ictx, absdl, ni);
 }
 
-uint32_t notcurses_getc(notcurses* nc, const struct timespec* ts,
-                        const void* unused, ncinput* ni){
-  (void)unused; // FIXME remove for abi3
-  return notcurses_get(nc, ts, ni);
-}
-
-uint32_t ncdirect_getc(ncdirect* nc, const struct timespec *ts,
-                       const void* unused, ncinput* ni){
-  (void)unused; // FIXME remove for abi3
-  return ncdirect_get(nc, ts, ni);
-}
-
-int get_cursor_location(inputctx* ictx, const char* u7, int* y, int* x){
+int get_cursor_location(inputctx* ictx, const char* u7, unsigned* y, unsigned* x){
   pthread_mutex_lock(&ictx->clock);
   while(ictx->cvalid == 0){
-    if(tty_emit(u7, ictx->ti->ttyfd)){
-      return -1;
+    if(ictx->coutstanding == 0){
+      if(tty_emit(u7, ictx->ti->ttyfd)){
+        return -1;
+      }
+      ++ictx->coutstanding;
     }
     pthread_cond_wait(&ictx->ccond, &ictx->clock);
   }
