@@ -1,6 +1,8 @@
-#include <zlib.h>
 #include "internal.h"
 #include "base64.h"
+#ifdef USE_DEFLATE
+#include <libdeflate.h>
+#endif
 
 // Kitty has its own bitmap graphics protocol, rather superior to DEC Sixel.
 // A header is written with various directives, followed by a number of
@@ -231,8 +233,8 @@ init_sprixel_animation(sprixel* s){
 // auxiliary vector back into the actual data. we then free the auxvector.
 int kitty_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
   const int totalpixels = s->pixy * s->pixx;
-  const int xpixels = s->cellpxx;
-  const int ypixels = s->cellpxy;
+  const int xpixels = ncplane_pile(s->n)->cellpxx;
+  const int ypixels = ncplane_pile(s->n)->cellpxy;
   int targx = xpixels;
   if((xcell + 1) * xpixels > s->pixx){
     targx = s->pixx - xcell * xpixels;
@@ -306,8 +308,8 @@ int kitty_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 // wiping)?
 static inline unsigned
 kitty_anim_auxvec_blitsource_p(const sprixel* s, const uint8_t* auxvec){
-  const size_t offset = s->cellpxy * s->cellpxx * 4;
-  if(auxvec[offset]){
+  size_t off = ncplane_pile(s->n)->cellpxy * ncplane_pile(s->n)->cellpxx * 4;
+  if(auxvec[off]){
     return 1;
   }
   return 0;
@@ -352,8 +354,8 @@ kitty_anim_auxvec(int dimy, int dimx, int posy, int posx,
   return a;
 }
 
-uint8_t* kitty_trans_auxvec(const tinfo* ti){
-  const size_t slen = ti->cellpixy * ti->cellpixx;
+uint8_t* kitty_trans_auxvec(const ncpile* p){
+  const size_t slen = p->cellpxy * p->cellpxx;
   uint8_t* a = malloc(slen);
   if(a){
     memset(a, 0, slen);
@@ -365,13 +367,14 @@ uint8_t* kitty_trans_auxvec(const tinfo* ti){
 // by the wipe proper, and when blitting a new frame with annihilations.
 static int
 kitty_blit_wipe_selfref(sprixel* s, fbuf* f, int ycell, int xcell){
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
   if(fbuf_printf(f, "\x1b_Ga=f,x=%d,y=%d,s=%d,v=%d,i=%d,X=1,r=2,c=1,q=2;",
-                 xcell * s->cellpxx, ycell * s->cellpxy,
-                 s->cellpxx, s->cellpxy, s->id) < 0){
+                 xcell * cellpxx, ycell * cellpxy, cellpxx, cellpxy, s->id) < 0){
     return -1;
   }
   // FIXME ought be smaller around the fringes!
-  int totalp = s->cellpxy * s->cellpxx;
+  int totalp = cellpxy * cellpxx;
   // FIXME preserve so long as cellpixel geom stays constant?
   #define TRINULLALPHA "AAAAAAAAAAAAAAAA"
   for(int p = 0 ; p + 3 <= totalp ; p += 3){
@@ -414,7 +417,7 @@ int kitty_wipe_animation(sprixel* s, int ycell, int xcell){
   }
   int tamidx = ycell * s->dimx + xcell;
   uint8_t* auxvec = s->n->tam[tamidx].auxvector;
-  auxvec[s->cellpxx * s->cellpxy * 4] = 0;
+  auxvec[ncplane_pile(s->n)->cellpxx * ncplane_pile(s->n)->cellpxy * 4] = 0;
   s->invalidated = SPRIXEL_INVALIDATED;
   return 1;
 }
@@ -442,14 +445,14 @@ sprixel* kitty_recycle(ncplane* n){
   int dimy = hides->dimy;
   int dimx = hides->dimx;
   sprixel_hide(hides);
-  return sprixel_alloc(&ncplane_notcurses_const(n)->tcache, n, dimy, dimx);
+  return sprixel_alloc(n, dimy, dimx);
 }
 
 // for pre-animation kitty (NCPIXEL_KITTY_STATIC), we need a byte per pixel,
 // in which we stash the alpha.
 static inline uint8_t*
 kitty_auxiliary_vector(const sprixel* s){
-  int pixels = s->cellpxy * s->cellpxx;
+  int pixels = ncplane_pile(s->n)->cellpxy * ncplane_pile(s->n)->cellpxx;
   uint8_t* ret = malloc(sizeof(*ret) * pixels);
   if(ret){
     memset(ret, 0, sizeof(*ret) * pixels);
@@ -464,8 +467,8 @@ int kitty_wipe(sprixel* s, int ycell, int xcell){
     return -1;
   }
   const int totalpixels = s->pixy * s->pixx;
-  const int xpixels = s->cellpxx;
-  const int ypixels = s->cellpxy;
+  const int xpixels = ncplane_pile(s->n)->cellpxx;
+  const int ypixels = ncplane_pile(s->n)->cellpxy;
   // if the cell is on the right or bottom borders, it might only be partially
   // filled by actual graphic data, and we need to cap our target area.
   int targx = xpixels;
@@ -513,7 +516,7 @@ int kitty_wipe(sprixel* s, int ycell, int xcell){
 //fprintf(stderr, "POSTCHOMP: [%.16s]\n", c + tripbytes);
       assert(chomped >= 0);
       auxvecidx += chomped;
-      assert(auxvecidx <= s->cellpxy * s->cellpxx);
+      assert(auxvecidx <= ypixels * xpixels);
       thisrow -= chomped;
 //fprintf(stderr, "POSTCHIMP CHOMP: %d pixoffset: %d next: %d tripbytes: %d tripskip: %d thisrow: %d\n", chomped, pixoffset, nextpixel, tripbytes, tripskip, thisrow);
       if(thisrow == 0){
@@ -547,7 +550,7 @@ int kitty_commit(fbuf* f, sprixel* s, unsigned noscroll){
   loginfo("Committing Kitty graphic id %u\n", s->id);
   int i;
   if(s->pxoffx || s->pxoffy){
-    i = fbuf_printf(f, "\e_Ga=p,i=%u,p=1,X=%u,Y=%u%s\e\\", s->id,
+    i = fbuf_printf(f, "\e_Ga=p,i=%u,p=1,X=%u,Y=%u%s,q=2\e\\", s->id,
                     s->pxoffx, s->pxoffy, noscroll ? ",C=1" : "");
   }else{
     i = fbuf_printf(f, "\e_Ga=p,i=%u,p=1,q=2%s\e\\", s->id, noscroll ? ",C=1" : "");
@@ -559,19 +562,17 @@ int kitty_commit(fbuf* f, sprixel* s, unsigned noscroll){
   return 0;
 }
 
-static inline void*
-zctx_origbuf(z_stream* zctx, int pixy, int pixx){
-  size_t blen = pixx * pixy * 4;
-  unsigned long b = deflateBound(zctx, blen);
-  return (unsigned char *)zctx->next_out - (b - zctx->avail_out);
-}
-
+// chunkify and write the collected buffer in the animated case. this might
+// or might not be compressed (depends on whether compression was useful).
 static int
-encode_and_chunkify(fbuf* f, z_stream* zctx, int pixy, int pixx){
-  unsigned long totw = zctx->total_out;
-  unsigned char* buf = zctx_origbuf(zctx, pixy, pixx);
+encode_and_chunkify(fbuf* f, const unsigned char* buf, size_t blen, unsigned compressed){
   // need to terminate the header, requiring semicolon
-  if(totw > 4096 * 3 / 4){
+  if(compressed){
+    if(fbuf_putn(f, ",o=z", 4) < 0){
+      return -1;
+    }
+  }
+  if(blen > 4096 * 3 / 4){
     if(fbuf_putn(f, ",m=1", 4) < 0){
       return -1;
     }
@@ -582,7 +583,7 @@ encode_and_chunkify(fbuf* f, z_stream* zctx, int pixy, int pixx){
   bool first = true;
   unsigned long i = 0;
   char b64d[4];
-  while(totw - i > 4096 * 3 / 4){
+  while(blen - i > 4096 * 3 / 4){
     if(!first){
       if(fbuf_putn(f, "\x1b_Gm=1;", 7) < 0){
         return -1;
@@ -606,13 +607,13 @@ encode_and_chunkify(fbuf* f, z_stream* zctx, int pixy, int pixx){
       return -1;
     }
   }
-  while(i < totw){
-    if(totw - i < 3){
-      base64final(buf + i, b64d, totw - i);
+  while(i < blen){
+    if(blen - i < 3){
+      base64final(buf + i, b64d, blen - i);
       if(fbuf_putn(f, b64d, 4) < 0){
         return -1;
       }
-      i += totw - i;
+      i += blen - i;
     }else{
       base64x3(buf + i, b64d);
       if(fbuf_putn(f, b64d, 4) < 0){
@@ -628,25 +629,41 @@ encode_and_chunkify(fbuf* f, z_stream* zctx, int pixy, int pixx){
 }
 
 static int
-finalize_deflator(z_stream* zctx, fbuf* f, int dimy, int dimx){
-  assert(0 == zctx->avail_in);
-  int zret = deflate(zctx, Z_FINISH);
-  if(zret != Z_STREAM_END){
-    logerror("Error deflating (%d)\n", zret);
+deflate_buf(void* buf, fbuf* f, int dimy, int dimx){
+  const size_t blen = dimx * dimy * 4;
+  void* cbuf = NULL;
+  size_t clen = 0;
+#ifdef USE_DEFLATE
+  // 2 has been shown to work pretty well for things that are actually going
+  // to compress; results per unit time fall off quickly after 2.
+  struct libdeflate_compressor* cmp = libdeflate_alloc_compressor(2);
+  if(cmp == NULL){
+    logerror("couldn't get libdeflate context\n");
     return -1;
   }
-  if(encode_and_chunkify(f, zctx, dimy, dimx)){
-    return -1;
+  // if this allocation fails, just skip compression, no need to bail
+  cbuf = malloc(blen);
+  if(cbuf){
+    clen = libdeflate_zlib_compress(cmp, buf, blen, cbuf, blen);
   }
-  return 0;
+  libdeflate_free_compressor(cmp);
+#endif
+  int ret;
+  if(0 == clen){ // wasn't enough room; compressed data is larger than original
+    loginfo("deflated in vain; using original %" PRIuPTR "B\n", blen);
+    ret = encode_and_chunkify(f, buf, blen, 0);
+  }else{
+    loginfo("deflated %" PRIuPTR "B to %" PRIuPTR "B\n", blen, clen);
+    ret = encode_and_chunkify(f, cbuf, clen, 1);
+  }
+  free(cbuf);
+  return ret;
 }
 
-// copy |encodeable| pixels to the deflate state
-static int
-add_to_deflator(z_stream* zctx, const uint32_t* src, int encodeable, bool wipe[static 3]){
-  assert(0 == zctx->avail_in);
-  uint32_t dst[3];
-  zctx->next_in = (void*)dst;
+// copy |encodeable| ([1..3]) pixels from |src| to the buffer |dst|, setting
+// alpha along the way according to |wipe|.
+static inline int
+add_to_buf(uint32_t *dst, const uint32_t* src, int encodeable, bool wipe[static 3]){
   dst[0] = *src++;
   if(wipe[0] || rgba_trans_p(dst[0], 0)){
     ncpixel_set_a(&dst[0], 0);
@@ -663,52 +680,25 @@ add_to_deflator(z_stream* zctx, const uint32_t* src, int encodeable, bool wipe[s
       }
     }
   }
-  zctx->avail_in = encodeable * 4;
-  int zret = deflate(zctx, Z_NO_FLUSH);
-  if(zret != Z_OK){
-    logerror("Error deflating (%d)\n", zret);
+  return 0;
+}
+
+// writes to |*animated| based on normalized |level|. if we're not animated,
+// we won't be using compression.
+static inline int
+prep_animation(ncpixelimpl_e level, uint32_t** buf, int leny, int lenx, unsigned* animated){
+  if(level < NCPIXEL_KITTY_ANIMATED){
+    *animated = false;
+    *buf = NULL;
+    return 0;
+  }
+  *animated = true;
+  if((*buf = malloc(lenx * leny * sizeof(uint32_t))) == NULL){
     return -1;
   }
   return 0;
 }
 
-// writes to |*animated| based on normalized |level|
-static int
-prep_deflator(ncpixelimpl_e level, z_stream* zctx, int pixy, int pixx,
-              unsigned* animated){
-  memset(zctx, 0, sizeof(*zctx));
-  if(level >= NCPIXEL_KITTY_ANIMATED){
-    int zret;
-    // 2 seems to work well for things that are going to compress up
-    // meaningfully at all, while not taking too much time.
-    if((zret = deflateInit(zctx, 2)) != Z_OK){
-      logerror("Couldn't get a deflate context (%d)\n", zret);
-      return -1;
-    }
-    size_t blen = pixx * pixy * 4;
-    unsigned long b = deflateBound(zctx, blen);
-    unsigned char* buf = malloc(b);
-    if(buf == NULL){
-      deflateEnd(zctx);
-      return -1;
-    }
-    zctx->avail_out = b;
-    zctx->next_out = buf;
-    *animated = true;
-  }else{
-    *animated = false;
-  }
-  return 0;
-}
-
-static void
-destroy_deflator(unsigned animated, z_stream* zctx, int pixy, int pixx){
-  if(animated){
-    free(zctx_origbuf(zctx, pixy, pixx));
-    deflateEnd(zctx);
-  }
-}
-
 // if we're NCPIXEL_KITTY_SELFREF, and we're blitting a secondary frame, we need
 // carry through the TAM's annihilation entires...but we also need load the
 // frame *without* annihilations, lest we be unable to build it. we thus go
@@ -718,10 +708,10 @@ destroy_deflator(unsigned animated, z_stream* zctx, int pixy, int pixx){
 static int
 finalize_multiframe_selfref(sprixel* s, fbuf* f){
   int prewiped = 0;
-  for(int y = 0 ; y < s->dimy ; ++y){
-    for(int x = 0 ; x < s->dimx ; ++x){
-      int tyxidx = y * s->dimx + x;
-      int state = s->n->tam[tyxidx].state;
+  for(unsigned y = 0 ; y < s->dimy ; ++y){
+    for(unsigned x = 0 ; x < s->dimx ; ++x){
+      unsigned tyxidx = y * s->dimx + x;
+      unsigned state = s->n->tam[tyxidx].state;
       if(state >= SPRIXCELL_ANNIHILATED){
         if(kitty_blit_wipe_selfref(s, f, y, x)){
           return -1;
@@ -730,7 +720,7 @@ finalize_multiframe_selfref(sprixel* s, fbuf* f){
       }
     }
   }
-  loginfo("transitively wiped %d/%d\n", prewiped, s->dimy * s->dimx);
+  loginfo("transitively wiped %d/%u\n", prewiped, s->dimy * s->dimx);
   return 0;
 }
 
@@ -746,19 +736,22 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
     logerror("stride (%d) badly aligned\n", linesize);
     return -1;
   }
-  // we only deflate if we're using animation, since otherwise we need be able
-  // to edit the encoded bitmap in-place for wipes/restores.
-  z_stream zctx;
   unsigned animated;
-  if(prep_deflator(level, &zctx, leny, lenx, &animated)){
+  uint32_t* buf;
+  // we'll be collecting the pixels, modified to reflect alpha nullification
+  // due to preexisting wipes, into a temporary buffer for compression (iff
+  // we're animated). pixels are 32 bits each.
+  if(prep_animation(level, &buf, leny, lenx, &animated)){
     return -1;
   }
+  unsigned bufidx = 0; // an index; the actual offset is bufidx * 4
   bool translucent = bargs->flags & NCVISUAL_OPTION_BLEND;
   sprixel* s = bargs->u.pixel.spx;
-  int sprixelid = s->id;
-  int cdimy = s->cellpxy;
-  int cdimx = s->cellpxx;
-  uint32_t transcolor = bargs->transcolor;
+  const int cdimy = bargs->u.pixel.cellpxy;
+  const int cdimx = bargs->u.pixel.cellpxx;
+  assert(0 != cdimy);
+  assert(0 != cdimx);
+  const uint32_t transcolor = bargs->transcolor;
   int total = leny * lenx; // total number of pixels (4 * total == bytecount)
   // number of 4KiB chunks we'll need
   int chunks = (total + (RGBA_MAXLEN - 1)) / RGBA_MAXLEN;
@@ -777,10 +770,11 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
     if(totalout == 0){
       // older versions of kitty will delete uploaded images when scrolling,
       // alas. see https://github.com/dankamongmen/notcurses/issues/1910 =[.
-      // parse_start isn't used in animation mode, so no worries there.
+      // parse_start isn't used in animation mode, so no worries about the
+      // fact that this doesn't complete the header in that case.
       *parse_start = fbuf_printf(f, "\e_Gf=32,s=%d,v=%d,i=%d,p=1,a=t,%s",
-                                 lenx, leny, sprixelid,
-                                 animated ? "o=z,q=2" : chunks ? "m=1;" : "q=2;");
+                                 lenx, leny, s->id,
+                                 animated ? "q=2" : chunks ? "m=1;" : "q=2;");
       if(*parse_start < 0){
         goto err;
       }
@@ -856,7 +850,7 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
           }else if(level == NCPIXEL_KITTY_SELFREF){
             selfref_annihilated = true;
           }else{
-            ((uint8_t*)tam[tyx].auxvector)[s->cellpxx * s->cellpxy * 4] = 1;
+            ((uint8_t*)tam[tyx].auxvector)[cdimx * cdimy * 4] = 1;
             wipe[e] = 1;
           }
           if(rgba_trans_p(source[e], transcolor)){
@@ -898,9 +892,10 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
       }
       totalout += encodeable;
       if(animated){
-        if(add_to_deflator(&zctx, source, encodeable, wipe)){
+        if(add_to_buf(buf + bufidx, source, encodeable, wipe)){
           goto err;
         }
+        bufidx += encodeable;
       }else{
         // we already took transcolor to alpha 0; there's no need to
         // check it again, so pass 0.
@@ -916,8 +911,10 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
       }
     }
   }
+  // we only deflate if we're using animation, since otherwise we need be able
+  // to edit the encoded bitmap in-place for wipes/restores.
   if(animated){
-    if(finalize_deflator(&zctx, f, leny, lenx)){
+    if(deflate_buf(buf, f, leny, lenx)){
       goto err;
     }
     if(selfref_annihilated){
@@ -927,13 +924,13 @@ write_kitty_data(fbuf* f, int linesize, int leny, int lenx, int cols,
     }
   }
   scrub_tam_boundaries(tam, leny, lenx, cdimy, cdimx);
-  destroy_deflator(animated, &zctx, leny, lenx);
+  free(buf);
   return 0;
 
 err:
   logerror("failed blitting kitty graphics\n");
   cleanup_tam(tam, (leny + cdimy - 1) / cdimy, (lenx + cdimx - 1) / cdimx);
-  destroy_deflator(animated, &zctx, leny, lenx);
+  free(buf);
   return -1;
 }
 
@@ -944,14 +941,16 @@ int kitty_rebuild_selfref(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
     return -1;
   }
   fbuf* f = &s->glyph;
-  const int ystart = ycell * s->cellpxy;
-  const int xstart = xcell * s->cellpxx;
-  const int xlen = xstart + s->cellpxx > s->pixx ? s->pixx - xstart : s->cellpxx;
-  const int ylen = ystart + s->cellpxy > s->pixy ? s->pixy - ystart : s->cellpxy;
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
+  const int ystart = ycell * cellpxy;
+  const int xstart = xcell * cellpxx;
+  const int xlen = xstart + cellpxx > s->pixx ? s->pixx - xstart : cellpxx;
+  const int ylen = ystart + cellpxy > s->pixy ? s->pixy - ystart : cellpxy;
   logdebug("rematerializing %u at %d/%d (%dx%d)\n", s->id, ycell, xcell, ylen, xlen);
   fbuf_printf(f, "\e_Ga=c,x=%d,y=%d,X=%d,Y=%d,w=%d,h=%d,i=%d,r=1,c=2,q=2;\x1b\\",
-              xcell * s->cellpxx, ycell * s->cellpxy,
-              xcell * s->cellpxx, ycell * s->cellpxy,
+              xcell * cellpxx, ycell * cellpxy,
+              xcell * cellpxx, ycell * cellpxy,
               xlen, ylen, s->id);
   const int tyx = xcell + ycell * s->dimx;
   memcpy(&s->n->tam[tyx].state, auxvec, sizeof(s->n->tam[tyx].state));
@@ -965,10 +964,12 @@ int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
     return -1;
   }
   fbuf* f = &s->glyph;
-  const int ystart = ycell * s->cellpxy;
-  const int xstart = xcell * s->cellpxx;
-  const int xlen = xstart + s->cellpxx > s->pixx ? s->pixx - xstart : s->cellpxx;
-  const int ylen = ystart + s->cellpxy > s->pixy ? s->pixy - ystart : s->cellpxy;
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
+  const int ystart = ycell * cellpxy;
+  const int xstart = xcell * cellpxx;
+  const int xlen = xstart + cellpxx > s->pixx ? s->pixx - xstart : cellpxx;
+  const int ylen = ystart + cellpxy > s->pixy ? s->pixy - ystart : cellpxy;
   const int linesize = xlen * 4;
   const int total = xlen * ylen;
   const int tyx = xcell + ycell * s->dimx;
@@ -979,13 +980,13 @@ int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
   int targetout = 0; // number of pixels expected out after this chunk
 //fprintf(stderr, "total: %d chunks = %d, s=%d,v=%d\n", total, chunks, lenx, leny);
   // FIXME this ought be factored out and shared with write_kitty_data()
-  logdebug("placing %d/%d at %d/%d\n", ylen, xlen, ycell * s->cellpxy, xcell * s->cellpxx);
+  logdebug("placing %d/%d at %d/%d\n", ylen, xlen, ycell * cellpxy, xcell * cellpxx);
   while(chunks--){
     if(totalout == 0){
       const int c = kitty_anim_auxvec_blitsource_p(s, auxvec) ? 2 : 1;
       const int r = kitty_anim_auxvec_blitsource_p(s, auxvec) ? 1 : 2;
       if(fbuf_printf(f, "\e_Ga=f,x=%d,y=%d,s=%d,v=%d,i=%d,X=1,c=%d,r=%d,%s;",
-                     xcell * s->cellpxx, ycell * s->cellpxy, xlen, ylen,
+                     xcell * cellpxx, ycell * cellpxy, xlen, ylen,
                      s->id, c, r, chunks ? "m=1" : "q=2") < 0){
         return -1;
       }
@@ -1029,13 +1030,13 @@ int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 //fprintf(stderr, "Tyx: %d y: %d (%d) * %d x: %d (%d) state %d %p\n", tyx, y, y / cdimy, cols, x, x / cdimx, tam[tyx].state, tam[tyx].auxvector);
         wipe[e] = 0;
         if(rgba_trans_p(source[e], 0)){
-          if(x % s->cellpxx == 0 && y % s->cellpxy == 0){
+          if(x % cellpxx == 0 && y % cellpxy == 0){
             s->n->tam[tyx].state = SPRIXCELL_TRANSPARENT;
           }else if(s->n->tam[tyx].state == SPRIXCELL_OPAQUE_KITTY){
             s->n->tam[tyx].state = SPRIXCELL_MIXED_KITTY;
           }
         }else{
-          if(x % s->cellpxx == 0 && y % s->cellpxy == 0){
+          if(x % cellpxx == 0 && y % cellpxy == 0){
             s->n->tam[tyx].state = SPRIXCELL_OPAQUE_KITTY;
           }else if(s->n->tam[tyx].state == SPRIXCELL_TRANSPARENT){
             s->n->tam[tyx].state = SPRIXCELL_MIXED_KITTY;
@@ -1117,7 +1118,7 @@ int kitty_blit_selfref(ncplane* n, int linesize, const void* data,
 
 int kitty_remove(int id, fbuf* f){
   loginfo("Removing graphic %u\n", id);
-  if(fbuf_printf(f, "\e_Ga=d,d=i,i=%d\e\\", id) < 0){
+  if(fbuf_printf(f, "\e_Ga=d,d=I,i=%d\e\\", id) < 0){
     return -1;
   }
   return 0;
@@ -1126,9 +1127,10 @@ int kitty_remove(int id, fbuf* f){
 // damages cells underneath the graphic which were OPAQUE
 int kitty_scrub(const ncpile* p, sprixel* s){
 //fprintf(stderr, "FROM: %d/%d state: %d s->n: %p\n", s->movedfromy, s->movedfromx, s->invalidated, s->n);
-  for(int yy = s->movedfromy ; yy < s->movedfromy + s->dimy && yy < p->dimy ; ++yy){
-    for(int xx = s->movedfromx ; xx < s->movedfromx + s->dimx && xx < p->dimx ; ++xx){
+  for(unsigned yy = s->movedfromy ; yy < s->movedfromy + s->dimy && yy < p->dimy ; ++yy){
+    for(unsigned xx = s->movedfromx ; xx < s->movedfromx + s->dimx && xx < p->dimx ; ++xx){
       const int ridx = yy * p->dimx + xx;
+      assert(0 <= ridx);
       struct crender *r = &p->crender[ridx];
       if(!r->sprixel){
         if(s->n){
@@ -1183,7 +1185,7 @@ int kitty_move(sprixel* s, fbuf* f, unsigned noscroll, int yoff, int xoff){
   const int targx = s->n->absx;
   logdebug("moving %u to %d %d\n", s->id, targy, targx);
   int ret = 0;
-  if(goto_location(ncplane_notcurses(s->n), f, targy + yoff, targx + xoff)){
+  if(goto_location(ncplane_notcurses(s->n), f, targy + yoff, targx + xoff, s->n)){
     ret = -1;
   }else if(fbuf_printf(f, "\e_Ga=p,i=%d,p=1,q=2%s\e\\", s->id,
                        noscroll ? ",C=1" : "") < 0){
