@@ -1,11 +1,11 @@
 #include "linux.h"
 #include "internal.h"
 
-// auxvecs for framebuffer are 1B each for s->cellpxx * s->cellpxy elements,
+// auxvecs for framebuffer are 1B each for cellpxx * cellpxy elements,
 // and store the original alpha value.
 static inline uint8_t*
 fbcon_auxiliary_vector(const sprixel* s){
-  int pixels = s->cellpxy * s->cellpxx;
+  int pixels = ncplane_pile(s->n)->cellpxy * ncplane_pile(s->n)->cellpxx;
   uint8_t* ret = malloc(sizeof(*ret) * pixels);
   if(ret){
     memset(ret, 0, sizeof(*ret) * pixels);
@@ -18,19 +18,21 @@ int fbcon_wipe(sprixel* s, int ycell, int xcell){
   if(auxvec == NULL){
     return -1;
   }
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
   char* glyph = s->glyph.buf;
-  for(int y = 0 ; y < s->cellpxy ; ++y){
-    if(ycell * s->cellpxy + y >= s->pixy){
+  for(int y = 0 ; y < cellpxy ; ++y){
+    if(ycell * cellpxy + y >= s->pixy){
       break;
     }
     // number of pixels total above our pixel row
-    const size_t yoff = s->pixx * (ycell * s->cellpxy + y);
-    for(int x = 0 ; x < s->cellpxx ; ++x){
-      if(xcell * s->cellpxx + x >= s->pixx){
+    const size_t yoff = s->pixx * (ycell * cellpxy + y);
+    for(int x = 0 ; x < cellpxx ; ++x){
+      if(xcell * cellpxx + x >= s->pixx){
         break;
       }
-      size_t offset = (yoff + xcell * s->cellpxx + x) * 4;
-      const int vyx = (y % s->cellpxy) * s->cellpxx + x;
+      size_t offset = (yoff + xcell * cellpxx + x) * 4;
+      const int vyx = (y % cellpxy) * cellpxx + x;
       auxvec[vyx] = glyph[offset + 3];
       glyph[offset + 3] = 0;
     }
@@ -43,8 +45,8 @@ int fbcon_blit(struct ncplane* n, int linesize, const void* data,
                int leny, int lenx, const struct blitterargs* bargs){
   uint32_t transcolor = bargs->transcolor;
   sprixel* s = bargs->u.pixel.spx;
-  int cdimx = s->cellpxx;
-  int cdimy = s->cellpxy;
+  int cdimx = bargs->u.pixel.cellpxx;
+  int cdimy = bargs->u.pixel.cellpxy;
   // FIXME this will need be a copy of the tinfo's fbuf map
   size_t flen = leny * lenx * 4;
   if(fbuf_reserve(&s->glyph, flen)){
@@ -124,18 +126,20 @@ int fbcon_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
   if(auxvec == NULL){
     return -1;
   }
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
   sprixcell_e state = SPRIXCELL_TRANSPARENT;
-  for(int y = 0 ; y < s->cellpxy ; ++y){
-    if(ycell * s->cellpxy + y >= s->pixy){
+  for(int y = 0 ; y < cellpxy ; ++y){
+    if(ycell * cellpxy + y >= s->pixy){
       break;
     }
-    const size_t yoff = s->pixx * (ycell * s->cellpxy + y);
-    for(int x = 0 ; x < s->cellpxx ; ++x){
-      if(xcell * s->cellpxx + x >= s->pixx){
+    const size_t yoff = s->pixx * (ycell * cellpxy + y);
+    for(int x = 0 ; x < cellpxx ; ++x){
+      if(xcell * cellpxx + x >= s->pixx){
         break;
       }
-      size_t offset = (yoff + xcell * s->cellpxx + x) * 4;
-      const int vyx = (y % s->cellpxy) * s->cellpxx + x;
+      size_t offset = (yoff + xcell * cellpxx + x) * 4;
+      const int vyx = (y % cellpxy) * cellpxx + x;
       if(x == 0 && y == 0){
         if(auxvec[vyx] == 0){
           state = SPRIXCELL_TRANSPARENT;
@@ -158,10 +162,13 @@ int fbcon_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 }
 
 int fbcon_draw(const tinfo* ti, sprixel* s, int y, int x){
+  logdebug("id %" PRIu32 " dest %d/%d\n", s->id, y, x);
   int wrote = 0;
-  for(unsigned l = 0 ; l < (unsigned)s->pixy && l + y * ti->cellpixy < ti->pixy ; ++l){
+  const int cellpxy = ncplane_pile(s->n) ? ncplane_pile(s->n)->cellpxy : ti->cellpxy;
+  const int cellpxx = ncplane_pile(s->n) ? ncplane_pile(s->n)->cellpxx : ti->cellpxx;
+  for(unsigned l = 0 ; l < (unsigned)s->pixy && l + y * cellpxy < ti->pixy ; ++l){
     // FIXME pixel size isn't necessarily 4B, line isn't necessarily psize*pixx
-    size_t offset = ((l + y * ti->cellpixy) * ti->pixx + x * ti->cellpixx) * 4;
+    size_t offset = ((l + y * cellpxy) * ti->pixx + x * cellpxx) * 4;
     uint8_t* tl = ti->linux_fbuffer + offset;
     const char* src = (char*)s->glyph.buf + (l * s->pixx * 4);
     for(unsigned c = 0 ; c < (unsigned)s->pixx && c < ti->pixx ; ++c){
@@ -178,22 +185,34 @@ int fbcon_draw(const tinfo* ti, sprixel* s, int y, int x){
   return wrote;
 }
 
+// we have some number of (cell) rows we want to scroll. scale by cell height,
+// and cap at the total pixel height (P) for N. that means we're *losing* N
+// rows from the top. we're *moving* all remaining P-N rows to the top, and
+// we're *clearing* N rows at the bottom. every pixel is written to once, and
+// they're written in order. if we're scrolling all rows, we're clearing the
+// entire space; we always clear something (we might not always move anything).
 void fbcon_scroll(const struct ncpile* p, tinfo* ti, int rows){
-  if(ti->cellpixy < 1){
+  const int cellpxy = p->cellpxy;
+  const int cellpxx = p->cellpxx;
+  if(cellpxy < 1){
     return;
   }
-  int totalrows = ti->cellpixy * p->dimy;
-  int srows = rows * ti->cellpixy; // number of pixel rows being scrolled
+  logdebug("scrolling %d\n", rows);
+  const int rowbytes = cellpxx * p->dimx * 4;
+  const int totalrows = cellpxy * p->dimy;
+  int srows = rows * cellpxy; // number of pixel rows being scrolled
   if(srows > totalrows){
     srows = totalrows;
   }
-  int rowbytes = ti->cellpixx * p->dimx * 4;
+  // srows is the number of rows we're *losing*
   uint8_t* targ = ti->linux_fbuffer;
   uint8_t* src = ti->linux_fbuffer + srows * rowbytes;
   unsigned tocopy = rowbytes * (totalrows - srows);
-  memmove(targ, src, tocopy);
+  if(tocopy){
+    memmove(targ, src, tocopy);
+  }
   targ += tocopy;
-  memset(targ, 0, srows * rowbytes);
+  memset(targ, 0, (totalrows * rowbytes) - tocopy);
 }
 
 // each row is a contiguous set of bits, starting at the msb
@@ -311,7 +330,7 @@ shim_lower_eighths(struct console_font_op* cfo, unsigned idx, int eighths){
 // add UCS2 codepoint |w| to |map| for font idx |fidx|
 static int
 add_to_map(struct unimapdesc* map, wchar_t w, unsigned fidx){
-  logdebug("Adding mapping U+%04x -> %03u\n", w, fidx);
+  logdebug("adding mapping U+%04x -> %03u\n", w, fidx);
   struct unipair* tmp = realloc(map->entries, sizeof(*map->entries) * (map->entry_ct + 1));
   if(tmp == NULL){
     return -1;
@@ -364,12 +383,13 @@ program_line_drawing_chars(int fd, struct unimapdesc* map){
   for(size_t sidx = 0 ; sidx < sizeof(sets) / sizeof(*sets) ; ++sidx){
     int fontidx = -1;
     struct simset* s = &sets[sidx];
-    bool found[wcslen(s->ws)];
-    memset(found, 0, sizeof(found));
+    size_t fsize = sizeof(bool) * wcslen(s->ws);
+    bool* found = malloc(fsize);
+    memset(found, 0, fsize);
     for(unsigned idx = 0 ; idx < map->entry_ct ; ++idx){
       for(size_t widx = 0 ; widx < wcslen(s->ws) ; ++widx){
         if(map->entries[idx].unicode == s->ws[widx]){
-          logtrace("Found desired character U+%04x -> %03u\n",
+          logtrace("found desired character U+%04x -> %03u\n",
                    map->entries[idx].unicode, map->entries[idx].fontpos);
           found[widx] = true;
           if(fontidx == -1){
@@ -382,14 +402,16 @@ program_line_drawing_chars(int fd, struct unimapdesc* map){
       for(size_t widx = 0 ; widx < wcslen(s->ws) ; ++widx){
         if(!found[widx]){
           if(add_to_map(map, s->ws[widx], fontidx)){
+            free(found);
             return -1;
           }
           ++toadd;
         }
       }
     }else{
-      logwarn("Couldn't find any glyphs for set %zu\n", sidx);
+      logwarn("couldn't find any glyphs for set %zu\n", sidx);
     }
+    free(found);
   }
   if(toadd == 0){
     return 0;
@@ -398,7 +420,7 @@ program_line_drawing_chars(int fd, struct unimapdesc* map){
     logwarn("error setting kernel unicode map (%s)\n", strerror(errno));
     return -1;
   }
-  loginfo("Successfully added %d kernel unicode mapping%s\n",
+  loginfo("added %d kernel unicode mapping%s\n",
           toadd, toadd == 1 ? "" : "s");
   return 0;
 }
@@ -428,7 +450,7 @@ copy_and_close_linux_fb(tinfo* ti, struct framebuffer_copy* fbdup){
 
 static void
 kill_fbcopy(struct framebuffer_copy* fbdup){
-  free(fbdup->map);
+  free(fbdup->map); // just a memdup(), not an actual mmap
 }
 
 static int
@@ -474,7 +496,7 @@ program_block_drawing_chars(tinfo* ti, int fd, struct console_font_op* cfo,
     if(map->entries[i].unicode >= 0x2580 && map->entries[i].unicode <= 0x259f){
       for(size_t s = 0 ; s < sizeof(half) / sizeof(*half) ; ++s){
         if(map->entries[i].unicode == half[s].w){
-          logdebug("Found %lc at fontidx %u\n", half[s].w, i);
+          logdebug("found %lc at fontidx %u\n", half[s].w, i);
           half[s].found = true;
           ++halvesfound;
           break;
@@ -482,7 +504,7 @@ program_block_drawing_chars(tinfo* ti, int fd, struct console_font_op* cfo,
       }
       for(size_t s = 0 ; s < sizeof(quads) / sizeof(*quads) ; ++s){
         if(map->entries[i].unicode == quads[s].w){
-          logdebug("Found %lc at fontidx %u\n", quads[s].w, i);
+          logdebug("found %lc at fontidx %u\n", quads[s].w, i);
           quads[s].found = true;
           ++numfound;
           break;
@@ -490,7 +512,7 @@ program_block_drawing_chars(tinfo* ti, int fd, struct console_font_op* cfo,
       }
       for(size_t s = 0 ; s < sizeof(eighths) / sizeof(*eighths) ; ++s){
         if(map->entries[i].unicode == eighths[s].w){
-          logdebug("Found %lc at fontidx %u\n", eighths[s].w, i);
+          logdebug("found %lc at fontidx %u\n", eighths[s].w, i);
           eighths[s].found = true;
           ++numfound;
           break;
@@ -605,7 +627,11 @@ program_block_drawing_chars(tinfo* ti, int fd, struct console_font_op* cfo,
     *quadrants = true;
   }
   added += halvesadded;
-  loginfo("successfully added %d kernel font glyph%s\n", added, added == 1 ? "" : "s");
+  loginfo("successfully added %d kernel font glyph%s via %d\n", added, added == 1 ? "" : "s", ti->linux_fb_fd);
+  if(ti->linux_fb_fd < 0){ // console doesn't imply framebuffer
+    kill_fbcopy(&fbdup);
+    return 0;
+  }
   unsigned pixely, pixelx;
   if(get_linux_fb_pixelgeom(ti, &pixely, &pixelx)){
     kill_fbcopy(&fbdup);
@@ -628,17 +654,17 @@ reprogram_linux_font(tinfo* ti, int fd, struct console_font_op* cfo,
     logwarn("error reading Linux kernelfont (%s)\n", strerror(errno));
     return -1;
   }
-  loginfo("Kernel font size (glyphcount): %u\n", cfo->charcount);
-  loginfo("Kernel font character geometry: %ux%u\n", cfo->width, cfo->height);
+  loginfo("kernel font size (glyphcount): %u\n", cfo->charcount);
+  loginfo("kernel font character geometry: %ux%u\n", cfo->width, cfo->height);
   if(cfo->charcount > 512){
-    logwarn("Warning: kernel returned excess charcount\n");
+    logwarn("warning: kernel returned excess charcount\n");
     return -1;
   }
   if(ioctl(fd, GIO_UNIMAP, map)){
     logwarn("error reading Linux unimap (%s)\n", strerror(errno));
     return -1;
   }
-  loginfo("Kernel Unimap size: %u/%u\n", map->entry_ct, USHRT_MAX);
+  loginfo("kernel unimap size: %u/%u\n", map->entry_ct, USHRT_MAX);
   // for certain sets of characters, we're not going to draw them in, but we
   // do want to ensure they map to something plausible...this doesn't reset
   // the framebuffer, even if we do some reprogramming.
@@ -693,7 +719,7 @@ bool is_linux_console(int fd){
   }
   int mode;
   if(ioctl(fd, KDGETMODE, &mode)){
-    logdebug("not a Linux console, KDGETMODE failed\n");
+    logdebug("not a Linux console (no KDGETMODE)\n");
     return false;
   }
   loginfo("verified Linux console, mode %d\n", mode);
@@ -710,10 +736,11 @@ int get_linux_fb_pixelgeom(tinfo* ti, unsigned* ypix, unsigned *xpix){
   }
   struct fb_var_screeninfo fbi = {};
   if(ioctl(ti->linux_fb_fd, FBIOGET_VSCREENINFO, &fbi)){
-    logwarn("no framebuffer info from %s (%s?)\n", ti->linux_fb_dev, strerror(errno));
+    logerror("no framebuffer info from %s %d (%s?)\n", ti->linux_fb_dev,
+             ti->linux_fb_fd, strerror(errno));
     return -1;
   }
-  loginfo("Linux %s geometry: %dx%d\n", ti->linux_fb_dev, fbi.yres, fbi.xres);
+  loginfo("linux %s geometry: %dx%d\n", ti->linux_fb_dev, fbi.yres, fbi.xres);
   *ypix = fbi.yres;
   *xpix = fbi.xres;
   size_t len = *ypix * *xpix * fbi.bits_per_pixel / 8;
@@ -726,11 +753,11 @@ int get_linux_fb_pixelgeom(tinfo* ti, unsigned* ypix, unsigned *xpix){
     ti->linux_fbuffer = mmap(NULL, len, PROT_READ|PROT_WRITE,
                              MAP_SHARED, ti->linux_fb_fd, 0);
     if(ti->linux_fbuffer == MAP_FAILED){
-      logerror("Couldn't map %zuB on %s (%s?)\n", len, ti->linux_fb_dev, strerror(errno));
+      logerror("couldn't map %zuB on %s (%s?)\n", len, ti->linux_fb_dev, strerror(errno));
       return -1;
     }
     ti->linux_fb_len = len;
-    loginfo("Mapped %zuB on %s\n", len, ti->linux_fb_dev);
+    loginfo("mapped %zuB on %s\n", len, ti->linux_fb_dev);
   }
   return 0;
 }
@@ -739,10 +766,10 @@ bool is_linux_framebuffer(tinfo* ti){
   // FIXME there might be multiple framebuffers present; how do we determine
   // which one is ours?
   const char* dev = "/dev/fb0";
-  loginfo("Checking for Linux framebuffer at %s\n", dev);
+  loginfo("checking for Linux framebuffer at %s\n", dev);
   int fd = open(dev, O_RDWR | O_CLOEXEC);
   if(fd < 0){
-    logdebug("Couldn't open framebuffer device %s\n", dev);
+    logdebug("couldn't open framebuffer device %s\n", dev);
     return false;
   }
   ti->linux_fb_fd = fd;
