@@ -4,12 +4,14 @@
 #include <notcurses/direct.h>
 #include "internal.h"
 
+sig_atomic_t sigcont_seen_for_render = 0;
+
 // update for a new visual area of |rows|x|cols|, neither of which may be zero.
 // copies that area of the lastframe (damage map) which is shared between the
 // two. new areas are initialized to empty, just like a new plane. lost areas
 // have their egcpool entries purged.
 static nccell*
-restripe_lastframe(notcurses* nc, int rows, int cols){
+restripe_lastframe(notcurses* nc, unsigned rows, unsigned cols){
   assert(rows);
   assert(cols);
   const size_t size = sizeof(*nc->lastframe) * (rows * cols);
@@ -21,7 +23,7 @@ restripe_lastframe(notcurses* nc, int rows, int cols){
   size_t maxlinecopy = sizeof(nccell) * copycols;
   size_t minlineset = sizeof(nccell) * cols - maxlinecopy;
   unsigned zorch = nc->lfdimx > cols ? nc->lfdimx - cols : 0;
-  for(int y = 0 ; y < rows ; ++y){
+  for(unsigned y = 0 ; y < rows ; ++y){
     if(y < nc->lfdimy){
       if(maxlinecopy){
         memcpy(&tmp[cols * y], &nc->lastframe[nc->lfdimx * y], maxlinecopy);
@@ -40,8 +42,8 @@ restripe_lastframe(notcurses* nc, int rows, int cols){
     }
   }
   // excise any egcpool entries from below the new plane area
-  for(int y = rows ; y < nc->lfdimy ; ++y){
-    for(int x = 0 ; x < nc->lfdimx ; ++x){
+  for(unsigned y = rows ; y < nc->lfdimy ; ++y){
+    for(unsigned x = 0 ; x < nc->lfdimx ; ++x){
       pool_release(&nc->pool, &nc->lastframe[fbcellidx(y, nc->lfdimx, x)]);
     }
   }
@@ -57,9 +59,9 @@ restripe_lastframe(notcurses* nc, int rows, int cols){
 // at the same origin. Initiates a resize cascade for the pile containing |pp|.
 // The current terminal geometry, changed or not, is written to |rows|/|cols|.
 static int
-notcurses_resize_internal(ncplane* pp, int* restrict rows, int* restrict cols){
+notcurses_resize_internal(ncplane* pp, unsigned* restrict rows, unsigned* restrict cols){
   notcurses* n = ncplane_notcurses(pp);
-  int r, c;
+  unsigned r, c;
   if(rows == NULL){
     rows = &r;
   }
@@ -67,13 +69,18 @@ notcurses_resize_internal(ncplane* pp, int* restrict rows, int* restrict cols){
     cols = &c;
   }
   ncpile* pile = ncplane_pile(pp);
-  int oldrows = pile->dimy;
-  int oldcols = pile->dimx;
+  unsigned oldrows = pile->dimy;
+  unsigned oldcols = pile->dimx;
   *rows = oldrows;
   *cols = oldcols;
-  if(update_term_dimensions(rows, cols, &n->tcache, n->margin_b)){
+  unsigned cgeo_changed;
+  unsigned pgeo_changed;
+  if(update_term_dimensions(rows, cols, &n->tcache, n->margin_b,
+                            &cgeo_changed, &pgeo_changed)){
     return -1;
   }
+  n->stats.s.cell_geo_changes += cgeo_changed;
+  n->stats.s.pixel_geo_changes += pgeo_changed;
   *rows -= n->margin_t + n->margin_b;
   if(*rows <= 0){
     *rows = 1;
@@ -111,7 +118,7 @@ notcurses_resize_internal(ncplane* pp, int* restrict rows, int* restrict cols){
 
 // Check for a window resize on the standard pile.
 static int
-notcurses_resize(notcurses* n, int* restrict rows, int* restrict cols){
+notcurses_resize(notcurses* n, unsigned* restrict rows, unsigned* restrict cols){
   pthread_mutex_lock(&n->pilelock);
   int ret = notcurses_resize_internal(notcurses_stdplane(n), rows, cols);
   pthread_mutex_unlock(&n->pilelock);
@@ -122,11 +129,6 @@ void nccell_release(ncplane* n, nccell* c){
   pool_release(&n->pool, c);
 }
 
-// FIXME deprecated, goes away in abi3
-void cell_release(ncplane* n, nccell* c){
-  nccell_release(n, c);
-}
-
 // Duplicate one cell onto another when they share a plane. Convenience wrapper.
 int nccell_duplicate(ncplane* n, nccell* targ, const nccell* c){
   if(cell_duplicate_far(&n->pool, targ, n, c) < 0){
@@ -136,11 +138,6 @@ int nccell_duplicate(ncplane* n, nccell* targ, const nccell* c){
   return 0;
 }
 
-// deprecated, goes away in abi3
-int cell_duplicate(struct ncplane* n, nccell* targ, const nccell* c){
-  return nccell_duplicate(n, targ, c);
-}
-
 // Emit fchannel with RGB changed to contrast effectively against bchannel.
 static uint32_t
 highcontrast(const tinfo* ti, uint32_t bchannel){
@@ -235,17 +232,18 @@ paint_sprixel(ncplane* p, struct crender* rvec, int starty, int startx,
 // the sprixelstack orders sprixels of the plane (so we needn't keep them
 // ordered between renders). each time we meet a sprixel, extract it from
 // the pile's sprixel list, and update the sprixelstack.
-__attribute__ ((nonnull (1, 2, 7)))
-static void
+__attribute__ ((nonnull (1, 2, 7))) static void
 paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
-      int dstabsy, int dstabsx, sprixel** sprixelstack){
-  int y, x, dimy, dimx, offy, offx;
+      int dstabsy, int dstabsx, sprixel** sprixelstack,
+      unsigned pgeo_changed){
+  unsigned y, x, dimy, dimx;
+  int offy, offx;
   ncplane_dim_yx(p, &dimy, &dimx);
   offy = p->absy - dstabsy;
   offx = p->absx - dstabsx;
 //fprintf(stderr, "PLANE %p %d %d %d %d %d %d %p\n", p, dimy, dimx, offy, offx, dstleny, dstlenx, p->sprite);
   // skip content above or to the left of the physical screen
-  int starty, startx;
+  unsigned starty, startx;
   if(offy < 0){
     starty = -offy;
   }else{
@@ -260,6 +258,10 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
   // glyph, but we *do* need to null out any cellregions that we've
   // scribbled upon.
   if(p->sprite){
+    if(pgeo_changed){
+      // do what on failure? FIXME
+      sprixel_rescale(p->sprite, ncplane_pile(p)->cellpxy, ncplane_pile(p)->cellpxx);
+    }
     paint_sprixel(p, rvec, starty, startx, offy, offx, dstleny, dstlenx);
     // decouple from the pile's sixel list
     if(p->sprite->next){
@@ -296,6 +298,12 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
       if(nccell_wide_right_p(targc)){
         continue;
       }
+      // if we encounter a newline (always a bare linefeed; Windows CRLF
+      // translation only matters for text stdio), we're done with this
+      // line *for this plane*.
+      if(targc->gcluster == '\n'){
+        break;
+      }
 
       if(nccell_fg_alpha(targc) > NCALPHA_OPAQUE){
         const nccell* vis = &p->fb[nfbcellidx(p, y, x)];
@@ -472,14 +480,13 @@ postpaint_cell(const tinfo* ti, nccell* lastframe, int dimx,
       // we don't need to change it when under an opaque cell, because
       // that's always printed on top.
       if(!crender->s.p_beats_sprixel){
-        // leaving out SPRIXCELL_OPAQUE_KITTY here results in cruft
         if(state != SPRIXCELL_OPAQUE_SIXEL){
 //fprintf(stderr, "damaged due to opaque %d/%d\n", y, *x);
           crender->s.damaged = 1;
         }
       }
     }else{
-//fprintf(stderr, "damaged due to opaque else %d/%d\n", y, *x);
+//fprintf(stderr, "damaged due to lastframe disagree %d/%d\n", y, *x);
       crender->s.damaged = 1;
     }
     assert(!nccell_wide_right_p(targc));
@@ -526,31 +533,71 @@ postpaint(const tinfo* ti, nccell* lastframe, int dimy, int dimx,
 // merging one plane down onto another is basically just performing a render
 // using only these two planes, with the result written to the lower plane.
 int ncplane_mergedown(ncplane* restrict src, ncplane* restrict dst,
-                      int begsrcy, int begsrcx, int leny, int lenx,
+                      int begsrcy, int begsrcx, unsigned leny, unsigned lenx,
                       int dsty, int dstx){
 //fprintf(stderr, "Merging down %d/%d @ %d/%d to %d/%d\n", leny, lenx, begsrcy, begsrcx, dsty, dstx);
-  if(dsty >= dst->leny || dstx >= dst->lenx){
-    logerror("Dest origin %d/%d ≥ dest dimensions %d/%d\n",
+  if(dsty < 0){
+    if(dsty != -1){
+      logerror("invalid dsty %d\n", dsty);
+      return -1;
+    }
+    dsty = dst->y;
+  }
+  if(dstx < 0){
+    if(dstx != -1){
+      logerror("invalid dstx %d\n", dstx);
+      return -1;
+    }
+    dstx = dst->x;
+  }
+  if((unsigned)dsty >= dst->leny || (unsigned)dstx >= dst->lenx){
+    logerror("dest origin %u/%u ≥ dest dimensions %d/%d\n",
              dsty, dstx, dst->leny, dst->lenx);
     return -1;
   }
-  if(dst->leny - leny < dsty || dst->lenx - lenx < dstx){
-    logerror("Dest len %d/%d ≥ dest dimensions %d/%d\n",
-             leny, lenx, dst->leny, dst->lenx);
-    return -1;
+  if(begsrcy < 0){
+    if(begsrcy != -1){
+      logerror("invalid begsrcy %d\n", begsrcy);
+      return -1;
+    }
+    begsrcy = src->y;
   }
-  if(begsrcy >= src->leny || begsrcx >= src->lenx){
-    logerror("Source origin %d/%d ≥ source dimensions %d/%d\n",
+  if(begsrcx < 0){
+    if(begsrcx != -1){
+      logerror("invalid begsrcx %d\n", begsrcx);
+      return -1;
+    }
+    begsrcx = src->x;
+  }
+  if((unsigned)begsrcy >= src->leny || (unsigned)begsrcx >= src->lenx){
+    logerror("source origin %u/%u ≥ source dimensions %d/%d\n",
              begsrcy, begsrcx, src->leny, src->lenx);
     return -1;
   }
-  if(src->leny - leny < begsrcy || src->lenx - lenx < begsrcx){
-    logerror("Source len %d/%d ≥ source dimensions %d/%d\n",
+  if(leny == 0){
+    if((leny = src->leny - begsrcy) == 0){
+      logerror("source area was zero height\n");
+      return -1;
+    }
+  }
+  if(lenx == 0){
+    if((lenx = src->lenx - begsrcx) == 0){
+      logerror("source area was zero width\n");
+      return -1;
+    }
+  }
+  if(dst->leny - leny < (unsigned)dsty || dst->lenx - lenx < (unsigned)dstx){
+    logerror("dest len %u/%u ≥ dest dimensions %d/%d\n",
+             leny, lenx, dst->leny, dst->lenx);
+    return -1;
+  }
+  if(src->leny - leny < (unsigned)begsrcy || src->lenx - lenx < (unsigned)begsrcx){
+    logerror("source len %u/%u ≥ source dimensions %d/%d\n",
              leny, lenx, src->leny, src->lenx);
     return -1;
   }
   if(src->sprite || dst->sprite){
-    logerror("Can't merge sprixel planes\n");
+    logerror("can't merge sprixel planes\n");
     return -1;
   }
   const int totalcells = dst->leny * dst->lenx;
@@ -558,16 +605,16 @@ int ncplane_mergedown(ncplane* restrict src, ncplane* restrict dst,
   const size_t crenderlen = sizeof(struct crender) * totalcells;
   struct crender* rvec = malloc(crenderlen);
   if(!rendfb || !rvec){
-    logerror("Error allocating render state for %dx%d\n", leny, lenx);
+    logerror("error allocating render state for %ux%u\n", leny, lenx);
     free(rendfb);
     free(rvec);
     return -1;
   }
   init_rvec(rvec, totalcells);
   sprixel* s = NULL;
-  paint(src, rvec, dst->leny, dst->lenx, dst->absy, dst->absx, &s);
+  paint(src, rvec, dst->leny, dst->lenx, dst->absy, dst->absx, &s, 0);
   assert(NULL == s);
-  paint(dst, rvec, dst->leny, dst->lenx, dst->absy, dst->absx, &s);
+  paint(dst, rvec, dst->leny, dst->lenx, dst->absy, dst->absx, &s, 0);
   assert(NULL == s);
 //fprintf(stderr, "Postpaint start (%dx%d)\n", dst->leny, dst->lenx);
   const struct tinfo* ti = &ncplane_notcurses_const(dst)->tcache;
@@ -580,23 +627,17 @@ int ncplane_mergedown(ncplane* restrict src, ncplane* restrict dst,
 }
 
 int ncplane_mergedown_simple(ncplane* restrict src, ncplane* restrict dst){
-  // have to check dst, since we used to accept a NULL dst to mean the
-  // standard plane (this was unsafe, since src might be in another pile).
-  if(dst == NULL){
-    return -1;
-  }
-  int dimy, dimx;
-  ncplane_dim_yx(dst, &dimy, &dimx);
-  return ncplane_mergedown(src, dst, 0, 0, ncplane_dim_y(src), ncplane_dim_x(src), 0, 0);
+  return ncplane_mergedown(src, dst, 0, 0, 0, 0, 0, 0);
 }
 
-// write the nccell's UTF-8 extended grapheme cluster to the provided FILE*.
-static int
+// write the nccell's UTF-8 extended grapheme cluster to the provided fbuf.
+static inline int
 term_putc(fbuf* f, const egcpool* e, const nccell* c){
   if(cell_simple_p(c)){
 //fprintf(stderr, "[%.4s] %08x\n", (const char*)&c->gcluster, c->gcluster); }
-    // we must not have any 'cntrl' characters at this point
-    if(c->gcluster == 0){
+    // we must not have any 'cntrl' characters at this point, except for
+    // nil or newline
+    if(c->gcluster == 0 || c->gcluster == '\n'){
       if(fbuf_putc(f, ' ') < 0){
         return -1;
       }
@@ -900,8 +941,7 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
         }else{
           if(s->n->absx == s->movedfromx){
             if(s->movedfromy - s->n->absy == scrolls){
-              s->invalidated = SPRIXEL_QUIESCENT; // FIXME might need to return to INVALIDATED?
-              loginfo("sprixel was scrolled %d, no redraw\n", scrolls);
+              s->invalidated = SPRIXEL_INVALIDATED;
               continue;
             }
           }
@@ -925,14 +965,14 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
 
 // scroll the lastframe data |rows| up, to reflect scrolling reality
 static void
-scroll_lastframe(notcurses* nc, int rows){
+scroll_lastframe(notcurses* nc, unsigned rows){
   // the top |rows| rows need be released (though not more than the actual
   // number of rows!)
   if(rows > nc->lfdimy){
     rows = nc->lfdimy;
   }
-  for(int targy = 0 ; targy < rows ; ++targy){
-    for(int targx = 0 ; targx < nc->lfdimx ; ++targx){
+  for(unsigned targy = 0 ; targy < rows ; ++targy){
+    for(unsigned targx = 0 ; targx < nc->lfdimx ; ++targx){
       const size_t damageidx = targy * nc->lfdimx + targx;
       nccell* c = &nc->lastframe[damageidx];
       pool_release(&nc->pool, c);
@@ -941,7 +981,7 @@ scroll_lastframe(notcurses* nc, int rows){
   // now for all rows subsequent, up through lfdimy - rows, move them back.
   // if we scrolled all rows, we will not move anything (and we just
   // released everything).
-  for(int targy = 0 ; targy < nc->lfdimy - rows ; ++targy){
+  for(unsigned targy = 0 ; targy < nc->lfdimy - rows ; ++targy){
     const size_t dstidx = targy * nc->lfdimx;
     nccell* dst = &nc->lastframe[dstidx];
     const size_t srcidx = dstidx + rows * nc->lfdimx;
@@ -949,7 +989,7 @@ scroll_lastframe(notcurses* nc, int rows){
     memcpy(dst, src, sizeof(*dst) * nc->lfdimx);
   }
   // now for the last |rows| rows, initialize them to 0.
-  int targy = nc->lfdimy - rows;
+  unsigned targy = nc->lfdimy - rows;
   while(targy < nc->lfdimy){
     const size_t dstidx = targy * nc->lfdimx;
     nccell* dst = &nc->lastframe[dstidx];
@@ -966,17 +1006,18 @@ rasterize_scrolls(const ncpile* p, fbuf* f){
     return 0;
   }
   logdebug("order-%d scroll\n", scrolls);
-  if(p->nc->rstate.logendy >= 0){
+  /*if(p->nc->rstate.logendy >= 0){
     p->nc->rstate.logendy -= scrolls;
     if(p->nc->rstate.logendy < 0){
       p->nc->rstate.logendy = 0;
       p->nc->rstate.logendx = 0;
     }
-  }
+  }*/
+  // FIXME probably need this to take place at the end of cycle...
   if(p->nc->tcache.pixel_scroll){
     p->nc->tcache.pixel_scroll(p, &p->nc->tcache, scrolls);
   }
-  if(goto_location(p->nc, f, p->dimy, 0)){
+  if(goto_location(p->nc, f, p->dimy, 0, NULL)){
     return -1;
   }
   // terminals advertising 'bce' will scroll in the current background color;
@@ -986,7 +1027,7 @@ rasterize_scrolls(const ncpile* p, fbuf* f){
       return -1;
     }
   }
-  return emit_scrolls(&p->nc->tcache, scrolls, f);
+  return emit_scrolls_track(p->nc, scrolls, f);
 }
 
 // second sprixel pass in rasterization. by this time, all sixels are handled
@@ -1004,7 +1045,7 @@ rasterize_sprixels(notcurses* nc, ncpile* p, fbuf* f){
   sprixel* s;
   sprixel** parent = &p->sprixelcache;
   while( (s = *parent) ){
-//fprintf(stderr, "YARR HARR HARR SPIRXLE %u STATE %d\n", s->id, s->invalidated);
+//fprintf(stderr, "raster YARR HARR HARR SPIRXLE %u STATE %d\n", s->id, s->invalidated);
     if(s->invalidated == SPRIXEL_INVALIDATED){
 //fprintf(stderr, "3 DRAWING BITMAP %d STATE %d AT %d/%d for %p\n", s->id, s->invalidated, nc->margin_t, nc->margin_l, s->n);
       int r = sprite_draw(&nc->tcache, p, s, f, nc->margin_t, nc->margin_l);
@@ -1012,19 +1053,21 @@ rasterize_sprixels(notcurses* nc, ncpile* p, fbuf* f){
         return -1;
       }else if(r > 0){
         bytesemitted += r;
-        nc->rstate.hardcursorpos = true;
+        nc->rstate.y = -1;
+        nc->rstate.x = -1;
       }
     }else if(s->invalidated == SPRIXEL_LOADED){
       if(nc->tcache.pixel_commit){
         int y, x;
         ncplane_abs_yx(s->n, &y, &x);
-        if(goto_location(nc, f, y + nc->margin_t, x + nc->margin_l)){
+        if(goto_location(nc, f, y + nc->margin_t, x + nc->margin_l, NULL)){
           return -1;
         }
         if(sprite_commit(&nc->tcache, f, s, false)){
           return -1;
         }
-        nc->rstate.hardcursorpos = true;
+        nc->rstate.y = -1;
+        nc->rstate.x = -1;
       }
     }else if(s->invalidated == SPRIXEL_HIDE){
       if(nc->tcache.pixel_remove){
@@ -1087,13 +1130,14 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
   struct crender* rvec = p->crender;
   // we only need to emit a coordinate if it was damaged. the damagemap is a
   // bit per coordinate, one per struct crender.
-  for(int y = nc->margin_t; y < p->dimy + nc->margin_t ; ++y){
+  for(unsigned y = nc->margin_t; y < p->dimy + nc->margin_t ; ++y){
     const int innery = y - nc->margin_t;
-    for(int x = nc->margin_l ; x < p->dimx + nc->margin_l ; ++x){
+    bool saw_linefeed = 0;
+    for(unsigned x = nc->margin_l ; x < p->dimx + nc->margin_l ; ++x){
       const int innerx = x - nc->margin_l;
       const size_t damageidx = innery * nc->lfdimx + innerx;
       unsigned r, g, b, br, bg, bb;
-      const nccell* srccell = &nc->lastframe[damageidx];
+      nccell* srccell = &nc->lastframe[damageidx];
       if(!rvec[damageidx].s.damaged){
         // no need to emit a cell; what we rendered appears to already be
         // here. no updates are performed to elision state nor lastframe.
@@ -1107,7 +1151,7 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
         // was not above a sprixel (and the cell is damaged). in the second
         // phase, we draw everything that remains damaged.
         ++nc->stats.s.cellemissions;
-        if(goto_location(nc, f, y, x)){
+        if(goto_location(nc, f, y, x, rvec[damageidx].p)){
           return -1;
         }
         // set the style. this can change the color back to the default; if it
@@ -1120,40 +1164,41 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
         //  * we are a partial glyph, and the previous was default on both, or
         //  * we are a no-foreground glyph, and the previous was default background, or
         //  * we are a no-background glyph, and the previous was default foreground
-        bool nobackground = cell_nobackground_p(srccell);
+        bool nobackground = nccell_nobackground_p(srccell);
+        bool rgbequal = nccell_rgbequal_p(srccell);
         if((nccell_fg_default_p(srccell)) || (!nobackground && nccell_bg_default_p(srccell))){
           if(raster_defaults(nc, nccell_fg_default_p(srccell),
                              !nobackground && nccell_bg_default_p(srccell), f)){
             return -1;
           }
         }
-        // if our cell has a non-default foreground, we can elide the
-        // non-default foreground set iff either:
-        //  * the previous was non-default, and matches what we have now, or
-        //  * we are a no-foreground glyph (iswspace() is true)
         if(nccell_fg_palindex_p(srccell)){ // palette-indexed foreground
           if(emit_fg_palindex(nc, f, srccell)){
             return -1;
           }
         }else if(!nccell_fg_default_p(srccell)){ // rgb foreground
+          // if our cell has a non-default foreground, we can elide the
+          // non-default foreground set iff either:
+          //  * the previous was non-default, and matches what we have now, or
+          //  * we are a no-foreground glyph (iswspace() is true) FIXME
           nccell_fg_rgb8(srccell, &r, &g, &b);
           if(nc->rstate.fgelidable && nc->rstate.lastr == r && nc->rstate.lastg == g && nc->rstate.lastb == b){
             ++nc->stats.s.fgelisions;
           }else{
-            if(term_fg_rgb8(&nc->tcache, f, r, g, b)){
-              return -1;
+            if(!rgbequal){ // if rgbequal, no need to set fg
+              if(term_fg_rgb8(&nc->tcache, f, r, g, b)){
+                return -1;
+              }
+              ++nc->stats.s.fgemissions;
+              nc->rstate.fgelidable = true;
+            }else{
+              r = nc->rstate.lastr; g = nc->rstate.lastg; b = nc->rstate.lastb;
             }
-            ++nc->stats.s.fgemissions;
-            nc->rstate.fgelidable = true;
           }
           nc->rstate.lastr = r; nc->rstate.lastg = g; nc->rstate.lastb = b;
           nc->rstate.fgdefelidable = false;
           nc->rstate.fgpalelidable = false;
         }
-        // if our cell has a non-default background, we can elide the
-        // non-default background set iff either:
-        //  * we do not use the background, because the cell is all-foreground,
-        //  * the previous was non-default, and matches what we have now, or
         if(nobackground){
           ++nc->stats.s.bgelisions;
         }else if(nccell_bg_palindex_p(srccell)){ // palette-indexed background
@@ -1161,6 +1206,10 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
             return -1;
           }
         }else if(!nccell_bg_default_p(srccell)){ // rgb background
+          // if our cell has a non-default background, we can elide the
+          // non-default background set iff either:
+          //  * we do not use the background, because the cell is all-foreground,
+          //  * the previous was non-default, and matches what we have now, or
           nccell_bg_rgb8(srccell, &br, &bg, &bb);
           if(nc->rstate.bgelidable && nc->rstate.lastbr == br && nc->rstate.lastbg == bg && nc->rstate.lastbb == bb){
             ++nc->stats.s.bgelisions;
@@ -1174,8 +1223,14 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
           nc->rstate.lastbr = br; nc->rstate.lastbg = bg; nc->rstate.lastbb = bb;
           nc->rstate.bgdefelidable = false;
           nc->rstate.bgpalelidable = false;
+          // FIXME see above -- if we don't have to change the fg, but do need
+          // to change the bg, emit full block and no rgb at all
+          if(rgbequal){
+            // FIXME need one per column of original glyph
+            pool_load_direct(&nc->pool, srccell, " ", 1, 1);
+          }
         }
-//fprintf(stderr, "RAST %08x [%s] to %d/%d cols: %u %016lx\n", srccell->gcluster, pool_extended_gcluster(&nc->pool, srccell), y, x, srccell->width, srccell->channels);
+//fprintf(stderr, "RAST %08x [%s] to %d/%d cols: %u %016" PRIx64 "\n", srccell->gcluster, pool_extended_gcluster(&nc->pool, srccell), y, x, srccell->width, srccell->channels);
         // this is used to invalidate the sprixel in the first text round,
         // which is only necessary for sixel, not kitty.
         if(rvec[damageidx].sprixel){
@@ -1189,6 +1244,9 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
         if(term_putc(f, &nc->pool, srccell)){
           return -1;
         }
+        if(srccell->gcluster == '\n'){
+          saw_linefeed = true;
+        }
         rvec[damageidx].s.damaged = 0;
         rvec[damageidx].s.p_beats_sprixel = 0;
         nc->rstate.x += srccell->width;
@@ -1197,9 +1255,31 @@ rasterize_core(notcurses* nc, const ncpile* p, fbuf* f, unsigned phase){
         }else{
           ++nc->rstate.x;
         }
+        if((int)y > nc->rstate.logendy || ((int)y == nc->rstate.logendy && (int)x > nc->rstate.logendx)){
+          if((int)y > nc->rstate.logendy){
+//fprintf(stderr, "**************8NATURAL PLACEMENT AT %u/ %u\n", y, x);
+            nc->rstate.logendy = y;
+            nc->rstate.logendx = 0;
+          }
+          if(x >= p->dimx + nc->margin_l - 1){
+            if(nc->rstate.logendy < (int)(p->dimy + nc->margin_t - 1)){
+              ++nc->rstate.logendy;
+            }
+            nc->rstate.logendx = 0;
+            saw_linefeed = false;
+//fprintf(stderr, "**********8SCROLLING PLACEMENT AT %u %u\n", nc->rstate.logendy, nc->rstate.logendx);
+          }else if((int)x >= nc->rstate.logendx){
+            nc->rstate.logendx = x;
+//fprintf(stderr, "**********HORIZ PLACEMENT AT %u %u\n", nc->rstate.logendy, nc->rstate.logendx);
+          }
+        }
       }
 //fprintf(stderr, "damageidx: %ld\n", damageidx);
     }
+    // shouldn't this happen only if the linefeed was on the last line? FIXME
+    if(saw_linefeed){
+      nc->rstate.logendx = 0;
+    }
   }
   return 0;
 }
@@ -1322,7 +1402,7 @@ notcurses_rasterize(notcurses* nc, ncpile* p, fbuf* f){
   if(cursory >= 0){
     notcurses_cursor_enable(nc, cursory, cursorx);
   }else if(nc->rstate.logendy >= 0){
-    goto_location(nc, f, nc->rstate.logendy, nc->rstate.logendx);
+    goto_location(nc, f, nc->rstate.logendy, nc->rstate.logendx, nc->rstate.lastsrcp);
     if(fbuf_flush(f, nc->ttyfp)){
       ret = -1;
     }
@@ -1338,32 +1418,20 @@ int clear_and_home(notcurses* nc, tinfo* ti, fbuf* f){
   const char* clearscr = get_escape(ti, ESCAPE_CLEAR);
   if(clearscr){
     if(fbuf_emit(f, clearscr) == 0){
-      goto success;
+      nc->rstate.x = nc->rstate.y = 0;
+      return 0;
     }
   }
-  const ncplane* stdn = notcurses_stdplane_const(nc);
-  // clearscr didn't fly. try scrolling everything off. first, go to the
-  // bottom of the screen, then write N newlines.
-  if(goto_location(nc, f, ncplane_dim_y(stdn) - 1, 0)){
+  if(emit_scrolls_track(nc, ncplane_dim_y(notcurses_stdplane_const(nc)), f)){
     return -1;
   }
-  for(int y = 0 ; y < ncplane_dim_y(stdn) ; ++y){
-    if(fbuf_putc(f, '\n') < 0){
-      return -1;
-    }
-  }
-  if(goto_location(nc, f, 0, 0)){
+  if(goto_location(nc, f, 0, 0, NULL)){
     return -1;
   }
-
-success:
-  nc->rstate.x = 0;
-  nc->rstate.y = 0;
   return 0;
 }
 
-// FIXME need to work with the most recently-rendered pile, no?
-int notcurses_refresh(notcurses* nc, int* restrict dimy, int* restrict dimx){
+int notcurses_refresh(notcurses* nc, unsigned* restrict dimy, unsigned* restrict dimx){
   if(notcurses_resize(nc, dimy, dimx)){
     return -1;
   }
@@ -1371,6 +1439,9 @@ int notcurses_refresh(notcurses* nc, int* restrict dimy, int* restrict dimx){
   if(clear_and_home(nc, &nc->tcache, &nc->rstate.f)){
     return -1;
   }
+  if(fbuf_flush(&nc->rstate.f, nc->ttyfp)){
+    return -1;
+  }
   if(nc->lfdimx == 0 || nc->lfdimy == 0){
     return 0;
   }
@@ -1405,15 +1476,15 @@ int ncpile_render_to_file(ncplane* n, FILE* fp){
   if(fbuf_init(&f)){
     return -1;
   }
-  const int count = (nc->lfdimx > p->dimx ? nc->lfdimx : p->dimx) *
-                    (nc->lfdimy > p->dimy ? nc->lfdimy : p->dimy);
+  const unsigned count = (nc->lfdimx > p->dimx ? nc->lfdimx : p->dimx) *
+                         (nc->lfdimy > p->dimy ? nc->lfdimy : p->dimy);
   p->crender = malloc(count * sizeof(*p->crender));
   if(p->crender == NULL){
     fbuf_free(&f);
     return -1;
   }
   init_rvec(p->crender, count);
-  for(int i = 0 ; i < count ; ++i){
+  for(unsigned i = 0 ; i < count ; ++i){
     p->crender[i].s.damaged = 1;
   }
   int ret = raster_and_write(nc, p, &f);
@@ -1429,23 +1500,22 @@ int ncpile_render_to_file(ncplane* n, FILE* fp){
   return ret;
 }
 
-int notcurses_render_to_file(notcurses* nc, FILE* fp){
-  return ncpile_render_to_file(notcurses_stdplane(nc), fp);
-}
-
 // We execute the painter's algorithm, starting from our topmost plane. The
 // damagevector should be all zeros on input. On success, it will reflect
 // which cells were changed. We solve for each coordinate's cell by walking
 // down the z-buffer, looking at intersections with ncplanes. This implies
 // locking down the EGC, the attributes, and the channels for each cell.
+// if |pgeo_changed|, the cell-pixel geometry for the pile has changed
+// since the last render, and thus all sprixels need be rescaled.
 static void
-ncpile_render_internal(ncplane* n, struct crender* rvec, int leny, int lenx){
+ncpile_render_internal(ncplane* n, struct crender* rvec, int leny, int lenx,
+                       unsigned pgeo_changed){
 //fprintf(stderr, "rendering %dx%d\n", leny, lenx);
   ncpile* np = ncplane_pile(n);
   ncplane* p = np->top;
   sprixel* sprixel_list = NULL;
   while(p){
-    paint(p, rvec, leny, lenx, 0, 0, &sprixel_list);
+    paint(p, rvec, leny, lenx, 0, 0, &sprixel_list, pgeo_changed);
     p = p->below;
   }
   if(sprixel_list){
@@ -1463,6 +1533,10 @@ ncpile_render_internal(ncplane* n, struct crender* rvec, int leny, int lenx){
 }
 
 int ncpile_rasterize(ncplane* n){
+  if(sigcont_seen_for_render){
+    sigcont_seen_for_render = 0;
+    notcurses_refresh(ncplane_notcurses(n), NULL, NULL);
+  }
   struct timespec start, rasterdone, writedone;
   clock_gettime(CLOCK_MONOTONIC, &start);
   ncpile* pile = ncplane_pile(n);
@@ -1476,7 +1550,7 @@ int ncpile_rasterize(ncplane* n){
   // accepts -1 as an indication of failure
   clock_gettime(CLOCK_MONOTONIC, &writedone);
   pthread_mutex_lock(&nc->stats.lock);
-    update_render_bytes(&nc->stats.s, bytes);
+    update_raster_bytes(&nc->stats.s, bytes);
     update_raster_stats(&rasterdone, &start, &nc->stats.s);
     update_write_stats(&writedone, &rasterdone, &nc->stats.s, bytes);
   pthread_mutex_unlock(&nc->stats.lock);
@@ -1515,11 +1589,17 @@ int ncpile_render(ncplane* n){
   notcurses* nc = ncplane_notcurses(n);
   ncpile* pile = ncplane_pile(n);
   // update our notion of screen geometry, and render against that
+  unsigned pgeo_changed = 0;
   notcurses_resize_internal(n, NULL, NULL);
+  if(pile->cellpxy != nc->tcache.cellpxy || pile->cellpxx != nc->tcache.cellpxx){
+    pile->cellpxy = nc->tcache.cellpxy;
+    pile->cellpxx = nc->tcache.cellpxx;
+    pgeo_changed = 1;
+  }
   if(engorge_crender_vector(pile)){
     return -1;
   }
-  ncpile_render_internal(n, pile->crender, pile->dimy, pile->dimx);
+  ncpile_render_internal(n, pile->crender, pile->dimy, pile->dimx, pgeo_changed);
   clock_gettime(CLOCK_MONOTONIC, &renderdone);
   pthread_mutex_lock(&nc->stats.lock);
     update_render_stats(&renderdone, &start, &nc->stats.s);
@@ -1527,21 +1607,7 @@ int ncpile_render(ncplane* n){
   return 0;
 }
 
-int notcurses_render(notcurses* nc){
-//fprintf(stderr, "--------------- BEGIN RENDER\n");
-//notcurses_debug(nc, stderr);
-  ncplane* stdn = notcurses_stdplane(nc);
-  if(ncpile_render(stdn)){
-    return -1;
-  }
-  int i = ncpile_rasterize(stdn);
-//fprintf(stderr, "----------------- END RENDER\n");
-  return i;
-}
-
-// for now, we just run the top half of notcurses_render(), and copy out the
-// memstream from within rstate. we want to allocate our own here, and return
-// it, to avoid the copy, but we need feed the params through to do so FIXME.
+// run the top half of notcurses_render(), and steal the buffer from rstate.
 int ncpile_render_to_buffer(ncplane* p, char** buf, size_t* buflen){
   if(ncpile_render(p)){
     return -1;
@@ -1551,23 +1617,17 @@ int ncpile_render_to_buffer(ncplane* p, char** buf, size_t* buflen){
   fbuf_reset(&nc->rstate.f);
   int bytes = notcurses_rasterize_inner(nc, ncplane_pile(p), &nc->rstate.f, &useasu);
   pthread_mutex_lock(&nc->stats.lock);
-    update_render_bytes(&nc->stats.s, bytes);
+    update_raster_bytes(&nc->stats.s, bytes);
   pthread_mutex_unlock(&nc->stats.lock);
   if(bytes < 0){
     return -1;
   }
-  *buf = memdup(nc->rstate.f.buf, nc->rstate.f.used);
-  if(buf == NULL){
-    return -1;
-  }
+  *buf = nc->rstate.f.buf;
   *buflen = nc->rstate.f.used;
+  fbuf_reset(&nc->rstate.f);
   return 0;
 }
 
-int notcurses_render_to_buffer(notcurses* nc, char** buf, size_t* buflen){
-  return ncpile_render_to_buffer(notcurses_stdplane(nc), buf, buflen);
-}
-
 // copy the UTF8-encoded EGC out of the cell, whether simple or complex. the
 // result is not tied to the ncplane, and persists across erases / destruction.
 static inline char*
@@ -1578,27 +1638,31 @@ pool_egc_copy(const egcpool* e, const nccell* c){
   return strdup(egcpool_extended_gcluster(e, c));
 }
 
-char* notcurses_at_yx(notcurses* nc, int yoff, int xoff, uint16_t* stylemask, uint64_t* channels){
-  char* egc = NULL;
-  if(nc->lastframe){
-    if(yoff >= 0 && yoff < nc->lfdimy){
-      if(xoff >= 0 || xoff < nc->lfdimx){
-        const nccell* srccell = &nc->lastframe[yoff * nc->lfdimx + xoff];
-        if(nccell_wide_right_p(srccell)){
-          return notcurses_at_yx(nc, yoff, xoff - 1, stylemask, channels);
-        }
-        if(stylemask){
-          *stylemask = srccell->stylemask;
-        }
-        if(channels){
-          *channels = srccell->channels;
-        }
-//fprintf(stderr, "COPYING: %d from %p\n", srccell->gcluster, &nc->pool);
-        egc = pool_egc_copy(&nc->pool, srccell);
-      }
-    }
+char* notcurses_at_yx(notcurses* nc, unsigned yoff, unsigned xoff, uint16_t* stylemask, uint64_t* channels){
+  if(nc->lastframe == NULL){
+    logerror("haven't yet rendered\n");
+    return NULL;
   }
-  return egc;
+  if(yoff >= nc->lfdimy){
+    logerror("invalid coordinates: %u/%u\n", yoff, xoff);
+    return NULL;
+  }
+  if(xoff >= nc->lfdimx){
+    logerror("invalid coordinates: %u/%u\n", yoff, xoff);
+    return NULL;
+  }
+  const nccell* srccell = &nc->lastframe[yoff * nc->lfdimx + xoff];
+  if(nccell_wide_right_p(srccell)){
+    return notcurses_at_yx(nc, yoff, xoff - 1, stylemask, channels);
+  }
+  if(stylemask){
+    *stylemask = srccell->stylemask;
+  }
+  if(channels){
+    *channels = srccell->channels;
+  }
+//fprintf(stderr, "COPYING: %d from %p\n", srccell->gcluster, &nc->pool);
+  return pool_egc_copy(&nc->pool, srccell);
 }
 
 int ncdirect_set_bg_rgb_f(ncdirect* nc, unsigned rgb, fbuf* f){
@@ -1682,7 +1746,7 @@ int notcurses_cursor_enable(notcurses* nc, int y, int x){
     return -1;
   }
   // updates nc->rstate.cursor{y,x}
-  if(goto_location(nc, &f, y + nc->margin_t, x + nc->margin_l)){
+  if(goto_location(nc, &f, y + nc->margin_t, x + nc->margin_l, nc->rstate.lastsrcp)){
     fbuf_free(&f);
     return -1;
   }
