@@ -19,16 +19,16 @@ typedef struct ncselector {
   unsigned selected;             // index of selection
   unsigned startdisp;            // index of first option displayed
   unsigned maxdisplay;           // max number of items to display, 0 -> no limit
-  int longop;                    // columns occupied by longest option
-  int longdesc;                  // columns occupied by longest description
+  unsigned longop;               // columns occupied by longest option
+  unsigned longdesc;             // columns occupied by longest description
   struct ncselector_int* items;  // list of items and descriptions, heap-copied
   unsigned itemcount;            // number of pairs in 'items'
   char* title;                   // can be NULL, in which case there's no riser
-  int titlecols;                 // columns occupied by title
+  unsigned titlecols;            // columns occupied by title
   char* secondary;               // can be NULL
-  int secondarycols;             // columns occupied by secondary
+  unsigned secondarycols;        // columns occupied by secondary
   char* footer;                  // can be NULL
-  int footercols;                // columns occupied by footer
+  unsigned footercols;           // columns occupied by footer
   uint64_t opchannels;           // option channels
   uint64_t descchannels;         // description channels
   uint64_t titlechannels;        // title channels
@@ -42,15 +42,15 @@ typedef struct ncmultiselector {
   unsigned current;               // index of highlighted item
   unsigned startdisp;             // index of first option displayed
   unsigned maxdisplay;            // max number of items to display, 0 -> no limit
-  int longitem;                   // columns occupied by longest item
+  unsigned longitem;              // columns occupied by longest item
   struct ncmselector_int* items;  // items, descriptions, and statuses, heap-copied
   unsigned itemcount;             // number of pairs in 'items'
   char* title;                    // can be NULL, in which case there's no riser
-  int titlecols;                  // columns occupied by title
+  unsigned titlecols;             // columns occupied by title
   char* secondary;                // can be NULL
-  int secondarycols;              // columns occupied by secondary
+  unsigned secondarycols;         // columns occupied by secondary
   char* footer;                   // can be NULL
-  int footercols;                 // columns occupied by footer
+  unsigned footercols;            // columns occupied by footer
   uint64_t opchannels;            // option channels
   uint64_t descchannels;          // description channels
   uint64_t titlechannels;         // title channels
@@ -62,7 +62,7 @@ typedef struct ncmultiselector {
 // ideal body width given the ncselector's items and secondary/footer
 static int
 ncselector_body_width(const ncselector* n){
-  int cols = 0;
+  unsigned cols = 0;
   // the body is the maximum of
   //  * longop + longdesc + 5
   //  * secondary + 2
@@ -83,7 +83,7 @@ ncselector_body_width(const ncselector* n){
 static int
 ncselector_draw(ncselector* n){
   ncplane_erase(n->ncp);
-  nccell transchar = CELL_TRIVIAL_INITIALIZER;
+  nccell transchar = NCCELL_TRIVIAL_INITIALIZER;
   nccell_set_fg_alpha(&transchar, NCALPHA_TRANSPARENT);
   nccell_set_bg_alpha(&transchar, NCALPHA_TRANSPARENT);
   // if we have a title, we'll draw a riser. the riser is two rows tall, and
@@ -94,21 +94,25 @@ ncselector_draw(ncselector* n){
     size_t riserwidth = n->titlecols + 4;
     int offx = ncplane_halign(n->ncp, NCALIGN_RIGHT, riserwidth);
     ncplane_cursor_move_yx(n->ncp, 0, 0);
-    ncplane_hline(n->ncp, &transchar, offx);
+    if(offx){
+      ncplane_hline(n->ncp, &transchar, offx);
+    }
     ncplane_cursor_move_yx(n->ncp, 0, offx);
     ncplane_rounded_box_sized(n->ncp, 0, n->boxchannels, 3, riserwidth, 0);
     n->ncp->channels = n->titlechannels;
     ncplane_printf_yx(n->ncp, 1, offx + 1, " %s ", n->title);
     yoff += 2;
     ncplane_cursor_move_yx(n->ncp, 1, 0);
-    ncplane_hline(n->ncp, &transchar, offx);
+    if(offx){
+      ncplane_hline(n->ncp, &transchar, offx);
+    }
   }
-  int bodywidth = ncselector_body_width(n);
-  int dimy, dimx;
+  unsigned bodywidth = ncselector_body_width(n);
+  unsigned dimy, dimx;
   ncplane_dim_yx(n->ncp, &dimy, &dimx);
   int xoff = ncplane_halign(n->ncp, NCALIGN_RIGHT, bodywidth);
   if(xoff){
-    for(int y = yoff + 1 ; y < dimy ; ++y){
+    for(unsigned y = yoff + 1 ; y < dimy ; ++y){
       ncplane_cursor_move_yx(n->ncp, y, 0);
       ncplane_hline(n->ncp, &transchar, xoff);
     }
@@ -158,8 +162,8 @@ ncselector_draw(ncselector* n){
   // Top line of body (background and possibly up arrow)
   ++yoff;
   ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-  for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-    nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+  for(unsigned i = xoff + 1 ; i < dimx - 1 ; ++i){
+    nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
     ncplane_putc(n->ncp, &transc);
   }
   const int bodyoffset = dimx - bodywidth + 2;
@@ -177,13 +181,13 @@ ncselector_draw(ncselector* n){
   n->uarrowy = yoff;
   unsigned printidx = n->startdisp;
   unsigned printed = 0;
-  for(yoff += 1 ; yoff < dimy - 2 ; ++yoff){
+  for(yoff += 1 ; yoff < (int)dimy - 2 ; ++yoff){
     if(n->maxdisplay && printed == n->maxdisplay){
       break;
     }
     ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-    for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-      nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+    for(int i = xoff + 1 ; i < (int)dimx - 1 ; ++i){
+      nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
       ncplane_putc(n->ncp, &transc);
     }
     n->ncp->channels = n->opchannels;
@@ -203,8 +207,8 @@ ncselector_draw(ncselector* n){
   }
   // Bottom line of body (background and possibly down arrow)
   ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-  for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-    nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+  for(int i = xoff + 1 ; i < (int)dimx - 1 ; ++i){
+    nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
     ncplane_putc(n->ncp, &transc);
   }
   if(n->maxdisplay && n->maxdisplay < n->itemcount){
@@ -221,10 +225,10 @@ ncselector_draw(ncselector* n){
 
 // calculate the necessary dimensions based off properties of the selector
 static void
-ncselector_dim_yx(const ncselector* n, int* ncdimy, int* ncdimx){
-  int rows = 0, cols = 0; // desired dimensions
-  int dimy, dimx; // dimensions of containing screen
+ncselector_dim_yx(const ncselector* n, unsigned* ncdimy, unsigned* ncdimx){
+  unsigned rows = 0, cols = 0; // desired dimensions
   const ncplane* parent = ncplane_parent(n->ncp);
+  unsigned dimy, dimx; // dimensions of containing plane
   ncplane_dim_yx(parent, &dimy, &dimx);
   if(n->title){ // header adds two rows for riser
     rows += 2;
@@ -245,7 +249,39 @@ ncselector_dim_yx(const ncselector* n, int* ncdimy, int* ncdimx){
   *ncdimx = cols;
 }
 
+static void
+ncselector_destroy_internal(ncselector* n){
+  if(n){
+    while(n->itemcount--){
+      free(n->items[n->itemcount].option);
+      free(n->items[n->itemcount].desc);
+    }
+    if(ncplane_set_widget(n->ncp, NULL, NULL) == 0){
+      ncplane_destroy(n->ncp);
+    }
+    free(n->items);
+    free(n->title);
+    free(n->secondary);
+    free(n->footer);
+    free(n);
+  }
+}
+
+void ncselector_destroy(ncselector* n, char** item){
+  if(n){
+    if(item){
+      *item = n->items[n->selected].option;
+      n->items[n->selected].option = NULL;
+    }
+    ncselector_destroy_internal(n);
+  }
+}
+
 ncselector* ncselector_create(ncplane* n, const ncselector_options* opts){
+  if(n == notcurses_stdplane(ncplane_notcurses(n))){
+    logerror("won't use the standard plane\n"); // would fail later on resize
+    return NULL;
+  }
   ncselector_options zeroed = {};
   if(!opts){
     opts = &zeroed;
@@ -265,14 +301,15 @@ ncselector* ncselector_create(ncplane* n, const ncselector_options* opts){
   }
   memset(ns, 0, sizeof(*ns));
   if(opts->defidx && opts->defidx >= itemcount){
+    logerror("default index %u too large (%u items)\n", opts->defidx, itemcount);
     goto freeitems;
   }
   ns->title = opts->title ? strdup(opts->title) : NULL;
-  ns->titlecols = opts->title ? ncstrwidth(opts->title) : 0;
+  ns->titlecols = opts->title ? ncstrwidth(opts->title, NULL, NULL) : 0;
   ns->secondary = opts->secondary ? strdup(opts->secondary) : NULL;
-  ns->secondarycols = opts->secondary ? ncstrwidth(opts->secondary) : 0;
+  ns->secondarycols = opts->secondary ? ncstrwidth(opts->secondary, NULL, NULL) : 0;
   ns->footer = opts->footer ? strdup(opts->footer) : NULL;
-  ns->footercols = opts->footer ? ncstrwidth(opts->footer) : 0;
+  ns->footercols = opts->footer ? ncstrwidth(opts->footer, NULL, NULL) : 0;
   ns->selected = opts->defidx;
   ns->longop = 0;
   if( (ns->maxdisplay = opts->maxdisplay) ){
@@ -301,13 +338,21 @@ ncselector* ncselector_create(ncplane* n, const ncselector_options* opts){
   }
   for(ns->itemcount = 0 ; ns->itemcount < itemcount ; ++ns->itemcount){
     const struct ncselector_item* src = &opts->items[ns->itemcount];
-    int cols = ncstrwidth(src->option);
+    int unsafe = ncstrwidth(src->option, NULL, NULL);
+    if(unsafe < 0){
+      goto freeitems;
+    }
+    unsigned cols = unsafe;
     ns->items[ns->itemcount].opcolumns = cols;
     if(cols > ns->longop){
       ns->longop = cols;
     }
     const char *desc = src->desc ? src->desc : "";
-    cols = ncstrwidth(desc);
+    unsafe = ncstrwidth(desc, NULL, NULL);
+    if(unsafe < 0){
+      goto freeitems;
+    }
+    cols = unsafe;
     ns->items[ns->itemcount].desccolumns = cols;
     if(cols > ns->longdesc){
       ns->longdesc = cols;
@@ -320,11 +365,13 @@ ncselector* ncselector_create(ncplane* n, const ncselector_options* opts){
       goto freeitems;
     }
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ns->ncp = n;
   ncselector_dim_yx(ns, &dimy, &dimx);
   if(ncplane_resize_simple(n, dimy, dimx)){
-    ncplane_destroy(ns->ncp);
+    goto freeitems;
+  }
+  if(ncplane_set_widget(ns->ncp, ns, (void(*)(void*))ncselector_destroy_internal)){
     goto freeitems;
   }
   ncselector_draw(ns); // deal with error here?
@@ -343,7 +390,7 @@ freeitems:
 }
 
 int ncselector_additem(ncselector* n, const struct ncselector_item* item){
-  int origdimy, origdimx;
+  unsigned origdimy, origdimx;
   ncselector_dim_yx(n, &origdimy, &origdimx);
   size_t newsize = sizeof(*n->items) * (n->itemcount + 1);
   struct ncselector_int* items = realloc(n->items, newsize);
@@ -354,18 +401,22 @@ int ncselector_additem(ncselector* n, const struct ncselector_item* item){
   n->items[n->itemcount].option = strdup(item->option);
   const char *desc = item->desc ? item->desc : "";
   n->items[n->itemcount].desc = strdup(desc);
-  int cols = ncstrwidth(item->option);
+  int usafecols = ncstrwidth(item->option, NULL, NULL);
+  if(usafecols < 0){
+    return -1;
+  }
+  unsigned cols = usafecols;
   n->items[n->itemcount].opcolumns = cols;
   if(cols > n->longop){
     n->longop = cols;
   }
-  cols = ncstrwidth(desc);
+  cols = ncstrwidth(desc, NULL, NULL);
   n->items[n->itemcount].desccolumns = cols;
   if(cols > n->longdesc){
     n->longdesc = cols;
   }
   ++n->itemcount;
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ncselector_dim_yx(n, &dimy, &dimx);
   if(origdimx < dimx || origdimy < dimy){ // resize if too small
     ncplane_resize_simple(n->ncp, dimy, dimx);
@@ -374,7 +425,7 @@ int ncselector_additem(ncselector* n, const struct ncselector_item* item){
 }
 
 int ncselector_delitem(ncselector* n, const char* item){
-  int origdimy, origdimx;
+  unsigned origdimy, origdimx;
   ncselector_dim_yx(n, &origdimy, &origdimx);
   bool found = false;
   int maxop = 0, maxdesc = 0;
@@ -393,11 +444,11 @@ int ncselector_delitem(ncselector* n, const char* item){
       found = true;
       --idx;
     }else{
-      int cols = ncstrwidth(n->items[idx].option);
+      int cols = ncstrwidth(n->items[idx].option, NULL, NULL);
       if(cols > maxop){
         maxop = cols;
       }
-      cols = ncstrwidth(n->items[idx].desc);
+      cols = ncstrwidth(n->items[idx].desc, NULL, NULL);
       if(cols > maxdesc){
         maxdesc = cols;
       }
@@ -406,7 +457,7 @@ int ncselector_delitem(ncselector* n, const char* item){
   if(found){
     n->longop = maxop;
     n->longdesc = maxdesc;
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncselector_dim_yx(n, &dimy, &dimx);
     if(origdimx > dimx || origdimy > dimy){ // resize if too big
       ncplane_resize_simple(n->ncp, dimy, dimx);
@@ -532,33 +583,14 @@ bool ncselector_offer_input(ncselector* n, const ncinput* nc){
   return false;
 }
 
-void ncselector_destroy(ncselector* n, char** item){
-  if(n){
-    if(item){
-      *item = n->items[n->selected].option;
-      n->items[n->selected].option = NULL;
-    }
-    while(n->itemcount--){
-      free(n->items[n->itemcount].option);
-      free(n->items[n->itemcount].desc);
-    }
-    ncplane_destroy(n->ncp);
-    free(n->items);
-    free(n->title);
-    free(n->secondary);
-    free(n->footer);
-    free(n);
-  }
-}
-
 ncplane* ncmultiselector_plane(ncmultiselector* n){
   return n->ncp;
 }
 
 // ideal body width given the ncselector's items and secondary/footer
-static int
+static unsigned
 ncmultiselector_body_width(const ncmultiselector* n){
-  int cols = 0;
+  unsigned cols = 0;
   // the body is the maximum of
   //  * longop + longdesc + 5
   //  * secondary + 2
@@ -579,31 +611,35 @@ ncmultiselector_body_width(const ncmultiselector* n){
 static int
 ncmultiselector_draw(ncmultiselector* n){
   ncplane_erase(n->ncp);
-  nccell transchar = CELL_TRIVIAL_INITIALIZER;
+  nccell transchar = NCCELL_TRIVIAL_INITIALIZER;
   nccell_set_fg_alpha(&transchar, NCALPHA_TRANSPARENT);
   nccell_set_bg_alpha(&transchar, NCALPHA_TRANSPARENT);
   // if we have a title, we'll draw a riser. the riser is two rows tall, and
   // exactly four columns longer than the title, and aligned to the right. we
   // draw a rounded box. the body will blow part or all of the bottom away.
-  int yoff = 0;
+  unsigned yoff = 0;
   if(n->title){
     size_t riserwidth = n->titlecols + 4;
     int offx = ncplane_halign(n->ncp, NCALIGN_RIGHT, riserwidth);
     ncplane_cursor_move_yx(n->ncp, 0, 0);
-    ncplane_hline(n->ncp, &transchar, offx);
+    if(offx){
+      ncplane_hline(n->ncp, &transchar, offx);
+    }
     ncplane_rounded_box_sized(n->ncp, 0, n->boxchannels, 3, riserwidth, 0);
     n->ncp->channels = n->titlechannels;
     ncplane_printf_yx(n->ncp, 1, offx + 1, " %s ", n->title);
     yoff += 2;
     ncplane_cursor_move_yx(n->ncp, 1, 0);
-    ncplane_hline(n->ncp, &transchar, offx);
+    if(offx){
+      ncplane_hline(n->ncp, &transchar, offx);
+    }
   }
-  int bodywidth = ncmultiselector_body_width(n);
-  int dimy, dimx;
+  unsigned bodywidth = ncmultiselector_body_width(n);
+  unsigned dimy, dimx;
   ncplane_dim_yx(n->ncp, &dimy, &dimx);
   int xoff = ncplane_halign(n->ncp, NCALIGN_RIGHT, bodywidth);
   if(xoff){
-    for(int y = yoff + 1 ; y < dimy ; ++y){
+    for(unsigned y = yoff + 1 ; y < dimy ; ++y){
       ncplane_cursor_move_yx(n->ncp, y, 0);
       ncplane_hline(n->ncp, &transchar, xoff);
     }
@@ -641,8 +677,8 @@ ncmultiselector_draw(ncmultiselector* n){
   // Top line of body (background and possibly up arrow)
   ++yoff;
   ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-  for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-    nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+  for(unsigned i = xoff + 1 ; i < dimx - 1 ; ++i){
+    nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
     ncplane_putc(n->ncp, &transc);
   }
   const int bodyoffset = dimx - bodywidth + 2;
@@ -662,8 +698,8 @@ ncmultiselector_draw(ncmultiselector* n){
       break;
     }
     ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-    for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-      nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+    for(unsigned i = xoff + 1 ; i < dimx - 1 ; ++i){
+      nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
       ncplane_putc(n->ncp, &transc);
     }
     n->ncp->channels = n->descchannels;
@@ -692,8 +728,8 @@ ncmultiselector_draw(ncmultiselector* n){
   }
   // Bottom line of body (background and possibly down arrow)
   ncplane_cursor_move_yx(n->ncp, yoff, xoff + 1);
-  for(int i = xoff + 1 ; i < dimx - 1 ; ++i){
-    nccell transc = CELL_TRIVIAL_INITIALIZER; // fall back to base cell
+  for(unsigned i = xoff + 1 ; i < dimx - 1 ; ++i){
+    nccell transc = NCCELL_TRIVIAL_INITIALIZER; // fall back to base cell
     ncplane_putc(n->ncp, &transc);
   }
   if(n->maxdisplay && n->maxdisplay < n->itemcount){
@@ -816,9 +852,9 @@ bool ncmultiselector_offer_input(ncmultiselector* n, const ncinput* nc){
 // calculate the necessary dimensions based off properties of the selector and
 // the containing plane
 static int
-ncmultiselector_dim_yx(const ncmultiselector* n, int* ncdimy, int* ncdimx){
-  int rows = 0, cols = 0; // desired dimensions
-  int dimy, dimx; // dimensions of containing screen
+ncmultiselector_dim_yx(const ncmultiselector* n, unsigned* ncdimy, unsigned* ncdimx){
+  unsigned rows = 0, cols = 0; // desired dimensions
+  unsigned dimy, dimx; // dimensions of containing screen
   ncplane_dim_yx(ncplane_parent(n->ncp), &dimy, &dimx);
   if(n->title){ // header adds two rows for riser
     rows += 2;
@@ -847,6 +883,10 @@ ncmultiselector_dim_yx(const ncmultiselector* n, int* ncdimy, int* ncdimx){
 }
 
 ncmultiselector* ncmultiselector_create(ncplane* n, const ncmultiselector_options* opts){
+  if(n == notcurses_stdplane(ncplane_notcurses(n))){
+    logerror("won't use the standard plane\n"); // would fail later on resize
+    return NULL;
+  }
   ncmultiselector_options zeroed = {};
   if(!opts){
     opts = &zeroed;
@@ -866,11 +906,11 @@ ncmultiselector* ncmultiselector_create(ncplane* n, const ncmultiselector_option
   }
   memset(ns, 0, sizeof(*ns));
   ns->title = opts->title ? strdup(opts->title) : NULL;
-  ns->titlecols = opts->title ? ncstrwidth(opts->title) : 0;
+  ns->titlecols = opts->title ? ncstrwidth(opts->title, NULL, NULL) : 0;
   ns->secondary = opts->secondary ? strdup(opts->secondary) : NULL;
-  ns->secondarycols = opts->secondary ? ncstrwidth(opts->secondary) : 0;
+  ns->secondarycols = opts->secondary ? ncstrwidth(opts->secondary, NULL, NULL) : 0;
   ns->footer = opts->footer ? strdup(opts->footer) : NULL;
-  ns->footercols = opts->footer ? ncstrwidth(opts->footer) : 0;
+  ns->footercols = opts->footer ? ncstrwidth(opts->footer, NULL, NULL) : 0;
   ns->current = 0;
   ns->startdisp = 0;
   ns->longitem = 0;
@@ -891,11 +931,19 @@ ncmultiselector* ncmultiselector_create(ncplane* n, const ncmultiselector_option
   }
   for(ns->itemcount = 0 ; ns->itemcount < itemcount ; ++ns->itemcount){
     const struct ncmselector_item* src = &opts->items[ns->itemcount];
-    int cols = ncstrwidth(src->option);
+    int unsafe = ncstrwidth(src->option, NULL, NULL);
+    if(unsafe < 0){
+      goto freeitems;
+    }
+    unsigned cols = unsafe;
     if(cols > ns->longitem){
       ns->longitem = cols;
     }
-    int cols2 = ncstrwidth(src->desc);
+    unsafe = ncstrwidth(src->desc, NULL, NULL);
+    if(unsafe < 0){
+      goto freeitems;
+    }
+    unsigned cols2 = unsafe;
     if(cols + cols2 > ns->longitem){
       ns->longitem = cols + cols2;
     }
@@ -908,7 +956,7 @@ ncmultiselector* ncmultiselector_create(ncplane* n, const ncmultiselector_option
       goto freeitems;
     }
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   ns->ncp = n;
   if(ncmultiselector_dim_yx(ns, &dimy, &dimx)){
     goto freeitems;
@@ -916,6 +964,9 @@ ncmultiselector* ncmultiselector_create(ncplane* n, const ncmultiselector_option
   if(ncplane_resize_simple(ns->ncp, dimy, dimx)){
     goto freeitems;
   }
+  if(ncplane_set_widget(ns->ncp, ns, (void(*)(void*))ncmultiselector_destroy)){
+    goto freeitems;
+  }
   ncmultiselector_draw(ns); // deal with error here?
   return ns;
 
@@ -937,7 +988,9 @@ void ncmultiselector_destroy(ncmultiselector* n){
       free(n->items[n->itemcount].option);
       free(n->items[n->itemcount].desc);
     }
-    ncplane_destroy(n->ncp);
+    if(ncplane_set_widget(n->ncp, NULL, NULL) == 0){
+      ncplane_destroy(n->ncp);
+    }
     free(n->items);
     free(n->title);
     free(n->secondary);
