@@ -27,7 +27,7 @@ sixelcount(int dimy, int dimx){
 // bytes, saving 7/8 of the space FIXME).
 static inline uint8_t*
 sixel_auxiliary_vector(const sprixel* s){
-  int pixels = s->cellpxy * s->cellpxx;
+  int pixels = ncplane_pile(s->n)->cellpxy * ncplane_pile(s->n)->cellpxx;
   uint8_t* ret = malloc(sizeof(*ret) * pixels * 2);
   if(ret){
     memset(ret, 0, sizeof(*ret) * pixels);
@@ -229,15 +229,17 @@ int sixel_wipe(sprixel* s, int ycell, int xcell){
   if(auxvec == NULL){
     return -1;
   }
-  memset(auxvec + s->cellpxx * s->cellpxy, 0xff, s->cellpxx * s->cellpxy);
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
+  memset(auxvec + cellpxx * cellpxy, 0xff, cellpxx * cellpxy);
   sixelmap* smap = s->smap;
-  const int startx = xcell * s->cellpxx;
-  const int starty = ycell * s->cellpxy;
-  int endx = ((xcell + 1) * s->cellpxx) - 1;
+  const int startx = xcell * cellpxx;
+  const int starty = ycell * cellpxy;
+  int endx = ((xcell + 1) * cellpxx) - 1;
   if(endx >= s->pixx){
     endx = s->pixx - 1;
   }
-  int endy = ((ycell + 1) * s->cellpxy) - 1;
+  int endy = ((ycell + 1) * cellpxy) - 1;
   if(endy >= s->pixy){
     endy = s->pixy - 1;
   }
@@ -248,7 +250,7 @@ int sixel_wipe(sprixel* s, int ycell, int xcell){
   int w = 0;
   for(int c = 0 ; c < smap->colors ; ++c){
     w |= wipe_color(smap, c, startband, endband, startx, endx, starty, endy,
-                    s->pixx, s->cellpxy, s->cellpxx, auxvec);
+                    s->pixx, cellpxy, cellpxx, auxvec);
   }
   if(w){
     s->wipes_outstanding = true;
@@ -277,9 +279,9 @@ scrub_color_table(sprixel* s){
   if(s->n && s->n->tam){
     // we use the sprixel cell geometry rather than the plane's because this
     // is called during our initial blit, before we've resized the plane.
-    for(int y = 0 ; y < s->dimy ; ++y){
-      for(int x = 0 ; x < s->dimx ; ++x){
-        int txyidx = y * s->dimx + x;
+    for(unsigned y = 0 ; y < s->dimy ; ++y){
+      for(unsigned x = 0 ; x < s->dimx ; ++x){
+        unsigned txyidx = y * s->dimx + x;
         sprixcell_e state = s->n->tam[txyidx].state;
         if(state == SPRIXCELL_ANNIHILATED || state == SPRIXCELL_ANNIHILATED_TRANS){
 //fprintf(stderr, "POSTEXRACT WIPE %d/%d\n", y, x);
@@ -382,14 +384,14 @@ void sixel_refresh(const ncpile* p, sprixel* s){
   }
   int absy, absx;
   ncplane_abs_yx(s->n, &absy, &absx);
-  for(int y = 0 ; y < s->dimy ; ++y){
-    const int yy = absy + y;
-    for(int x = 0 ; x < s->dimx ; ++x){
-      int idx = y * s->dimx + x;
+  for(unsigned y = 0 ; y < s->dimy ; ++y){
+    const unsigned yy = absy + y;
+    for(unsigned x = 0 ; x < s->dimx ; ++x){
+      unsigned idx = y * s->dimx + x;
       if(s->needs_refresh[idx]){
-        const int xx = absx + x;
+        const unsigned xx = absx + x;
         if(xx < p->dimx && yy < p->dimy){
-          int ridx = yy * p->dimx + xx;
+          unsigned ridx = yy * p->dimx + xx;
           struct crender *r = &p->crender[ridx];
           r->s.damaged = 1;
         }
@@ -428,9 +430,8 @@ extract_color_table(const uint32_t* data, int linesize, int cols,
                     tament* tam, const blitterargs* bargs){
   const int begx = bargs->begx;
   const int begy = bargs->begy;
-  sprixel* s = bargs->u.pixel.spx;
-  const int cdimy = s->cellpxy;
-  const int cdimx = s->cellpxx;
+  const int cdimy = bargs->u.pixel.cellpxy;
+  const int cdimx = bargs->u.pixel.cellpxx;
   unsigned char mask = 0xc0;
   int pos = 0; // pixel position
   unsigned char* rmatrix = bargs->u.pixel.spx->needs_refresh;
@@ -876,11 +877,14 @@ sixel_reblit(sprixel* s){
 // scaled geometry in pixels. We calculate output geometry herein, and supply
 // transparent filler input for any missing rows.
 static inline int
-sixel_blit_inner(int leny, int lenx, sixeltable* stab, sprixel* s, tament* tam){
+sixel_blit_inner(int leny, int lenx, sixeltable* stab, const blitterargs* bargs, tament* tam){
   fbuf f;
   if(fbuf_init(&f)){
     return -1;
   }
+  sprixel* s = bargs->u.pixel.spx;
+  const int cellpxy = bargs->u.pixel.cellpxy;
+  const int cellpxx = bargs->u.pixel.cellpxx;
   int parse_start = 0;
   int outy = leny;
   if(leny % 6){
@@ -892,7 +896,7 @@ sixel_blit_inner(int leny, int lenx, sixeltable* stab, sprixel* s, tament* tam){
     fbuf_free(&f);
     return -1;
   }
-  scrub_tam_boundaries(tam, outy, lenx, s->cellpxy, s->cellpxx);
+  scrub_tam_boundaries(tam, outy, lenx, cellpxy, cellpxx);
   // take ownership of buf on success
   if(plane_blit_sixel(s, &f, outy, lenx, parse_start, tam, SPRIXEL_INVALIDATED) < 0){
     fbuf_free(&f);
@@ -945,7 +949,7 @@ int sixel_blit(ncplane* n, int linesize, const void* data, int leny, int lenx,
   }
   refine_color_table(data, linesize, bargs->begy, bargs->begx, leny, lenx, &stable);
   // takes ownership of sixelmap on success
-  int r = sixel_blit_inner(leny, lenx, &stable, bargs->u.pixel.spx, n->tam);
+  int r = sixel_blit_inner(leny, lenx, &stable, bargs, n->tam);
   if(r < 0){
     sixelmap_free(stable.map);
   }
@@ -962,8 +966,8 @@ int sixel_scrub(const ncpile* p, sprixel* s){
   loginfo("%d state %d at %d/%d (%d/%d)\n", s->id, s->invalidated, s->movedfromy, s->movedfromx, s->dimy, s->dimx);
   int starty = s->movedfromy;
   int startx = s->movedfromx;
-  for(int yy = starty ; yy < starty + s->dimy && yy < p->dimy ; ++yy){
-    for(int xx = startx ; xx < startx + s->dimx && xx < p->dimx ; ++xx){
+  for(int yy = starty ; yy < starty + (int)s->dimy && yy < (int)p->dimy ; ++yy){
+    for(int xx = startx ; xx < startx + (int)s->dimx && xx < (int)p->dimx ; ++xx){
       int ridx = yy * p->dimx + xx;
       struct crender *r = &p->crender[ridx];
       if(!s->n){
@@ -972,11 +976,11 @@ int sixel_scrub(const ncpile* p, sprixel* s){
         continue;
       }
       sprixel* trues = r->sprixel ? r->sprixel : s;
-      if(yy >= trues->n->leny || yy - trues->n->absy < 0){
+      if(yy >= (int)trues->n->leny || yy - trues->n->absy < 0){
         r->s.damaged = 1;
         continue;
       }
-      if(xx >= trues->n->lenx || xx - trues->n->absx < 0){
+      if(xx >= (int)trues->n->lenx || xx - trues->n->absx < 0){
         r->s.damaged = 1;
         continue;
       }
@@ -1010,15 +1014,15 @@ int sixel_draw(const tinfo* ti, const ncpile* p, sprixel* s, fbuf* f,
   if(p){
     const int targy = s->n->absy + yoff;
     const int targx = s->n->absx + xoff;
-    if(goto_location(p->nc, f, targy, targx)){
+    if(goto_location(p->nc, f, targy, targx, NULL)){
       return -1;
     }
     if(s->invalidated == SPRIXEL_MOVED){
-      for(int yy = s->movedfromy ; yy < s->movedfromy + s->dimy && yy < p->dimy ; ++yy){
+      for(int yy = s->movedfromy ; yy < s->movedfromy + (int)s->dimy && yy < (int)p->dimy ; ++yy){
         if(yy < 0){
           continue;
         }
-        for(int xx = s->movedfromx ; xx < s->movedfromx + s->dimx && xx < p->dimx ; ++xx){
+        for(int xx = s->movedfromx ; xx < s->movedfromx + (int)s->dimx && xx < (int)p->dimx ; ++xx){
           if(xx < 0){
             continue;
           }
@@ -1063,13 +1067,15 @@ int sixel_init_inverted(int fd){
 int sixel_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
   s->wipes_outstanding = true;
   sixelmap* smap = s->smap;
-  const int startx = xcell * s->cellpxx;
-  const int starty = ycell * s->cellpxy;
-  int endx = ((xcell + 1) * s->cellpxx) - 1;
+  const int cellpxx = ncplane_pile(s->n)->cellpxx;
+  const int cellpxy = ncplane_pile(s->n)->cellpxy;
+  const int startx = xcell * cellpxx;
+  const int starty = ycell * cellpxy;
+  int endx = ((xcell + 1) * cellpxx) - 1;
   if(endx > s->pixx){
     endx = s->pixx;
   }
-  int endy = ((ycell + 1) * s->cellpxy) - 1;
+  int endy = ((ycell + 1) * cellpxy) - 1;
   if(endy > s->pixy){
     endy = s->pixy;
   }
@@ -1077,8 +1083,8 @@ int sixel_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
 //fprintf(stderr, "%d/%d start: %d/%d end: %d/%d bands: %d-%d\n", ycell, xcell, starty, startx, endy, endx, starty / 6, endy / 6);
   for(int x = startx ; x <= endx ; ++x){
     for(int y = starty ; y <= endy ; ++y){
-      int auxvecidx = (y - starty) * s->cellpxx + (x - startx);
-      int trans = auxvec[s->cellpxx * s->cellpxy + auxvecidx];
+      int auxvecidx = (y - starty) * cellpxx + (x - startx);
+      int trans = auxvec[cellpxx * cellpxy + auxvecidx];
       if(!trans){
         int color = auxvec[auxvecidx];
         int didx = ctable_to_dtable(smap->table + color * CENTSIZE);
@@ -1094,7 +1100,7 @@ int sixel_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec){
     }
   }
   sprixcell_e newstate;
-  if(transparent == s->cellpxx * s->cellpxy){
+  if(transparent == cellpxx * cellpxy){
     newstate = SPRIXCELL_TRANSPARENT;
   }else if(transparent){
     newstate = SPRIXCELL_MIXED_SIXEL;
@@ -1111,8 +1117,8 @@ int sixel_shutdown(fbuf* f){
   return 0;
 }
 
-uint8_t* sixel_trans_auxvec(const tinfo* ti){
-  const size_t slen = 2 * ti->cellpixy * ti->cellpixx;
+uint8_t* sixel_trans_auxvec(const ncpile* p){
+  const size_t slen = 2 * p->cellpxy * p->cellpxx;
   uint8_t* a = malloc(slen);
   if(a){
     memset(a, 0, slen);
