@@ -1,5 +1,6 @@
 #include <fcntl.h>
 #include <unistd.h>
+#include <curses.h>
 #ifdef __linux__
 #include <sys/utsname.h>
 #endif
@@ -38,7 +39,7 @@ static void
 get_default_geometry(tinfo* ti){
   ti->default_rows = get_default_dimension("LINES", "lines", 24);
   ti->default_cols = get_default_dimension("COLUMNS", "cols", 80);
-  loginfo("Default geometry: %d row%s, %d column%s\n",
+  loginfo("default geometry: %d row%s, %d column%s\n",
           ti->default_rows, ti->default_rows != 1 ? "s" : "",
           ti->default_cols, ti->default_cols != 1 ? "s" : "");
   ti->dimy = ti->default_rows;
@@ -308,8 +309,8 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // XTVERSION. Replies with DCS > | ... ST
 #define XTVERSION "\x1b[>0q"
 
-// XTGETTCAP['TN'] (Terminal Name)
-#define XTGETTCAPTN "\x1bP+q544e\x1b\\"
+// XTGETTCAP['TN', 'RGB'] (Terminal Name, RGB)
+#define XTGETTCAP  "\x1bP+q544e;524742\x1b\\"
 
 // Secondary Device Attributes, necessary to get Alacritty's version. Since
 // this doesn't uniquely identify a terminal, we ask it last, so that if any
@@ -325,16 +326,22 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // send this with the other identification queries, since APCs tend not to
 // be consumed by certain terminal emulators (looking at you, Linux console)
 // which can be identified directly, sans queries.
+// we do not send this query on Windows because it is bled through ConHost,
+// and echoed onto the standard output.
+#ifndef __MINGW64__
 #define KITTYQUERY "\x1b_Gi=1,a=q;\x1b\\"
+#else
+#define KITTYQUERY
+#endif
 
 // request kitty keyboard protocol features 1, 2, and 8, first pushing current.
 // see https://sw.kovidgoyal.net/kitty/keyboard-protocol/#progressive-enhancement
-#define KBDSUPPORT "\x1b[=11u"
+#define KKBDSUPPORT "\x1b[=11u"
 
 // the kitty keyboard protocol allows unambiguous, complete identification of
 // input events. this queries for the level of support. we want to do this
 // because the "keyboard pop" control code is mishandled by kitty < 0.20.0.
-#define KBDQUERY "\x1b[?u"
+#define KKBDQUERY "\x1b[?u"
 
 // set modifyFunctionKeys (2) if supported, allowing us to disambiguate
 // function keys when used with modifiers. set modifyOtherKeys (4) if
@@ -346,14 +353,14 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // to, there's no point in sending them.
 #define IDQUERIES TRIDEVATTR \
                   XTVERSION \
-                  XTGETTCAPTN \
+                  XTGETTCAP \
                   SECDEVATTR
 
 // query background, replies in X color https://www.x.org/releases/X11R7.7/doc/man/man7/X.7.xhtml#heading11
-// GNU screen passes this on to the underlying terminal rather than answering
-// itself, unlike most other queries, so send this first since it will take
-// longer to be answered.
-#define CSI_BGQ "\x1b]11;?\e\\"
+// GNU screen passes this on to the underlying terminal rather than answering itself,
+// unlike most other queries, so send this first since it will take longer to be
+// answered. note the "\x1b]"; this is an Operating System Command, not CSI.
+#define DEFBGQ "\x1b]11;?\e\\"
 
 // FIXME ought be using the u7 terminfo string here, if it exists. the great
 // thing is, if we get a response to this, we know we can use it for u7!
@@ -379,8 +386,8 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
 // request the cell geometry of the textual area
 #define GEOMCELL "\x1b[18t"
 
-#define DIRECTIVES CSI_BGQ \
-                   KBDQUERY \
+#define DIRECTIVES DEFBGQ \
+                   KKBDQUERY \
                    SUMQUERY \
                    "\x1b[?1;3;256S" /* try to set 256 cregs */ \
                    KITTYQUERY \
@@ -390,47 +397,69 @@ init_terminfo_esc(tinfo* ti, const char* name, escape_e idx,
                    GEOMCELL \
                    PRIDEVATTR
 
+// kitty keyboard push, used at start
+#define KKEYBOARD_PUSH "\x1b[>u"
+
 // written whenever we switch between standard and alternate screen, or upon
 // startup (that's an entry into a screen! presumably the standard one).
-#define KBDENTER KKEYBOARD_PUSH XTMODKEYS KBDSUPPORT
-#define KBDLEAVE KKEYBOARD_POP // FIXME undo XTMODKEYS!
+#define KKBDENTER KKEYBOARD_PUSH KKBDSUPPORT
 
 // enter the alternate screen (smcup). we could technically get this from
 // terminfo, but everyone who supports it supports it the same way, and we
 // need to send it before our other directives if we're going to use it.
-#define SMCUP "\x1b[?1049h" KBDENTER
-#define RMCUP KBDLEAVE "\x1b[?1049l"
+#define SMCUP DECSET(SET_SMCUP)
+#define RMCUP DECRST(SET_SMCUP)
 
 // we send an XTSMGRAPHICS to set up 256 color registers (the most we can
 // currently take advantage of; we need at least 64 to use sixel at all).
 // maybe that works, maybe it doesn't. then query both color registers
 // and geometry. send XTGETTCAP for terminal name. if 'minimal' is set, don't
 // send any identification queries (we've already identified the terminal).
+// write DSRCPR as early as possible, so that it precedes any query material
+// that's bled onto stdin and echoed. if 'noaltscreen' is set, do not send
+// an smcup. if 'draininput' is set, do not send any keyboard modifiers.
 static int
-send_initial_queries(int fd, bool minimal, bool noaltscreen){
+send_initial_queries(int fd, unsigned minimal, unsigned noaltscreen,
+                     unsigned draininput){
   const char *queries;
   if(noaltscreen){
     if(minimal){
-      queries = KBDENTER DSRCPR DIRECTIVES;
+      if(draininput){
+        queries = DSRCPR DIRECTIVES;
+      }else{
+        queries = DSRCPR KKBDENTER DIRECTIVES;
+      }
     }else{
-      queries = KBDENTER DSRCPR IDQUERIES DIRECTIVES;
+      if(draininput){
+        queries = DSRCPR IDQUERIES DIRECTIVES;
+      }else{
+        queries = DSRCPR KKBDENTER IDQUERIES DIRECTIVES;
+      }
     }
   }else{
     if(minimal){
-      queries = SMCUP DSRCPR DIRECTIVES;
+      if(draininput){
+        queries = SMCUP DSRCPR DIRECTIVES;
+      }else{
+        queries = SMCUP DSRCPR KKBDENTER DIRECTIVES;
+      }
     }else{
-      queries = SMCUP DSRCPR IDQUERIES DIRECTIVES;
+      if(draininput){
+        queries = SMCUP DSRCPR IDQUERIES DIRECTIVES;
+      }else{
+        queries = SMCUP DSRCPR KKBDENTER IDQUERIES DIRECTIVES;
+      }
     }
   }
   size_t len = strlen(queries);
-  loginfo("sending %lluB queries\n", (unsigned long long)len);
+  loginfo("sending %lluB\n", (unsigned long long)len);
   if(blocking_write(fd, queries, len)){
     return -1;
   }
   return 0;
 }
 
-int enter_alternate_screen(FILE* fp, tinfo* ti, bool flush){
+int enter_alternate_screen(FILE* fp, tinfo* ti, unsigned flush, unsigned drain){
   if(ti->in_alt_screen){
     return 0;
   }
@@ -439,14 +468,25 @@ int enter_alternate_screen(FILE* fp, tinfo* ti, bool flush){
     logerror("alternate screen is unavailable");
     return -1;
   }
-  if(term_emit(smcup, fp, false) || term_emit(KBDENTER, fp, flush)){
+  if(term_emit(smcup, fp, false) < 0){
     return -1;
   }
+  if(!drain){
+    if(ti->kbdlevel){
+      if(term_emit(KKBDENTER, fp, flush)){
+        return -1;
+      }
+    }else{
+      if(term_emit(XTMODKEYS, fp, flush)){
+        return -1;
+      }
+    }
+  }
   ti->in_alt_screen = true;
   return 0;
 }
 
-int leave_alternate_screen(FILE* fp, tinfo* ti){
+int leave_alternate_screen(FILE* fp, tinfo* ti, unsigned drain){
   if(!ti->in_alt_screen){
     return 0;
   }
@@ -455,11 +495,31 @@ int leave_alternate_screen(FILE* fp, tinfo* ti){
     logerror("can't leave alternate screen");
     return -1;
   }
-  if(term_emit(KBDLEAVE, fp, false) ||
-     term_emit(rmcup, fp, false) ||
-     term_emit(KBDENTER, fp, true)){
+  if(!drain){
+    if(ti->kbdlevel){
+      if(term_emit(KKEYBOARD_POP, fp, false)){
+        return -1;
+      }
+    }else{
+      if(term_emit(XTMODKEYSUNDO, fp, false)){
+        return -1;
+      }
+    }
+  }
+  if(term_emit(rmcup, fp, drain)){
     return -1;
   }
+  if(!drain){
+    if(ti->kbdlevel){
+      if(term_emit(KKBDENTER, fp, true)){
+        return -1;
+      }
+    }else{
+      if(term_emit(XTMODKEYS, fp, true)){
+        return -1;
+      }
+    }
+  }
   ti->in_alt_screen = false;
   return 0;
 }
@@ -585,6 +645,7 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     // kitty SUM doesn't want long sequences, which is exactly where we use
     // it. remove support (we pick it up from queries).
     kill_appsync_escapes(ti);
+    ti->gratuitous_hpa = true;
   }else if(qterm == TERMINAL_ALACRITTY){
     termname = "Alacritty";
     ti->caps.quadrants = true;
@@ -655,8 +716,9 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
     }
     ti->bce = true;
   }else if(qterm == TERMINAL_MSTERMINAL){
-    termname = "Windows Terminal";
+    termname = "Windows ConHost";
     ti->caps.rgb = true;
+    ti->caps.quadrants = true;
   }else if(qterm == TERMINAL_CONTOUR){
     termname = "Contour";
     ti->caps.quadrants = true;
@@ -681,10 +743,10 @@ apply_term_heuristics(tinfo* ti, const char* termname, queried_terminals_e qterm
       ti->termversion = strdup(un.release);
     }
     if(is_linux_framebuffer(ti)){
-      termname = "Linux framebuffer";
+      termname = "FBcon";
       setup_fbcon_bitmaps(ti, ti->linux_fb_fd);
     }else{
-      termname = "Linux console";
+      termname = "VT";
     }
     ti->caps.halfblocks = false;
     ti->caps.braille = false; // no caps.braille, no caps.sextants in linux console
@@ -792,7 +854,6 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     cursor_y = &foolcursor_y;
   }
   *cursor_x = *cursor_y = -1;
-  memset(ti, 0, sizeof(*ti));
   ti->bg_collides_default = 0xfe000000;
   ti->kbdlevel = UINT_MAX; // see comment in tinfo definition
   ti->qterm = TERMINAL_UNKNOWN;
@@ -812,9 +873,6 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     logpanic("failed opening Windows ConPTY\n");
     return -1;
   }
-  if(cursor_y && cursor_x){
-    locate_cursor(ti, cursor_y, cursor_x);
-  }
 #elif defined(__linux__)
   ti->linux_fb_fd = -1;
   ti->linux_fbuffer = MAP_FAILED;
@@ -841,7 +899,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     // if we already know our terminal (e.g. on the linux console), there's no
     // need to send the identification queries. the controls are sufficient.
     bool minimal = (ti->qterm != TERMINAL_UNKNOWN);
-    if(send_initial_queries(ti->ttyfd, minimal, noaltscreen)){
+    if(send_initial_queries(ti->ttyfd, minimal, noaltscreen, draininput)){
       goto err;
     }
   }
@@ -945,7 +1003,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
   if(ti->caps.colors > 1){
     const char* initc = get_escape(ti, ESCAPE_INITC);
     if(initc){
-      ti->caps.can_change_colors = tigetflag("ccc") == 1;
+      ti->caps.can_change_colors = true;
     }
   }else{ // disable initc if there's no color support
     ti->escindices[ESCAPE_INITC] = 0;
@@ -998,20 +1056,29 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     if((iresp = inputlayer_get_responses(ti->ictx)) == NULL){
       goto err;
     }
+    ti->termversion = iresp->version; // takes ownership
     if(iresp->appsync_supported){
       if(add_appsync_escapes_sm(ti, &tablelen, &tableused)){
-        free(iresp->version);
         free(iresp);
         goto err;
       }
     }
-    ti->kbdlevel = iresp->kbdlevel;
+    if((ti->kbdlevel = iresp->kbdlevel) == UINT_MAX){
+      ti->kbdlevel = 0;
+      if(!draininput){
+        if(tty_emit(XTMODKEYS, ti->ttyfd) < 0){
+          free(iresp);
+          goto err;
+        }
+      }
+    }else{
+      ti->kittykbdsupport = true;
+    }
     if(iresp->qterm != TERMINAL_UNKNOWN){
       ti->qterm = iresp->qterm;
     }
     *cursor_y = iresp->cursory;
     *cursor_x = iresp->cursorx;
-    ti->termversion = iresp->version;
     if(iresp->dimy && iresp->dimx){
       // FIXME probably oughtn't be setting the defaults, as this is just some
       // random transient measurement?
@@ -1020,13 +1087,16 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
       ti->dimy = iresp->dimy;
       ti->dimx = iresp->dimx;
     }
+    if(iresp->rgb){
+      ti->caps.rgb = true;
+    }
     if(iresp->pixy && iresp->pixx){
       ti->pixy = iresp->pixy;
       ti->pixx = iresp->pixx;
     }
     if(ti->default_rows && ti->default_cols){
-      ti->cellpixy = ti->pixy / ti->default_rows;
-      ti->cellpixx = ti->pixx / ti->default_cols;
+      ti->cellpxy = ti->pixy / ti->default_rows;
+      ti->cellpxx = ti->pixx / ti->default_cols;
     }
     if(iresp->got_bg){
       // reset the 0xfe000000 we loaded during initialization. if we're
@@ -1036,6 +1106,7 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
     // kitty trumps sixel, when both are available
     if((kitty_graphics = iresp->kitty_graphics) == 0){
       ti->color_registers = iresp->color_registers;
+      ti->sixel_maxy_pristine = iresp->sixely;
       ti->sixel_maxy = iresp->sixely;
       ti->sixel_maxx = iresp->sixelx;
     }
@@ -1075,7 +1146,12 @@ int interrogate_terminfo(tinfo* ti, FILE* out, unsigned utf8,
 
 err:
   if(ti->ttyfd >= 0){
-    tty_emit(KKEYBOARD_POP, ti->ttyfd);
+    // if we haven't yet received a reply confirming lack of kitty keyboard
+    // support, it'll be UINT_MAX, and we ought try to pop (in case we died
+    // following the keyboard set, but before confirming support).
+    if(ti->kbdlevel){
+      tty_emit(KKEYBOARD_POP, ti->ttyfd);
+    }
     tty_emit(RMCUP, ti->ttyfd);
   }
   if(ti->tpreserved){
@@ -1111,21 +1187,11 @@ char* termdesc_longterm(const tinfo* ti){
 
 // send a u7 request, and wait until we have a cursor report. if input's ttyfd
 // is valid, we can just camp there. otherwise, we need dance with potential
-// user input looking at infd.
-int locate_cursor(tinfo* ti, int* cursor_y, int* cursor_x){
-#ifdef __MINGW64__
-  if(ti->outhandle){
-    CONSOLE_SCREEN_BUFFER_INFO conbuf;
-    if(!GetConsoleScreenBufferInfo(ti->outhandle, &conbuf)){
-      logerror("couldn't get cursor info\n");
-      return -1;
-    }
-    *cursor_y = conbuf.dwCursorPosition.Y;
-    *cursor_x = conbuf.dwCursorPosition.X;
-    loginfo("got a report from y=%d x=%d\n", *cursor_y, *cursor_x);
-    return 0;
-  }
-#endif
+// user input looking at infd. note that we do not use Windows's
+// GetConsoleScreenBufferInfo() because it is unreliable for this purpose
+// when the viewing area is not aligned with the forward edge of the buffer,
+// and also due to negative interactions with ssh.
+int locate_cursor(tinfo* ti, unsigned* cursor_y, unsigned* cursor_x){
   const char* u7 = get_escape(ti, ESCAPE_U7);
   if(u7 == NULL){
     logwarn("No support in terminfo\n");
@@ -1183,14 +1249,11 @@ int cbreak_mode(tinfo* ti){
     return -1;
   }
 #else
-  // we don't yet have a way to take Cygwin/MSYS2 out of canonical mode. we'll
-  // hit this stanza in MSYS2; allow the GetConsoleMode() to fail for now. this
-  // means we'll need enter pressed after the query response, obviously an
-  // unacceptable state of affairs...FIXME
+  // we don't yet have a way to take Cygwin/MSYS2 out of canonical mode FIXME.
   DWORD mode;
   if(!GetConsoleMode(ti->inhandle, &mode)){
     logerror("error acquiring input mode\n");
-    return 0; // FIXME is it safe?
+    return -1;
   }
   mode &= ~(ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT);
   if(!SetConsoleMode(ti->inhandle, mode)){
