@@ -4,7 +4,6 @@
 #ifdef __MINGW64__
 // ti has been memset to all zeroes. windows configuration is static.
 int prepare_windows_terminal(tinfo* ti, size_t* tablelen, size_t* tableused){
-  // FIXME need to add ccc + initc
   const struct wtermdesc {
     escape_e esc;
     const char* tinfo;
@@ -16,6 +15,7 @@ int prepare_windows_terminal(tinfo* ti, size_t* tablelen, size_t* tableused){
     { ESCAPE_HPA,   "\x1b[%i%p1%dG", },
     { ESCAPE_SC,    "\x1b[s", },
     { ESCAPE_RC,    "\x1b[u", },
+    { ESCAPE_INITC, "\x1b]4;%p1%d;rgb:%p2%{255}%*%{1000}%/%2.2X/%p3%{255}%*%{1000}%/%2.2X/%p4%{255}%*%{1000}%/%2.2X\E\\", },
     { ESCAPE_CLEAR, "\x1b[2J", },
     { ESCAPE_SMCUP, "\x1b[?1049h", },
     { ESCAPE_RMCUP, "\x1b[?1049l", },
@@ -36,7 +36,7 @@ int prepare_windows_terminal(tinfo* ti, size_t* tablelen, size_t* tableused){
     { ESCAPE_RMUL,  "\x1b[24m", },
     { ESCAPE_SGR0,  "\x1b[0m", },
     { ESCAPE_MAX, NULL, }
-  }, *w; 
+  }, *w;
   for(w = wterms ; w->tinfo; ++w){
     if(grow_esc_table(ti, w->tinfo, w->esc, tablelen, tableused)){
       return -1;
@@ -79,6 +79,7 @@ int prepare_windows_terminal(tinfo* ti, size_t* tablelen, size_t* tableused){
   }
   // if we're a true Windows Terminal, SetConsoleMode() ought succeed.
   // otherwise, we're something else; go ahead and try.
+  // FIXME handle redirection to a file, where this fails
   if(!SetConsoleMode(ti->outhandle, ENABLE_PROCESSED_OUTPUT
                      | ENABLE_WRAP_AT_EOL_OUTPUT
                      | ENABLE_VIRTUAL_TERMINAL_PROCESSING
