@@ -14,7 +14,7 @@ int main(void){
   if(nc == NULL){
     return EXIT_FAILURE;
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   struct ncplane* n = notcurses_stddim_yx(nc, &dimy, &dimx);
   wchar_t wc = 0x4e00;
   ncplane_set_scrolling(n, true);
