@@ -7,9 +7,8 @@
 #include <iostream>
 #include <climits>
 #include <sys/stat.h>
-#include <filesystem>
 
-const char* datadir = NOTCURSES_SHARE;
+const char* datadir = notcurses_data_dir();
 // NCLOGLEVEL_INFO for initial testing framework creation. we then switch to
 // command line-specified loglevel, _SILENT if none specified.
 ncloglevel_e cliloglevel = NCLOGLEVEL_INFO;
@@ -133,6 +132,7 @@ auto lang_and_term() -> void {
   }else{
     std::cout << "Running with LANG=" << lang << std::endl;
   }
+#ifndef __MINGW64__
   const char* term = getenv("TERM");
   // ubuntu's buildd sets TERM=unknown, fuck it, handle this atrocity
   if(term == nullptr || strcmp(term, "unknown") == 0){
@@ -140,14 +140,15 @@ auto lang_and_term() -> void {
     exit(EXIT_SUCCESS);
   }
   std::cout << "Running with TERM=" << term << std::endl;
+#endif
   auto nc = testing_notcurses();
   if(!nc){
     std::cerr << "Couldn't create notcurses testing framework" << std::endl;
     exit(EXIT_FAILURE);
   }
-  int dimy, dimx;
+  unsigned dimy, dimx;
   notcurses_stddim_yx(nc, &dimy, &dimx);
-  std::cout << "Detected geometry: " << dimx << 'x' << dimy << std::endl;
+  std::cout << "Detected cell geometry: " << dimx << 'x' << dimy << std::endl;
   notcurses_stop(nc);
   if(dimx < 50 || dimy < 24){ // minimum assumed geometry
     std::cerr << "Terminal was too small for tests (minimum 50x24)" << std::endl;
