@@ -3,6 +3,9 @@
 #include <vector>
 #include <cmath>
 
+// only run through this many frames of video
+constexpr auto FRAMECOUNT = 100;
+
 TEST_CASE("Media") {
   auto nc_ = testing_notcurses();
   REQUIRE(nullptr != nc_);
@@ -52,7 +55,7 @@ TEST_CASE("Media") {
   }
 
   SUBCASE("LoadImageCreatePlane") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     auto ncv = ncvisual_from_file(find_data("changes.jpg").get());
     REQUIRE(ncv);
@@ -72,7 +75,7 @@ TEST_CASE("Media") {
   }
 
   SUBCASE("LoadImage") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     auto ncv = ncvisual_from_file(find_data("changes.jpg").get());
     REQUIRE(ncv);
@@ -88,18 +91,22 @@ TEST_CASE("Media") {
   }
 
   SUBCASE("InflateImage") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     auto ncv = ncvisual_from_file(find_data("changes.jpg").get());
     REQUIRE(ncv);
-    int odimy, odimx, ndimy, ndimx;
     struct ncvisual_options opts{};
     opts.n = ncp_;
-    CHECK(0 == ncvisual_blitter_geom(nc_, ncv, &opts, &odimy, &odimx, nullptr, nullptr, nullptr));
+    ncvgeom geom{};
+    CHECK(0 == ncvisual_geom(nc_, ncv, &opts, &geom));
+    int odimy = geom.pixy;
+    int odimx = geom.pixx;
     CHECK(ncvisual_blit(nc_, ncv, &opts));
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncvisual_resize_noninterpolative(ncv, ncv->pixy * 2, ncv->pixx * 2));
-    CHECK(0 == ncvisual_blitter_geom(nc_, ncv, &opts, &ndimy, &ndimx, nullptr, nullptr, nullptr));
+    CHECK(0 == ncvisual_geom(nc_, ncv, &opts, &geom));
+    int ndimy = geom.pixy;
+    int ndimx = geom.pixx;
     CHECK(ndimy == odimy * 2);
     CHECK(ndimx == odimx * 2);
     CHECK(ncvisual_blit(nc_, ncv, &opts));
@@ -108,7 +115,7 @@ TEST_CASE("Media") {
   }
 
   SUBCASE("PlaneDuplicate") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(ncp_, &dimy, &dimx);
     auto ncv = ncvisual_from_file(find_data("changes.jpg").get());
     REQUIRE(ncv);
@@ -119,7 +126,8 @@ TEST_CASE("Media") {
     void* needle = malloc(1);
     REQUIRE(nullptr != needle);
     struct ncplane* newn = ncplane_dup(ncp_, needle);
-    int ndimx, ndimy;
+    free(needle);
+    unsigned ndimx, ndimy;
     REQUIRE(nullptr != newn);
     ncvisual_destroy(ncv);
     ncplane_erase(ncp_);
@@ -132,12 +140,13 @@ TEST_CASE("Media") {
 
   SUBCASE("LoadVideoASCIIScale") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
       REQUIRE(ncv);
       // FIXME can't we use use ncvisual_stream() here?
-      for(;;){ // run at the highest speed we can
+      // run at the highest speed we can
+      for(int frame = 0 ; frame < FRAMECOUNT ; ++frame){
         int ret = ncvisual_decode(ncv);
         if(1 == ret){
           break;
@@ -156,11 +165,12 @@ TEST_CASE("Media") {
 
   SUBCASE("LoadVideoHalfScale") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
       REQUIRE(ncv);
-      for(;;){ // run at the highest speed we can
+      // run at the highest speed we can
+      for(int frame = 0 ; frame < FRAMECOUNT ; ++frame){
         int ret = ncvisual_decode(ncv);
         if(1 == ret){
           break;
@@ -180,11 +190,12 @@ TEST_CASE("Media") {
   // quadblitter is default for NCSCALE_SCALE_HIRES
   SUBCASE("LoadVideoQuadScale") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
       REQUIRE(ncv);
-      for(;;){ // run at the highest speed we can
+      // run at the highest speed we can
+      for(int frame = 0 ; frame < FRAMECOUNT ; ++frame){
         int ret = ncvisual_decode(ncv);
         if(1 == ret){
           break;
@@ -203,11 +214,11 @@ TEST_CASE("Media") {
 
   SUBCASE("LoadVideoSexScale") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
-      REQUIRE(ncv);
-      for(;;){ // run at the highest speed we can
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
+      // run at the highest speed we can
+      for(int frame = 0 ; frame < FRAMECOUNT ; ++frame){
         int ret = ncvisual_decode(ncv);
         if(1 == ret){
           break;
@@ -226,11 +237,11 @@ TEST_CASE("Media") {
 
   SUBCASE("LoadVideoBrailleScale") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
-      REQUIRE(ncv);
-      for(;;){ // run at the highest speed we can
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
+      // run at the highest speed we can
+      for(int frame = 0 ; frame < FRAMECOUNT ; ++frame){
         int ret = ncvisual_decode(ncv);
         if(1 == ret){
           break;
@@ -247,15 +258,20 @@ TEST_CASE("Media") {
     }
   }
 
-  // do a pixel video, reusing the same plane over and over
-  SUBCASE("LoadVideoPixelScaleOnePlane") {
+  // do a pixel video, reusing the same pile over and over
+  SUBCASE("LoadVideoPixelStretchOnePile") {
     if(notcurses_check_pixel_support(nc_) > 0){
       if(notcurses_canopen_videos(nc_)){
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncp_, &dimy, &dimx);
-        auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+        auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
         REQUIRE(ncv);
-        struct ncplane* n = NULL;
+        ncplane_options nopts{};
+        // shrink it down so this test runs more quickly
+        nopts.rows = 5;
+        nopts.cols = 20;
+        auto n = ncpile_create(nc_, &nopts);
+        REQUIRE(nullptr != n);
         for(;;){ // run at the highest speed we can
           int ret = ncvisual_decode(ncv);
           if(1 == ret){
@@ -263,26 +279,28 @@ TEST_CASE("Media") {
           }
           CHECK(0 == ret);
           struct ncvisual_options opts{};
-          opts.scaling = NCSCALE_SCALE;
+          opts.scaling = NCSCALE_STRETCH;
           opts.blitter = NCBLIT_PIXEL;
           opts.n = n;
           n = ncvisual_blit(nc_, ncv, &opts);
           REQUIRE(nullptr != n);
-          CHECK(0 == notcurses_render(nc_));
+          CHECK(0 == ncpile_render(n));
+          CHECK(0 == ncpile_rasterize(n));
         }
-        CHECK(0 == ncplane_destroy(n));
         ncvisual_destroy(ncv);
+        CHECK(0 == ncplane_destroy(n));
+        CHECK(0 == notcurses_render(nc_));
       }
     }
   }
 
-  // do a pixel video with a different plane for each frame
-  SUBCASE("LoadVideoPixelScaleDifferentPlanes") {
+  // do a pixel video with a different pile for each frame
+  SUBCASE("LoadVideoPixelStretchDifferentPiles") {
     if(notcurses_check_pixel_support(nc_) > 0){
       if(notcurses_canopen_videos(nc_)){
-        int dimy, dimx;
+        unsigned dimy, dimx;
         ncplane_dim_yx(ncp_, &dimy, &dimx);
-        auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+        auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
         REQUIRE(ncv);
         for(;;){ // run at the highest speed we can
           int ret = ncvisual_decode(ncv);
@@ -290,22 +308,31 @@ TEST_CASE("Media") {
             break;
           }
           CHECK(0 == ret);
+          ncplane_options nopts{};
+          // shrink it down so this test runs more quickly
+          nopts.rows = 5;
+          nopts.cols = 20;
+          auto n = ncpile_create(nc_, &nopts);
           struct ncvisual_options opts{};
-          opts.scaling = NCSCALE_SCALE;
+          opts.scaling = NCSCALE_STRETCH;
           opts.blitter = NCBLIT_PIXEL;
-          REQUIRE(ncvisual_blit(nc_, ncv, &opts));
-          CHECK(0 == notcurses_render(nc_));
+          opts.n = n;
+          CHECK(n == ncvisual_blit(nc_, ncv, &opts));
+          CHECK(0 == ncpile_render(n));
+          CHECK(0 == ncpile_rasterize(n));
+          CHECK(0 == ncplane_destroy(n));
         }
         ncvisual_destroy(ncv);
+        CHECK(0 == notcurses_render(nc_));
       }
     }
   }
 
   SUBCASE("LoopVideo") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+      auto ncv = ncvisual_from_file(find_data("notcursesIII.mov").get());
       REQUIRE(ncv);
       int ret;
       while((ret = ncvisual_decode(ncv)) == 0){
@@ -330,9 +357,9 @@ TEST_CASE("Media") {
 
   SUBCASE("LoadVideoCreatePlane") {
     if(notcurses_canopen_videos(nc_)){
-      int dimy, dimx;
+      unsigned dimy, dimx;
       ncplane_dim_yx(ncp_, &dimy, &dimx);
-      auto ncv = ncvisual_from_file(find_data("notcursesIII.mkv").get());
+      auto ncv = ncvisual_from_file(find_data("fm6.mov").get());
       REQUIRE(ncv);
       CHECK(0 == ncvisual_decode(ncv));
       /*CHECK(dimy * 2 == frame->height);
@@ -347,7 +374,7 @@ TEST_CASE("Media") {
     }
   }
 
-  // test NCVISUAL_OPTIONS_CHILDPLANE + stretch + null alignment, using a file
+  // test NCVISUAL_OPTION_CHILDPLANE + stretch + null alignment, using a file
   SUBCASE("ImageFileChildScaling") {
     struct ncplane_options opts = {
       .y = 0, .x = 0,
