@@ -2,7 +2,7 @@
 #include "main.h"
 
 void BoxPermutationsRounded(struct notcurses* nc, struct ncplane* n, unsigned edges) {
-  int dimx, dimy;
+  unsigned dimx, dimy;
   ncplane_dim_yx(n, &dimy, &dimx);
   REQUIRE(2 < dimy);
   REQUIRE(47 < dimx);
@@ -30,7 +30,7 @@ TEST_CASE("Plane") {
 
   // Starting position ought be 0, 0 (the origin)
   SUBCASE("StdPlanePosition") {
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == x);
     CHECK(0 == y);
@@ -38,9 +38,9 @@ TEST_CASE("Plane") {
 
   // Dimensions of the standard plane ought be the same as those of the context
   SUBCASE("StdPlaneDimensions") {
-    int cols, rows;
+    unsigned cols, rows;
     notcurses_term_dim_yx(nc_, &rows, &cols);
-    int ncols, nrows;
+    unsigned ncols, nrows;
     ncplane_dim_yx(n_, &nrows, &ncols);
     CHECK(rows == nrows);
     CHECK(cols == ncols);
@@ -48,10 +48,10 @@ TEST_CASE("Plane") {
 
   // Verify that we can move to all four coordinates of the standard plane
   SUBCASE("MoveStdPlaneDimensions") {
-    int cols, rows;
+    unsigned cols, rows;
     notcurses_term_dim_yx(nc_, &rows, &cols);
     CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(y == 0);
     CHECK(x == 0);
@@ -71,7 +71,7 @@ TEST_CASE("Plane") {
 
   // Verify that we can move to all four coordinates of the standard plane
   SUBCASE("MoveBeyondPlaneFails") {
-    int cols, rows;
+    unsigned cols, rows;
     notcurses_term_dim_yx(nc_, &rows, &cols);
     CHECK(0 > ncplane_cursor_move_yx(n_, -2, 0));
     CHECK(0 > ncplane_cursor_move_yx(n_, -2, -2));
@@ -102,20 +102,35 @@ TEST_CASE("Plane") {
     CHECK(0 > ncplane_set_fg_rgb8(n_, 256, 256, 256));
   }
 
+  SUBCASE("StandardPlaneChild") {
+    struct ncplane_options nopts{};
+    nopts.rows = ncplane_dim_y(n_);
+    nopts.cols = ncplane_dim_x(n_);
+    auto n = ncplane_create(n_, &nopts);
+    REQUIRE(nullptr != n);
+    CHECK(ncplane_dim_y(n) == ncplane_dim_y(n_));
+    CHECK(ncplane_dim_x(n) == ncplane_dim_x(n_));
+    CHECK(0 == ncplane_destroy(n));
+  }
+
   // Verify we can emit a NUL character, and it advances the cursor after
   // wiping out whatever we printed it atop.
   SUBCASE("EmitNULCell") {
-    nccell c = CELL_CHAR_INITIALIZER('a');
+    nccell c = NCCELL_CHAR_INITIALIZER('a');
     CHECK(0 < ncplane_putc_yx(n_, 0, 0, &c));
-    CHECK(0 == strcmp("a", ncplane_at_yx(n_, 0, 0, nullptr, nullptr)));
-    int y, x;
+    auto egc = ncplane_at_yx(n_, 0, 0, nullptr, nullptr);
+    CHECK(0 == strcmp("a", egc));
+    free(egc);
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(1 == x);
     CHECK(0 == notcurses_render(nc_));
-    c = CELL_TRIVIAL_INITIALIZER;
+    c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 < ncplane_putc_yx(n_, 0, 0, &c));
-    CHECK(0 == strcmp("", ncplane_at_yx(n_, 0, 0, nullptr, nullptr)));
+    egc = ncplane_at_yx(n_, 0, 0, nullptr, nullptr);
+    CHECK(0 == strcmp("", egc));
+    free(egc);
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(1 == x);
@@ -128,7 +143,7 @@ TEST_CASE("Plane") {
     nccell c{};
     CHECK(strlen(cchar) == nccell_load(n_, &c, cchar));
     CHECK(0 < ncplane_putc(n_, &c));
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(1 == x);
@@ -138,9 +153,8 @@ TEST_CASE("Plane") {
   // Verify we can emit a wchar_t, and it advances the cursor
   SUBCASE("EmitWcharT") {
     const wchar_t* w = L"✔";
-    int sbytes = 0;
-    CHECK(0 < ncplane_putwegc(n_, w, &sbytes));
-    int x, y;
+    CHECK(0 < ncplane_putwegc(n_, w, nullptr));
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == y);
     CHECK(1 == x);
@@ -155,9 +169,9 @@ TEST_CASE("Plane") {
     for(const char** s = ss ; *s ; ++s){
       CHECK(0 == ncplane_cursor_move_yx(n_, s - ss, 0));
       int wrote = ncplane_putstr(n_, *s);
-      CHECK(ncstrwidth(*s) == wrote);
+      CHECK(ncstrwidth(*s, NULL, NULL) == wrote);
     }
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(2 == y);
     CHECK(10 <= x);
@@ -174,7 +188,7 @@ TEST_CASE("Plane") {
       int wrote = ncplane_putwstr(n_, *s);
       CHECK(0 < wrote);
     }
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(2 == y);
     CHECK(10 <= x);
@@ -190,7 +204,7 @@ TEST_CASE("Plane") {
     CHECK(!ncplane_set_scrolling(n_, true));
     int wrote = ncplane_putwstr(n_, e);
     CHECK(0 < wrote);
-    int x, y;
+    unsigned x, y;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK_LE(0, y);
     CHECK(1 <= x); // FIXME tighten in on this
@@ -198,16 +212,16 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("HorizontalLines") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(0 < y);
     REQUIRE(0 < x);
     nccell c{};
     nccell_load(n_, &c, "-");
-    for(int yidx = 0 ; yidx < y ; ++yidx){
+    for(unsigned yidx = 0 ; yidx < y ; ++yidx){
       CHECK(0 == ncplane_cursor_move_yx(n_, yidx, 1));
       CHECK(x - 2 == ncplane_hline(n_, &c, x - 2));
-      int posx, posy;
+      unsigned posx, posy;
       ncplane_cursor_yx(n_, &posy, &posx);
       CHECK(yidx == posy);
       CHECK(x - 1 == posx);
@@ -217,16 +231,16 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("VerticalLines") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(0 < y);
     REQUIRE(0 < x);
     nccell c{};
     nccell_load(n_, &c, "|");
-    for(int xidx = 1 ; xidx < x - 1 ; ++xidx){
+    for(unsigned xidx = 1 ; xidx < x - 1 ; ++xidx){
       CHECK(0 == ncplane_cursor_move_yx(n_, 1, xidx));
       CHECK(y - 2 == ncplane_vline(n_, &c, y - 2));
-      int posx, posy;
+      unsigned posx, posy;
       ncplane_cursor_yx(n_, &posy, &posx);
       CHECK(y - 2 == posy);
       CHECK(xidx == posx - 1);
@@ -237,7 +251,7 @@ TEST_CASE("Plane") {
 
   // reject attempts to draw boxes beyond the boundaries of the ncplane
   SUBCASE("BadlyPlacedBoxen") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(2 < y);
     REQUIRE(2 < x);
@@ -274,7 +288,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("BoxPermutationsDouble") {
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
     REQUIRE(2 < dimx);
     REQUIRE(47 < dimx);
@@ -289,7 +303,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("PerimeterRoundedBox") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(2 < y);
     REQUIRE(2 < x);
@@ -299,7 +313,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("PerimeterRoundedBoxSized") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(2 < y);
     REQUIRE(2 < x);
@@ -309,7 +323,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("PerimeterDoubleBox") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(2 < y);
     REQUIRE(2 < x);
@@ -319,7 +333,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("PerimeterDoubleBoxSized") {
-    int x, y;
+    unsigned x, y;
     ncplane_dim_yx(n_, &y, &x);
     REQUIRE(2 < y);
     REQUIRE(2 < x);
@@ -339,9 +353,9 @@ TEST_CASE("Plane") {
     const char* w1 = "à"; // U+00E0, U+0000         (c3 a0)
     const char* w2 = "à"; // U+0061, U+0300, U+0000 (61 cc 80)
     const char* w3 = "a"; // U+0061, U+0000         (61)
-    nccell cell1 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell2 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell3 = CELL_TRIVIAL_INITIALIZER;
+    nccell cell1 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell2 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell3 = NCCELL_TRIVIAL_INITIALIZER;
     auto u1 = nccell_load(n_, &cell1, w1);
     auto u2 = nccell_load(n_, &cell2, w2);
     auto u3 = nccell_load(n_, &cell3, w3);
@@ -357,18 +371,18 @@ TEST_CASE("Plane") {
     const char* w1 = "à"; // U+00E0, U+0000         (c3 a0)
     const char* w2 = "à"; // U+0061, U+0300, U+0000 (61 cc 80)
     const char* w3 = "a"; // U+0061, U+0000         (61)
-    nccell cell1 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell2 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell3 = CELL_TRIVIAL_INITIALIZER;
+    nccell cell1 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell2 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell3 = NCCELL_TRIVIAL_INITIALIZER;
     auto u1 = nccell_load(n_, &cell1, w1);
     auto u2 = nccell_load(n_, &cell2, w2);
     auto u3 = nccell_load(n_, &cell3, w3);
     REQUIRE(2 == u1);
     REQUIRE(3 == u2);
     REQUIRE(1 == u3);
-    nccell cell4 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell5 = CELL_TRIVIAL_INITIALIZER;
-    nccell cell6 = CELL_TRIVIAL_INITIALIZER;
+    nccell cell4 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell5 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell cell6 = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == nccell_duplicate(n_, &cell4, &cell1));
     CHECK(0 == nccell_duplicate(n_, &cell5, &cell2));
     CHECK(0 == nccell_duplicate(n_, &cell6, &cell3));
@@ -383,15 +397,15 @@ TEST_CASE("Plane") {
   SUBCASE("CellMultiColumn") {
     const char* w1 = "\xf0\x9f\x91\xa9"; // U+1F469 WOMAN
     const char* w2 = "N";
-    nccell c1 = CELL_TRIVIAL_INITIALIZER;
-    nccell c2 = CELL_TRIVIAL_INITIALIZER;
+    nccell c1 = NCCELL_TRIVIAL_INITIALIZER;
+    nccell c2 = NCCELL_TRIVIAL_INITIALIZER;
     auto u1 = nccell_load(n_, &c1, w1);
     auto u2 = nccell_load(n_, &c2, w2);
     REQUIRE(0 < u1);
     REQUIRE(0 < u2);
     REQUIRE(strlen(w1) == u1);
     REQUIRE(strlen(w2) == u2);
-    CHECK(ncstrwidth(w1) == 1 + nccell_double_wide_p(&c1));
+    CHECK(ncstrwidth(w1, NULL, NULL) == 1 + nccell_double_wide_p(&c1));
     CHECK_FALSE(nccell_double_wide_p(&c2));
     nccell_release(n_, &c1);
     nccell_release(n_, &c2);
@@ -401,7 +415,7 @@ TEST_CASE("Plane") {
   // for the standard plane, and that we can change it.
   SUBCASE("UserPtr") {
     CHECK(nullptr == ncplane_userptr(n_));
-    int x, y;
+    unsigned x, y;
     void* sentinel = &x;
     notcurses_term_dim_yx(nc_, &y, &x);
     struct ncplane_options nopts = {
@@ -427,7 +441,7 @@ TEST_CASE("Plane") {
   // create a new plane, the same size as the terminal, and verify that it
   // occupies the same dimensions as the standard plane.
   SUBCASE("NewPlaneSameSize") {
-    int x, y;
+    unsigned x, y;
     notcurses_term_dim_yx(nc_, &y, &x);
     struct ncplane_options nopts = {
       .y = 0,
@@ -439,11 +453,11 @@ TEST_CASE("Plane") {
     };
     struct ncplane* ncp = ncplane_create(n_, &nopts);
     REQUIRE(ncp);
-    int px, py;
+    unsigned px, py;
     ncplane_dim_yx(ncp, &py, &px);
     CHECK(y == py);
     CHECK(x == px);
-    int sx, sy;
+    unsigned sx, sy;
     ncplane_dim_yx(n_, &sy, &sx);
     CHECK(sy == py);
     CHECK(sx == px);
@@ -453,7 +467,7 @@ TEST_CASE("Plane") {
   // create a new plane, the same size as the terminal, and verify that it
   // occupies the same dimensions as the standard plane, but on another pile.
   SUBCASE("NewPileSameSize") {
-    int x, y;
+    unsigned x, y;
     notcurses_term_dim_yx(nc_, &y, &x);
     struct ncplane_options nopts = {
       .y = 0,
@@ -465,11 +479,11 @@ TEST_CASE("Plane") {
     };
     struct ncplane* ncp = ncpile_create(nc_, &nopts);
     REQUIRE(ncp);
-    int px, py;
+    unsigned px, py;
     ncplane_dim_yx(ncp, &py, &px);
     CHECK(y == py);
     CHECK(x == px);
-    int sx, sy;
+    unsigned sx, sy;
     ncplane_dim_yx(n_, &sy, &sx);
     CHECK(sy == py);
     CHECK(sx == px);
@@ -484,7 +498,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("ShrinkPlane") {
-    int maxx, maxy;
+    unsigned maxx, maxy;
     int x = 0, y = 0;
     notcurses_term_dim_yx(nc_, &maxy, &maxx);
     struct ncplane_options nopts = {
@@ -528,9 +542,9 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("GrowPlane") {
-    int maxx = 2, maxy = 2;
+    unsigned maxx = 2, maxy = 2;
     int x = 0, y = 0;
-    int dimy, dimx;
+    unsigned dimy, dimx;
     notcurses_term_dim_yx(nc_, &dimy, &dimx);
     x = dimx / 2 - 1;
     y = dimy / 2 - 1;
@@ -552,7 +566,7 @@ TEST_CASE("Plane") {
       REQUIRE(0 == ncplane_resize(newp, 1, 1, maxy - 3, maxx - 3, 1, 1, maxy, maxx));
       // FIXME check dims, pos
     }
-    while(y < dimy){
+    while(y < static_cast<int>(dimy)){
       ++maxy;
       if(y){
         ++y;
@@ -560,7 +574,7 @@ TEST_CASE("Plane") {
       REQUIRE(0 == ncplane_resize(newp, 1, 0, maxy - 2, maxx, 1, 0, maxy, maxx));
       // FIXME check dims, pos
     }
-    while(x < dimx){
+    while(x < static_cast<int>(dimx)){
       ++maxx;
       if(x){
         ++x;
@@ -582,17 +596,17 @@ TEST_CASE("Plane") {
     ncplane_set_styles(n_, NCSCALE_NONE);
     REQUIRE(0 == ncplane_cursor_move_yx(n_, 0, 0));
     REQUIRE(0 < ncplane_putstr(n_, STR1));
-    nccell testcell = CELL_TRIVIAL_INITIALIZER;
+    nccell testcell = NCCELL_TRIVIAL_INITIALIZER;
     REQUIRE(0 == ncplane_at_cursor_cell(n_, &testcell)); // want nothing at the cursor
     CHECK(0 == testcell.gcluster);
     CHECK(0 == testcell.stylemask);
     CHECK(0 == testcell.channels);
     nccell_release(n_, &testcell);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
     REQUIRE(0 == ncplane_cursor_move_yx(n_, 1, dimx - strlen(STR2)));
     REQUIRE(0 < ncplane_putstr(n_, STR2));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     REQUIRE(1 == y);
     REQUIRE(dimx == x);
@@ -620,17 +634,17 @@ TEST_CASE("Plane") {
     ncplane_set_styles(n_, NCSTYLE_NONE);
     REQUIRE(0 == ncplane_cursor_move_yx(n_, 0, 0));
     REQUIRE(0 < ncplane_putstr(n_, STR1));
-    nccell testcell = CELL_TRIVIAL_INITIALIZER;
+    nccell testcell = NCCELL_TRIVIAL_INITIALIZER;
     ncplane_at_cursor_cell(n_, &testcell); // should be nothing at the cursor
     CHECK(0 == testcell.gcluster);
     CHECK(0 == testcell.stylemask);
     CHECK(0 == testcell.channels);
     nccell_release(n_, &testcell);
-    int dimy, dimx;
+    unsigned dimy, dimx;
     ncplane_dim_yx(n_, &dimy, &dimx);
     REQUIRE(0 == ncplane_cursor_move_yx(n_, 1, dimx - mbstowcs(nullptr, STR2, 0)));
     REQUIRE(0 < ncplane_putstr(n_, STR2));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     REQUIRE(1 == y);
     REQUIRE(dimx == x);
@@ -657,7 +671,7 @@ TEST_CASE("Plane") {
     const char STR3[] = "da chronic lives";
     ncplane_set_styles(n_, NCSTYLE_BOLD);
     REQUIRE(0 < ncplane_putstr_yx(n_, 0, 0, STR1));
-    int y, x;
+    unsigned y, x;
     ncplane_cursor_yx(n_, &y, &x);
     CHECK(0 == ncplane_cursor_move_yx(n_, y + 1, x - strlen(STR2)));
     ncplane_on_styles(n_, NCSTYLE_ITALIC);
@@ -667,10 +681,10 @@ TEST_CASE("Plane") {
     REQUIRE(0 < ncplane_putstr(n_, STR3));
     ncplane_off_styles(n_, NCSTYLE_ITALIC);
     CHECK(0 == notcurses_render(nc_));
-    int newx;
+    unsigned newx;
     ncplane_cursor_yx(n_, &y, &newx);
     CHECK(newx == x);
-    nccell testcell = CELL_TRIVIAL_INITIALIZER;
+    nccell testcell = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == ncplane_cursor_move_yx(n_, y - 2, x - 1));
     REQUIRE(1 == ncplane_at_cursor_cell(n_, &testcell));
     CHECK(testcell.gcluster == htole(STR1[strlen(STR1) - 1]));
@@ -684,7 +698,7 @@ TEST_CASE("Plane") {
 
   SUBCASE("BoxGradients") {
     const auto sidesz = 5;
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
     REQUIRE(20 < dimy);
     REQUIRE(40 < dimx);
@@ -722,7 +736,7 @@ TEST_CASE("Plane") {
 
   SUBCASE("BoxSideColors") {
     const auto sidesz = 5;
-    int dimx, dimy;
+    unsigned dimx, dimy;
     ncplane_dim_yx(n_, &dimy, &dimx);
     REQUIRE(20 < dimy);
     REQUIRE(40 < dimx);
@@ -761,8 +775,7 @@ TEST_CASE("Plane") {
   SUBCASE("RightToLeft") {
     // give us some room on both sides
     CHECK(0 == ncplane_cursor_move_yx(n_, 1, 5));
-    int sbytes = -1;
-    CHECK(0 < ncplane_putegc(n_, "־", &sbytes));
+    CHECK(0 < ncplane_putegc(n_, "־", nullptr));
     CHECK(0 == notcurses_render(nc_));
     CHECK(0 == ncplane_cursor_move_yx(n_, 3, 5));
     CHECK(0 < ncplane_putstr(n_, "I write English + מילים בעברית together."));
@@ -778,14 +791,14 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("NewPlaneOnRight") {
-    int ncols, nrows;
+    unsigned ncols, nrows;
     ncplane_dim_yx(n_, &nrows, &ncols);
     nccell ul{}, ll{}, lr{}, ur{}, hl{}, vl{};
     int y, x;
     ncplane_yx(n_, &y, &x);
     struct ncplane_options nopts = {
       .y = y,
-      .x = ncols - 3,
+      .x = static_cast<int>(ncols) - 3,
       .rows = 2,
       .cols = 2,
       .userptr = nullptr, .name = nullptr, .resizecb = nullptr, .flags = 0,
@@ -801,7 +814,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("MoveToLowerRight") {
-    int ncols, nrows;
+    unsigned ncols, nrows;
     ncplane_dim_yx(n_, &nrows, &ncols);
     nccell ul{}, ll{}, lr{}, ur{}, hl{}, vl{};
     int y, x;
@@ -826,24 +839,23 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("Perimeter") {
-    nccell c = CELL_CHAR_INITIALIZER('X');
+    nccell c = NCCELL_CHAR_INITIALIZER('X');
     CHECK(0 == ncplane_perimeter(n_, &c, &c, &c, &c, &c, &c, 0));
     CHECK(0 == notcurses_render(nc_));
   }
 
   SUBCASE("EGCStained") {
-    nccell c = CELL_TRIVIAL_INITIALIZER;
-    int sbytes;
+    nccell c = NCCELL_TRIVIAL_INITIALIZER;
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x444444));
-    CHECK(1 == ncplane_putegc(n_, "A", &sbytes));
+    CHECK(1 == ncplane_putegc(n_, "A", nullptr));
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x888888));
-    CHECK(1 == ncplane_putegc(n_, "B", &sbytes));
+    CHECK(1 == ncplane_putegc(n_, "B", nullptr));
     CHECK(0 == ncplane_cursor_move_yx(n_, 0, 0));
     CHECK(0 == notcurses_render(nc_));
     // EGC should change, but not the color
     CHECK(0 == ncplane_set_fg_rgb(n_, 0x222222));
-    CHECK(1 == ncplane_putegc_stained(n_, "C", &sbytes));
-    CHECK(1 == ncplane_putegc_stained(n_, "D", &sbytes));
+    CHECK(1 == ncplane_putegc_stained(n_, "C", nullptr));
+    CHECK(1 == ncplane_putegc_stained(n_, "D", nullptr));
     uint64_t channels = 0;
     CHECK(1 == ncplane_at_yx_cell(n_, 0, 0, &c));
     CHECK(cell_simple_p(&c));
@@ -859,7 +871,7 @@ TEST_CASE("Plane") {
   }
 
   SUBCASE("MouseEvent") {
-    int dimy, dimx;
+    unsigned dimy, dimx;
     notcurses_stddim_yx(nc_, &dimy, &dimx);
     struct ncplane_options nopts = {
       .y = 1,
@@ -1051,22 +1063,22 @@ TEST_CASE("Plane") {
     CHECK(x == 2);
     CHECK(1 == ncplane_set_base(n, " ", 0, channels));
     CHECK(0 == notcurses_render(nc_));
-    CHECK(0 == ncplane_moverel(n, -1, 0)); // up
+    CHECK(0 == ncplane_move_rel(n, -1, 0)); // up
     ncplane_yx(n, &y, &x);
     CHECK(y == 1);
     CHECK(x == 2);
     CHECK(0 == notcurses_render(nc_));
-    CHECK(0 == ncplane_moverel(n, 2, 0)); // down
+    CHECK(0 == ncplane_move_rel(n, 2, 0)); // down
     ncplane_yx(n, &y, &x);
     CHECK(y == 3);
     CHECK(x == 2);
     CHECK(0 == notcurses_render(nc_));
-    CHECK(0 == ncplane_moverel(n, 0, -1)); // left
+    CHECK(0 == ncplane_move_rel(n, 0, -1)); // left
     ncplane_yx(n, &y, &x);
     CHECK(y == 3);
     CHECK(x == 1);
     CHECK(0 == notcurses_render(nc_));
-    CHECK(0 == ncplane_moverel(n, 0, 2)); // right
+    CHECK(0 == ncplane_move_rel(n, 0, 2)); // right
     ncplane_yx(n, &y, &x);
     CHECK(y == 3);
     CHECK(x == 3);
@@ -1084,17 +1096,17 @@ TEST_CASE("Plane") {
     REQUIRE(nullptr != n);
     CHECK(false == ncplane_set_scrolling(n, true));
     uint64_t channels = NCCHANNELS_INITIALIZER(0, 0xff, 0, 0xff, 0, 0xff);
-    int y, x;
+    unsigned y, x;
     CHECK(1 == ncplane_set_base(n, " ", 0, channels));
     CHECK(0 == notcurses_render(nc_));
-    for(int i = 0 ; i < ncplane_dim_y(n) ; ++i){
+    for(unsigned i = 0 ; i < ncplane_dim_y(n) ; ++i){
       ncplane_cursor_yx(n, &y, &x);
       CHECK(i == y);
       CHECK(0 == x);
       CHECK(0 < ncplane_putstr(n, "here's a line\n"));
       CHECK(0 == notcurses_render(nc_));
     }
-    for(int i = 0 ; i < ncplane_dim_y(n) ; ++i){
+    for(unsigned i = 0 ; i < ncplane_dim_y(n) ; ++i){
       ncplane_cursor_yx(n, &y, &x);
       CHECK(ncplane_dim_y(n) - 1 == y);
       CHECK(0 == x);
